@@ -6,9 +6,12 @@
  *
  * Authors: Chris.Hessing@utah.edu, Terry.Simons@utah.edu
  *
- * $Id: xsup_monitor.c,v 1.14 2005/08/09 01:39:12 chessing Exp $
- * $Date: 2005/08/09 01:39:12 $
+ * $Id: xsup_monitor.c,v 1.15 2005/10/16 04:31:29 chessing Exp $
+ * $Date: 2005/10/16 04:31:29 $
  * $Log: xsup_monitor.c,v $
+ * Revision 1.15  2005/10/16 04:31:29  chessing
+ * Changed gui_interface.c/h to be a library called libxsupgui, and moved it to a new lib directory.
+ *
  * Revision 1.14  2005/08/09 01:39:12  chessing
  * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
@@ -23,7 +26,7 @@
 #include <sys/socket.h>
 #include <net/if.h>
 
-#include "../../common/gui_interface.h"
+#include "../../../lib/libxsupgui/xsupgui.h"
 
 int connected;
 char activeint[16];
@@ -39,7 +42,7 @@ void block_while_not_xsup(char *intname)
 {
   int disp=0;
 
-  while (is_xsup_running(intname) == FALSE)
+  while (xsupgui_is_xsup_running(intname) == FALSE)
     {
       if (disp == 0)
 	{
@@ -49,7 +52,7 @@ void block_while_not_xsup(char *intname)
 
       if (connected == 1)
 	{
-	  gui_interface_disconnect(skfd);
+	  xsupgui_disconnect(skfd);
 	  connected = 0;
 	}
 
@@ -74,9 +77,9 @@ void do_password(char *eaptype, char *challenge)
       return;
     }
 
-  gui_interface_set_password(if_nametoindex(activeint), (char *)&buffer, &outptr, passwd);
+  xsupgui_set_password(if_nametoindex(activeint), (char *)&buffer, &outptr, passwd);
 
-  switch (gui_interface_send_packet(skfd, (char *)&buffer, outptr))
+  switch (xsupgui_send_packet(skfd, (char *)&buffer, outptr))
     {
     case ERR_PKT_BAD:
       fprintf(stderr, "Error! Packet appears to be bad!\n");
@@ -94,7 +97,7 @@ void do_password(char *eaptype, char *challenge)
     {
 
       bufptr = 1520;
-      if (gui_interface_get_packet(skfd, (char *)&buffer, &bufptr, TRUE) == ERR_SOCK)
+      if (xsupgui_get_packet(skfd, (char *)&buffer, &bufptr, TRUE) == ERR_SOCK)
 	{
 	  done = TRUE;
 	  continue;
@@ -102,7 +105,7 @@ void do_password(char *eaptype, char *challenge)
 
       bufptr = 0;
 
-      i = gui_interface_parse_packet((char *)&buffer, &bufptr, (char *)&result,
+      i = xsupgui_parse_packet((char *)&buffer, &bufptr, (char *)&result,
 				     &dlen, (char *)&activeint);
 
       if (i != PASSWORD)
@@ -199,7 +202,7 @@ int main(int argc, char *argv[])
       // If we have not connected before, or if we need to reconnect.
       if (connected != 1)
 	{
-	  skfd = gui_interface_connect(argv[1]);
+	  skfd = xsupgui_connect(argv[1]);
 	  if (skfd < 1)
 	    {
 	      fprintf(stderr, "Error! Couldn't get a handle to the daemon! "
@@ -212,7 +215,7 @@ int main(int argc, char *argv[])
       bzero(&buffer, 1520);
       bufptr = 1520;
 
-      gui_interface_get_packet(skfd, (char *)&buffer, (int *)&bufptr, TRUE);
+      xsupgui_get_packet(skfd, (char *)&buffer, (int *)&bufptr, TRUE);
 
       outptr = 0;
       curptr = 0;
@@ -220,7 +223,7 @@ int main(int argc, char *argv[])
       while ((bufptr > 0) && (curptr < bufptr))
 	{
 	  // Process the message we have been given.
-	  i=gui_interface_parse_packet((char *)&buffer, &curptr, 
+	  i=xsupgui_parse_packet((char *)&buffer, &curptr, 
 				       (char *)&result, &dlen, 
 				       (char *)&activeint);
 
