@@ -6,9 +6,12 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: xsup_set_pwd.c,v 1.12 2005/08/09 01:39:12 chessing Exp $
- * $Date: 2005/08/09 01:39:12 $
+ * $Id: xsup_set_pwd.c,v 1.13 2005/10/16 04:31:29 chessing Exp $
+ * $Date: 2005/10/16 04:31:29 $
  * $Log: xsup_set_pwd.c,v $
+ * Revision 1.13  2005/10/16 04:31:29  chessing
+ * Changed gui_interface.c/h to be a library called libxsupgui, and moved it to a new lib directory.
+ *
  * Revision 1.12  2005/08/09 01:39:12  chessing
  * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
@@ -23,7 +26,7 @@
 #include <net/if.h>
 
 
-#include "../../common/gui_interface.h"
+#include "../../../lib/libxsupgui/xsupgui.h"
 
 #define XSSP_DEBUG 1
 
@@ -79,14 +82,14 @@ int main(int argc, char *argv[])
       exit(1);
     }
 
-  if (is_xsup_running(intname) == FALSE)
+  if (xsupgui_is_xsup_running(intname) == FALSE)
     {
       fprintf(stderr, "Error!  XSupplicant must be running to use this " 
               "tool!\n");
       exit(1);
     }
 
-  skfd = gui_interface_connect(intname);
+  skfd = xsupgui_connect(intname);
   if (skfd < 1)
     {
       fprintf(stderr, "Error!  Couldn't get a handle to the daemon!\n");
@@ -100,7 +103,7 @@ int main(int argc, char *argv[])
   if (passwd == NULL)
     {
       fprintf(stderr, "No password set!\n");
-      gui_interface_disconnect(skfd);
+      xsupgui_disconnect(skfd);
       exit(1);
     }
 
@@ -108,25 +111,25 @@ int main(int argc, char *argv[])
   dump_packet("Sending", (char *)&buffer, bufptr);
 #endif
 
-  gui_interface_set_password(if_nametoindex(intname), (char *)&buffer, 
+  xsupgui_set_password(if_nametoindex(intname), (char *)&buffer, 
 			     &bufptr, passwd);
 
-  switch (gui_interface_send_packet(skfd, (char *)&buffer, bufptr))
+  switch (xsupgui_send_packet(skfd, (char *)&buffer, bufptr))
     {
     case ERR_PKT_BAD:
       fprintf(stderr, "Error! Packet appears to be bad!\n");
-      gui_interface_disconnect(skfd);
+      xsupgui_disconnect(skfd);
       exit(1);
       break;
     case ERR_SOCK:
       fprintf(stderr, "Error! There was a socket error!\n");
-      gui_interface_disconnect(skfd);
+      xsupgui_disconnect(skfd);
       exit(1);
       break;
     }
 
   bzero((char *)&buffer, 1520);
-  gui_interface_get_packet(skfd, (char *)&buffer, &bufptr, TRUE);
+  xsupgui_get_packet(skfd, (char *)&buffer, &bufptr, TRUE);
 
 #if XSSP_DEBUG
   dump_packet("Got", (char *)&buffer, bufptr);
@@ -134,7 +137,7 @@ int main(int argc, char *argv[])
 
   bufptr = 2;
 
-  if (gui_interface_parse_packet((char *)&buffer, &bufptr, (char *)&result, 
+  if (xsupgui_parse_packet((char *)&buffer, &bufptr, (char *)&result, 
 				 &dlen, (char *)&myint) == ERR_PKT_BAD)
     {
       fprintf(stderr, "The daemon returned an invalid packet!\n");
@@ -147,6 +150,6 @@ int main(int argc, char *argv[])
       fprintf(stderr, "Error!  Password not set!\n");
     }
 
-  gui_interface_disconnect(skfd);
+  xsupgui_disconnect(skfd);
   return 0;
 }
