@@ -21,18 +21,13 @@
 #define WPA_CCMP 3
 
 // Reconsider if these are needed.
-#define MASTER    0x01   // Is this an AP in master mode?
-#define WEP       0x02
+#define ENC       0x02
 #define WPA_IE    0x04
 #define RSN_IE    0x08
 
 #define DRIVER_NONE        0
 #define DRIVER_WEXT        1
-#define DRIVER_HOSTAP      2
 #define DRIVER_ATMEL       3
-#define DRIVER_PRISM54     4
-#define DRIVER_NDISWRAPPER 5
-#define DRIVER_IPW         6
 #define DRIVER_MADWIFI     7
 
 // Reasons for a disassociation.
@@ -51,15 +46,23 @@
 #define DISASSOC_DOT1X_FAILURE        23    // 802.1X authentication failure.
 #define DISASSOC_CIPHER_REJECT        24    // Cipher suite rejected.
 
-// The length of time to wait before we assume an assocation failed. (Defined
-// by 802.11i section 8.1.3 as dot11RSNAConfigSA-Timeout.)  Default is 60, but
-// we may want to provide the ability to change this in the future.
-#define ASSOCIATION_TIMEOUT           60    // in seconds.
+// Return values for associated, and unassociated.
+#define IS_ASSOCIATED                 1
+#define IS_UNASSOCIATED               2
+
+// Different encryption/authentication modes that are supported.  (Since
+// any interface should be able to do 802.1X, we don't flag that here.)
+#define DOES_WPA                      0x00000001
+#define DOES_WPA2                     0x00000002
+#define DOES_WEP40                    0x00000004
+#define DOES_WEP104                   0x00000008
+#define DOES_TKIP                     0x00000010
+#define DOES_CCMP                     0x00000020
 
 struct cardif_funcs {
 
   // Issue a scan request.
-  int (*scan)(struct interface_data *);
+  int (*scan)(struct interface_data *, char);
 
   // Send a disassociate message to the AP
   int (*disassociate)(struct interface_data *, int);
@@ -78,9 +81,6 @@ struct cardif_funcs {
   // Delete a key.
   int (*delete_key)(struct interface_data *, int, int);
 
-  // Tell the card to join a specific SSID.
-  int (*set_ssid)(struct interface_data *, char *);
-
   // Tell the card to associate to a specific SSID.
   void (*associate)(struct interface_data *, char *);
 
@@ -96,8 +96,8 @@ struct cardif_funcs {
   // Enable WPA in the card.
   int (*wpa)(struct interface_data *, char);
 
-  // Roam to a new AP.
-  int (*roam)(struct interface_data *, int);
+  // Set WEP keys needed to connect to a new AP.
+  int (*wep_associate)(struct interface_data *, int);
 
   // Tell the driver to enable countermeasures.
   int (*countermeasures)(struct interface_data *, char);
@@ -113,6 +113,12 @@ struct cardif_funcs {
 
   // Disable encryption.
   int (*enc_disable)(struct interface_data *);
+
+  // Get the encryption capabilities of the card. (WEP, WPA, WPA2, TKIP, CCMP)
+  void (*enc_capabilities)(struct interface_data *);
+
+  // Change the bssid that we are connected to.
+  int (*setbssid)(struct interface_data *, u_char *);
 };
 
 // Stuff needed by both wired, and wireless interfaces.
@@ -135,7 +141,7 @@ void cardif_set_search_ssid(char *);
 // should either return XENOTWIRELSS, or just return (in the case of a void)
 
 int cardif_enable_wpa(struct interface_data *);
-int cardif_do_wireless_scan(struct interface_data *);
+int cardif_do_wireless_scan(struct interface_data *, char);
 int cardif_set_wep_key(struct interface_data *, u_char *, int, int);
 int cardif_set_tkip_key(struct interface_data *, char *, int, int, char *, int,
 			char *, int);
@@ -148,10 +154,9 @@ int cardif_GetSSID(struct interface_data *, char *);
 int cardif_check_ssid(struct interface_data *);
 int cardif_GetBSSID(struct interface_data *, char *);
 int cardif_get_int(int, char *);
-int cardif_is_associated(struct interface_data *);
 int cardif_int_is_wireless(char *);
 int cardif_reset_card(struct interface_data *);
-int cardif_roam(struct interface_data *);
+int cardif_wep_associate(struct interface_data *, int);
 int cardif_disable_wpa_state(struct interface_data *);
 int cardif_enable_wpa_state(struct interface_data *);
 int cardif_drop_unencrypted(struct interface_data *, char);
@@ -159,9 +164,12 @@ int cardif_countermeasures(struct interface_data *, char);
 int cardif_get_wpa_ie(struct interface_data *, char *, int *);
 int cardif_get_wpa2_ie(struct interface_data *, char *, int *);
 int cardif_clear_keys(struct interface_data *);
-void cardif_check_associated(struct interface_data *);
+int cardif_check_associated(struct interface_data *);
 void cardif_reassociate(struct interface_data *, u_char);
 void cardif_association_timeout_expired(struct interface_data *);
 int cardif_enc_disable(struct interface_data *);
+void cardif_get_abilities(struct interface_data *);
+void cardif_wait_for_int(char *);
+void cardif_passive_scan_timeout(struct interface_data *);
 
 #endif
