@@ -12,9 +12,9 @@
 #ifndef _CORE_H_
 #define _CORE_H_
 
-void event_core_setup();
-void event_core(struct interface_data *);
-void event_core_cleanup();
+void ext_event_init();
+void ext_event_core(struct interface_data *, struct wireless_state *);
+void ext_event_deinit();
 
 #endif
 
