@@ -9,9 +9,21 @@
  * on the work of Ivan Voras <ivoras@fer.hr> and the Linux version by
  * Chris.Hessing@utah.edu.
  *
- * $Id: cardif_freebsd.c,v 1.2 2005/08/09 01:39:14 chessing Exp $
- * $Date: 2005/08/09 01:39:14 $
+ * $Id: cardif_freebsd.c,v 1.6 2006/01/19 05:37:04 chessing Exp $
+ * $Date: 2006/01/19 05:37:04 $
  * $Log: cardif_freebsd.c,v $
+ * Revision 1.6  2006/01/19 05:37:04  chessing
+ * WPA2 is working correctly.  Added the ability to query the card to gather encryption/authentication capabilities.  1.2.3 is now ready to go.
+ *
+ * Revision 1.5  2005/09/05 01:00:36  chessing
+ * Major overhaul to most of the state machines in Xsupplicant.  Also added additional error messages to the TLS functions to try to debug the one of the problems reported on the list.  Basic testing shows this new code to be more stable than previous code, but it needs more testing.
+ *
+ * Revision 1.4  2005/08/25 03:34:05  chessing
+ * Removed a bunch of functions from config.c that could be handled better in other ways.
+ *
+ * Revision 1.3  2005/08/25 02:20:20  chessing
+ * Some cleanup in xsup_debug.c, added the ability to wait for an interface to come up if it is down when Xsupplicant is started.  Roughed in the ability to choose between having Xsupplicant auto associate you, or allow you to set the ssid manually.  The stale key timer can now be set in the config file.  The association timeout can be set in the config file, and will also be used once the functionality is in place to attempt to guess the key settings needed for association, as well as the functionality to auto associate.
+ *
  * Revision 1.2  2005/08/09 01:39:14  chessing
  * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
@@ -193,9 +205,10 @@ struct cardif_funcs *wireless;
  * Determine if we are currently associated. 
  *
  ***********************************************/
-void cardif_check_associated(struct interface_data *intdata)
+int cardif_check_associated(struct interface_data *intdata)
 {
   debug_printf(DEBUG_EXCESSIVE, "%s not implemented.\n", __FUNCTION__);
+  return -1;
 }
 
 /***********************************************
@@ -227,6 +240,22 @@ int cardif_init(struct interface_data *thisint, char driver, int block_wpa)
 {
   struct ifreq ifr;
   struct fbsd_sock_data *sockData;
+  struct config_globals *globals;
+
+  if (thisint == NULL)
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid interface data in %s!\n",
+		   __FUNCTION__);
+      return XEGENERROR;
+    }
+  
+  globals = config_get_globals();
+
+  if (globals == NULL)
+    {
+      debug_printf(DEBUG_NORMAL, "No valid configuration globals available!\n");
+      return XEGENERROR;
+    }
 
   debug_printf(DEBUG_INT, "Initializing bpf for interface %s..\n",
 	       thisint->intName);
@@ -261,7 +290,7 @@ int cardif_init(struct interface_data *thisint, char driver, int block_wpa)
     }        
 
   // Check if we want ALLMULTI mode, and enable it.
-  if (config_get_allmulti() == 1)
+  if (TEST_FLAG(globals->flags, CONFIG_GLOBALS_ALLMULTI)
     {
       // Tell the ifreq struct which interface we want to use.
       strncpy((char *)&ifr.ifr_name, thisint->intName, sizeof(ifr.ifr_name));
@@ -366,7 +395,7 @@ int cardif_deinit(struct interface_data *thisint)
   sockData= (struct fbsd_sock_data *) thisint->sockData;
 
   // Check if we want ALLMULTI mode, and enable it.
-  if (config_get_allmulti() == 1)
+  if (TEST_FLAG(thisint->flags, ALLMULTI))
     {
       // Tell the ifreq struct which interface we want to use.
       strncpy((char *)&ifr.ifr_name, thisint->intName, sizeof(ifr.ifr_name));
@@ -454,25 +483,6 @@ int cardif_delete_key(struct interface_data *intdata, int key_idx, int set_tx)
   return -1;
 }
 
-/******************************************
- *
- * Ask the wireless card for the ESSID that we are currently connected to.  If
- * this is not a wireless card, or the information is not available, we should
- * return an error.
- *
- ******************************************/
-int cardif_SetSSID(struct interface_data *thisint, char *ssid_name)
-{
-  if (wireless == NULL) 
-    {
-      debug_printf(DEBUG_NORMAL, "No valid call to set SSID on this driver!"
-		   "\n");
-      return -1;
-    }
-  debug_printf(DEBUG_EXCESSIVE, "%s not implemented.\n", __FUNCTION__);
-  return -1;
-}
-
 /******************************************
  *
  * If our association timer expires, we need to attempt to associate again.
@@ -484,22 +494,6 @@ void cardif_association_timeout_expired(struct interface_data *intdata)
   cardif_associate(intdata, intdata->cur_essid);
 }
 
-/******************************************
- *
- * Set the association timeout.
- *
- ******************************************/
-void cardif_set_association_timeout()
-{
-  if (timer_check_existing(ASSOCIATION_TIMER))
-    {
-      timer_reset_timer_count(ASSOCIATION_TIMER, ASSOCIATION_TIMEOUT);
-    } else {
-      timer_add_timer(ASSOCIATION_TIMER, ASSOCIATION_TIMEOUT, NULL,
-		      &cardif_association_timeout_expired);
-    }
-}
-
 /******************************************
  *
  * Do whatever we need to do in order to associate based on the flags in
@@ -515,9 +509,6 @@ void cardif_associate(struct interface_data *intdata, char *newssid)
       return;
     }
 
-  // Set up our association timeout.
-  cardif_set_association_timeout();
-
   debug_printf(DEBUG_EXCESSIVE, "%s not implemented.\n", __FUNCTION__);
   return;
 }
@@ -987,4 +978,14 @@ void cardif_reassociate(struct interface_data *intiface, u_char reason)
   debug_printf(DEBUG_EXCESSIVE, "%s not implemented.\n", __FUNCTION__);
 }
 
+void cardif_try_associate(struct interface_data *intiface)
+{
+  debug_printf(DEBUG_EXCESSIVE, "%s not implemented!\n", __FUNCTION__);
+}
+
+void cardif_get_abilitites(struct interface_data *intdata)
+{
+  intdata->enc_capa = 0;
+}
+
 #endif
