@@ -7,9 +7,75 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: cardif_linux_wext.c,v 1.37 2005/08/20 19:06:54 chessing Exp $
- * $Date: 2005/08/20 19:06:54 $
+ * $Id: cardif_linux_wext.c,v 1.58 2006/02/23 22:26:53 chessing Exp $
+ * $Date: 2006/02/23 22:26:53 $
  * $Log: cardif_linux_wext.c,v $
+ * Revision 1.58  2006/02/23 22:26:53  chessing
+ * Fix for bug id #1415020.  'Building Xsupplicant 1.2.3 Fails on FC4'.
+ *
+ * Revision 1.57  2006/01/24 04:42:26  chessing
+ * A few more fixes to WPA code, along with a fix to the scan reaping code.
+ *
+ * Revision 1.56  2006/01/23 05:28:37  chessing
+ * Fixed a few settings that were causing errors with IOCTLs on some cards.  Updated WPA2 code to properly process group key packets. We now record quality, signal level, and noise level from scan results, so that we can start to make better decisions on which AP to associate to.
+ *
+ * Revision 1.55  2006/01/19 05:37:04  chessing
+ * WPA2 is working correctly.  Added the ability to query the card to gather encryption/authentication capabilities.  1.2.3 is now ready to go.
+ *
+ * Revision 1.54  2006/01/14 06:22:10  chessing
+ * Fixed WPA functionality for cards that don't use IWAUTH to set up the WPA IE.
+ *
+ * Revision 1.53  2006/01/12 17:54:03  chessing
+ * WPA almost works again on cards that use the GENIE setting.  Added a small fix to allow Xsupplicant to build on Open SuSE 10.
+ *
+ * Revision 1.52  2005/12/25 04:54:11  chessing
+ * Fixup to code that added wext support needed for using ndiswrapper.
+ *
+ * Revision 1.51  2005/12/24 04:51:51  chessing
+ * Removed an unneeded file.  Updated scanning code to parse IWEVGENIE events correctly, and make use of them.
+ *
+ * Revision 1.50  2005/12/24 02:39:48  chessing
+ * Fixes to autoconf script to correctly identify the number of arguments that iw_extract_event_stream() takes, along with checking iwlib.h compiles correctly.
+ *
+ * Revision 1.49  2005/12/18 07:44:12  chessing
+ * Added the ability to associate using IWAUTH options instead of setting a full IE.  This allows NDISwrapper to work with vanilla wireless extensions.  For some reason, we can't parse IWEVASSOCREQIE/IWEVASSOCRESPIE since we see the length as 0, even though iwevent sees the correct IE information. :-/  Need to figure out why.
+ *
+ * Revision 1.48  2005/12/03 22:18:51  chessing
+ * Added an include to fix some problems when compiling with WE19.
+ *
+ * Revision 1.47  2005/11/14 17:27:39  chessing
+ * Removed the driver specific code for hostap, ipw, and ndiswrapper, since they all support WE18 now.
+ *
+ * Revision 1.46  2005/11/10 04:56:54  chessing
+ * Added patch from Ben Gardner to add support for setting a specific WEP key prior to attempting to associte.  (With a few slight modifications by me to make it fit in the current CVS code, and get it supported in config-parse.)  Added patch from Pekka Savola to fix some header ordering issues, and a potential buffer overflow.
+ *
+ * Revision 1.45  2005/10/17 03:56:54  chessing
+ * Updates to the libxsupconfig library.  It no longer relies on other source from the main tree, so it can be used safely in other code with problems.
+ *
+ * Revision 1.44  2005/10/14 02:26:18  shaftoe
+ * - cleanup gcc 4 warnings
+ * - (re)add support for a pid in the form of /var/run/xsupplicant.<iface>.pid
+ *
+ * -- Eric Evans <eevans@sym-link.com>
+ *
+ * Revision 1.43  2005/10/13 18:46:47  chessing
+ * Fixed to the Madwifi driver to allow it to do dynamic WEP again.  Fixed up the wext driver to behave correctly again. ;)
+ *
+ * Revision 1.42  2005/09/19 21:45:46  chessing
+ * Fix for the madwifi driver when it connects to certain APs to do WEP.  (Currently the only known one with this problem is the Trapeze APs when they are running MSS 4.x+.)
+ *
+ * Revision 1.41  2005/09/17 18:38:17  chessing
+ * Some updates to make the madwifi driver work with dynamic WEP again.
+ *
+ * Revision 1.40  2005/09/14 02:50:44  chessing
+ * Major updates.  Auto association now works.  Started to rewrite the rtnetlink pieces using iwlib from the wireless tools to avoid compatibility issues.  As a result, getting the WPA and RSN IEs via the IWEVGENIE event no longer works for some reason, but the old style using IWEVCUSTOM events should still work like a champ.
+ *
+ * Revision 1.39  2005/09/08 16:27:02  chessing
+ * Some small updates to the new state machine code.  First attempt at an auto association mode.  (It mostly works. ;)
+ *
+ * Revision 1.38  2005/09/05 01:00:37  chessing
+ * Major overhaul to most of the state machines in Xsupplicant.  Also added additional error messages to the TLS functions to try to debug the one of the problems reported on the list.  Basic testing shows this new code to be more stable than previous code, but it needs more testing.
+ *
  * Revision 1.37  2005/08/20 19:06:54  chessing
  * Patch from Carsten Grohmann to fix a few things in xsup_get_state.c.  Also added the ability to define an empty network clause, that will set the card in to encryption disabled mode.  From there, anything short of changing the SSID will be ignored by Xsupplicant.
  *
@@ -21,6 +87,9 @@
 
 #ifdef LINUX_FRAMER
 
+// Use kernel headers.
+#define HEADERS_KERNEL
+
 #include <string.h>
 #include <strings.h>
 #include <errno.h>
@@ -30,22 +99,24 @@
 #include <netinet/in.h>
 #include <linux/types.h>
 #include <linux/socket.h>
-#include <linux/if.h>
+#include <linux/compiler.h>
+#include <linux/wireless.h>
+#include <iwlib.h>
 #include <linux/if_packet.h>
 #include <linux/netlink.h>
-#include <linux/wireless.h>
-#include <linux/if_arp.h>
 
-#include "config.h"
-#include "config_ssid.h"
 #include "profile.h"
+#include "xsupconfig.h"
+#include "config_ssid.h"
 #include "xsup_debug.h"
 #include "xsup_err.h"
 #include "wpa.h"
 #include "wpa2.h"
+#include "wpa_common.h"
 #include "cardif/cardif.h"
 #include "cardif/linux/cardif_linux.h"
 #include "cardif/linux/cardif_linux_wext.h"
+#include "wireless_sm.h"
 #include "cardif/linux/cardif_linux_rtnetlink.h"
 
 // Old versions of wireless.h may not have this defined.
@@ -58,12 +129,15 @@
  * Tell the wireless card to start scanning for wireless networks.
  *
  **************************************************************/
-int cardif_linux_wext_scan(struct interface_data *thisint)
+int cardif_linux_wext_scan(struct interface_data *thisint, char passive)
 {
   struct lin_sock_data *sockData;
   struct iwreq iwr;
+#if WIRELESS_EXT > 17
+  struct iw_scan_req iwsr;
+#endif
 
-  if (!(thisint->flags & IS_WIRELESS))
+  if (!TEST_FLAG(thisint->flags, IS_WIRELESS))
     {
       debug_printf(DEBUG_INT, "%s is not a wireless interface!\n", 
 		   thisint->intName);
@@ -75,14 +149,45 @@ int cardif_linux_wext_scan(struct interface_data *thisint)
   if (sockData->sockInt <= 0)
     return XENOSOCK;
 
-  debug_printf(DEBUG_INT, "Issuing scan request for interface %s!\n",
-	       thisint->intName);
+  cardif_linux_wext_wpa_state(thisint, 1);
 
-  // XXX Probably want to update this to support extended scanning.
+  debug_printf(DEBUG_INT, "Issuing %s scan request for interface %s!\n",
+	       passive ? "passive":"active", thisint->intName);
+
+#if WIRELESS_EXT > 17  
+  // Build our extended scan structure.
+  bzero(&iwsr, sizeof(iwsr));
+
+  if (passive)
+    {
+      iwsr.scan_type = IW_SCAN_TYPE_PASSIVE;
+      //      iwsr.scan_type = IW_SCAN_TYPE_ACTIVE;
+
+      // If we are doing a passive scan, then we only care about other APs
+      // that are on this SSID.  Otherwise, we might end up picking an SSID
+      // later that isn't in the same layer2/3 space.
+      //      iwr.u.data.flags = IW_SCAN_THIS_ESSID | IW_SCAN_ALL_FREQ | 
+      //	IW_SCAN_THIS_MODE | IW_SCAN_ALL_RATE;
+
+      //      iwr.u.data.flags = IW_SCAN_DEFAULT;
+      iwr.u.data.flags = IW_SCAN_THIS_ESSID;
+    }
+  else
+    {
+      iwsr.scan_type = IW_SCAN_TYPE_ACTIVE;
+      iwr.u.data.flags = IW_SCAN_DEFAULT;
+    }
+
+  // We aren't looking for a specific BSSID.
+  memset(iwsr.bssid.sa_data, 0xff, 6);
+
+  iwr.u.data.length = sizeof(iwsr);
+  iwr.u.data.pointer = (caddr_t) &iwsr;
+
+#else
   iwr.u.data.flags = IW_SCAN_DEFAULT;
-  iwr.u.data.length = 0;
-  iwr.u.data.pointer = NULL;
-
+#endif
+  
   strcpy((char *)&iwr.ifr_name, thisint->intName);
 
   if (ioctl(sockData->sockInt, SIOCSIWSCAN, &iwr) < 0)
@@ -93,12 +198,7 @@ int cardif_linux_wext_scan(struct interface_data *thisint)
       return -1;
     }
 
-  while ((cardif_linux_rtnetlink_check_event(thisint) != XDATA) &&
-	 (thisint->flags & SCANNING))
-    {
-      debug_printf(DEBUG_INT, "Waiting for SSID information...\n");
-      usleep(250000);
-    }
+  SET_FLAG(thisint->flags, SCANNING);
 
   return XENONE;
 }
@@ -119,10 +219,10 @@ void cardif_linux_wext_set_zero_keys(struct interface_data *thisint)
 
   // We set the key index to 0x80, to force key 0 to be set to all 0s,
   // and to have key 0 be set as the default transmit key.
-  cardif_set_wep_key(thisint, (char *)&zerokey, keylen, 0x80);
-  cardif_set_wep_key(thisint, (char *)&zerokey, keylen, 0x01);
-  cardif_set_wep_key(thisint, (char *)&zerokey, keylen, 0x02);
-  cardif_set_wep_key(thisint, (char *)&zerokey, keylen, 0x03);
+  cardif_set_wep_key(thisint, (u_char *)&zerokey, keylen, 0x80);
+  cardif_set_wep_key(thisint, (u_char *)&zerokey, keylen, 0x01);
+  cardif_set_wep_key(thisint, (u_char *)&zerokey, keylen, 0x02);
+  cardif_set_wep_key(thisint, (u_char *)&zerokey, keylen, 0x03);
 }
 
 /**************************************************************
@@ -221,11 +321,12 @@ int cardif_linux_wext_get_wpa2_ie(struct interface_data *thisint,
 				  char *iedata, int *ielen)
 {
 #if WIRELESS_EXT > 17
+
   // Should we use capabilities here?
   wpa2_gen_ie(thisint, iedata, ielen);
 
   debug_printf(DEBUG_INT, "Setting WPA2 IE : ");
-  debug_hex_printf(DEBUG_INT, iedata, *ielen);
+  debug_hex_printf(DEBUG_INT, (u_char *)iedata, *ielen);
   debug_printf(DEBUG_INT, "\n");
 #else
   debug_printf(DEBUG_NORMAL, "WPA2 isn't implemented in this version of the "
@@ -243,12 +344,13 @@ int cardif_linux_wext_get_wpa_ie(struct interface_data *thisint,
 				 char *iedata, int *ielen)
 {
 #if WIRELESS_EXT > 17
+
   // Should we use capabilities here?
   wpa_gen_ie(thisint, iedata);
   *ielen = 24;
 
   debug_printf(DEBUG_INT, "Setting WPA IE : ");
-  debug_hex_printf(DEBUG_INT, iedata, *ielen);
+  debug_hex_printf(DEBUG_INT, (u_char *)iedata, *ielen);
   debug_printf(DEBUG_INT, "\n");
 #else
   debug_printf(DEBUG_NORMAL, "WPA isn't implemented in this version of the "
@@ -477,6 +579,8 @@ int cardif_linux_wext_set_ssid(struct interface_data *thisint, char *ssid_name)
 
   if (ioctl(sockData->sockInt, SIOCSIWESSID, &iwr) < 0) return XENOWIRELESS;
 
+  debug_printf(DEBUG_INT, "Requested SSID be set to '%s'\n", newssid);
+
   thisint->flags &= (~WAS_DOWN);
 
   return XENONE;
@@ -511,13 +615,14 @@ int cardif_linux_wext_set_bssid(struct interface_data *intdata, u_char *bssid)
 
   strncpy((char *)&wrq.ifr_name, intdata->intName, IFNAMSIZ);
   memcpy(&wrq.u.ap_addr.sa_data, bssid, 6);
+  wrq.u.ap_addr.sa_family = ARPHRD_ETHER;
 
   if (ioctl(sockData->sockInt, SIOCSIWAP, &wrq) < 0)
     {
       // If we couldn't set the BSSID, it isn't the end of the world.  The
       // driver just may not need it.
-      //      debug_printf(DEBUG_NORMAL, "Error setting BSSID!  We may not associate/"
-      //		   "authenticate correctly!\n");
+      debug_printf(DEBUG_NORMAL, "Error setting BSSID!  We may not associate/"
+      		   "authenticate correctly!\n");
       return XESOCKOP;
     }
 
@@ -599,17 +704,13 @@ int cardif_linux_wext_get_ssid(struct interface_data *thisint, char *ssid_name)
 
   sockData = thisint->sockData;
  
-  if (!(thisint->flags & IS_WIRELESS))
+  if (!TEST_FLAG(thisint->flags, IS_WIRELESS))
     {
       // We want to verify that the interface is in fact, not wireless, and
       // not that we are in a situation where the interface has just been 
       // down.
-      if (!(thisint->flags & WAS_DOWN))
-	{
-	  debug_printf(DEBUG_NORMAL, "Interface appears to be down!  Cannot "
-		       "determine the SSID!\n");
-	  return XENOWIRELESS;
-	}
+      debug_printf(DEBUG_NORMAL, "This interface isn't wireless!\n");
+      return XENOWIRELESS;
     } 
 
   // If we get here, and isWireless == FALSE, then we need to double
@@ -657,18 +758,43 @@ int cardif_linux_wext_get_ssid(struct interface_data *thisint, char *ssid_name)
  *  reset the card to a state where it can associate with a new AP.
  *
  ***********************************************************************/
-int cardif_linux_wext_roam(struct interface_data *intdata, int zero_keys)
+int cardif_linux_wext_wep_associate(struct interface_data *intdata, 
+				    int zero_keys)
 {
+  char *bssid;
+  struct config_globals *globals;
+
+  cardif_linux_wext_wpa_state(intdata, 0);
+
+  cardif_linux_wext_set_ssid(intdata, intdata->cur_essid);
+
   if (zero_keys == 0)
     {
-      debug_printf(DEBUG_INT, "ROAMING: turning encryption off.\n");
+      debug_printf(DEBUG_INT, "WEP: turning encryption off.\n");
       return cardif_linux_wext_enc_disable(intdata);
     } else if (zero_keys == 1)
       {
-      debug_printf(DEBUG_INT, "ROAMING: zero keys.\n");
+      debug_printf(DEBUG_INT, "WEP: zero keys.\n");
 	cardif_linux_wext_zero_keys(intdata);
 	return XENONE;
-      }
+      } else
+	{
+	  debug_printf(DEBUG_NORMAL, "Invalid association value.\n");
+	}
+
+  bssid = (char *) config_ssid_get_mac();
+  if (bssid != NULL)
+    {
+      debug_printf(DEBUG_INT, "Dest. BSSID : ");
+      debug_hex_printf(DEBUG_INT, (u_char *) bssid, 6);
+    }
+
+  globals = config_get_globals();
+
+  if ((!globals) || (!TEST_FLAG(globals->flags, CONFIG_GLOBALS_FIRMWARE_ROAM)))
+    {
+      cardif_linux_wext_set_bssid(intdata, bssid);
+    }
 
   return XENOTHING_TO_DO;
 }
@@ -761,12 +887,15 @@ int cardif_linux_wext_set_key_ext(struct interface_data *intdata, int alg,
     {
       iwee->ext_flags |= IW_ENCODE_EXT_RX_SEQ_VALID;
       memcpy(iwee->rx_seq, seq, seqlen);
+      debug_printf(DEBUG_INT, "SEQ (%d) : ", seqlen);
+      debug_hex_printf(DEBUG_INT, seq, seqlen);
     }
   
   if (settx) 
     {
       iwee->ext_flags |= IW_ENCODE_EXT_SET_TX_KEY;
-      if ((addr != NULL) && (memcmp(addr, "\0xff\0xff\0xff\0xff\0xff\0xff", 6) != 0))
+      if ((addr != NULL) && 
+	  (memcmp(addr, "\0xff\0xff\0xff\0xff\0xff\0xff", 6) != 0))
 	{
 	  memcpy(iwee->addr.sa_data, addr, 6);
 	} else {
@@ -781,8 +910,15 @@ int cardif_linux_wext_set_key_ext(struct interface_data *intdata, int alg,
   iwee->key_len = keylen;
   memcpy(iwee->key, key, keylen);
 
-  wrq.u.data.pointer = (caddr_t)iwee;
-  wrq.u.data.length = sizeof(struct iw_encode_ext) + keylen;
+  if (key)
+    {
+      debug_printf(DEBUG_INT, "Setting key : ");
+      debug_hex_printf(DEBUG_INT, iwee->key, keylen);
+    }
+
+  wrq.u.encoding.pointer = (caddr_t)iwee;
+  wrq.u.encoding.flags = (keyidx + 1);
+  wrq.u.encoding.length = sizeof(struct iw_encode_ext) + keylen;
 
   if (ioctl(sockData->sockInt, SIOCSIWENCODEEXT, &wrq) < 0)
     {
@@ -804,7 +940,6 @@ int cardif_linux_wext_set_tkip_key(struct interface_data *intdata,
 				   int keylen)
 {
 #if WIRELESS_EXT > 17
-
   return cardif_linux_wext_set_key_ext(intdata, IW_ENCODE_ALG_TKIP, addr,
 				       keyidx, settx, seq, seqlen, key, 
 				       keylen);
@@ -893,14 +1028,12 @@ int cardif_linux_wext_wpa_state(struct interface_data *intdata, char state)
     {
       // Enable WPA if the interface doesn't already have it.
       cardif_linux_wext_wpa(intdata, TRUE);
-
-
     } else {
       cardif_linux_wext_wpa(intdata, FALSE);
 
       // Otherwise, make sure we don't have an IE set.
       memset(wpaie, 0, sizeof(wpaie));
-      if (cardif_linux_wext_set_wpa_ie(intdata, wpaie, 0) < 0)
+      if (cardif_linux_wext_set_wpa_ie(intdata, (unsigned char *)wpaie, 0) < 0)
 	{
 	  debug_printf(DEBUG_NORMAL, "Couldn't clear WPA IE on device %s!\n",
 		       intdata->intName);
@@ -939,40 +1072,223 @@ int cardif_linux_wext_unencrypted_eapol(struct interface_data *intdata,
   return XENONE;
 }
 
+/*************************************************************************
+ *
+ * Set values for IWAUTH.
+ *
+ *************************************************************************/
+int cardif_linux_wext_set_iwauth(struct interface_data *intdata,
+				 int setting, uint32_t value, 
+				 char *setting_name)
+{
+#if WIRELESS_EXT > 17
+  struct iwreq wrq;
+  struct lin_sock_data *sockData;
+
+  sockData = intdata->sockData;
+
+  bzero(&wrq, sizeof(wrq));
+  strncpy((char *)&wrq.ifr_name, intdata->intName, IFNAMSIZ);
+
+  wrq.u.param.flags = setting & IW_AUTH_INDEX;
+  wrq.u.param.value = value;
+
+  if (ioctl(sockData->sockInt, SIOCSIWAUTH, &wrq) < 0)
+    {
+      if (errno != ENOTSUP)
+	{
+	  debug_printf(DEBUG_NORMAL, "Error changing setting for '%s'! "
+		       "It is possible that your driver does not support the "
+		       "needed functionality.\n", setting_name);
+	  return -1;
+	}
+    }
+#endif
+  return XENONE;
+}
+
+/***********************************************************************
+ *
+ * Convert our cipher designator to something that will be understood by
+ * the linux wireless extensions.
+ *
+ ***********************************************************************/
+int cardif_linux_wext_iw_cipher(int cipher)
+{
+#if WIRELESS_EXT > 17
+  switch (cipher)
+    {
+    case CIPHER_NONE:
+      return 0;
+      break;
+
+    case CIPHER_WEP40:
+      return IW_AUTH_CIPHER_WEP40;
+      break;
+
+    case CIPHER_TKIP:
+      return IW_AUTH_CIPHER_TKIP;
+      break;
+
+    case CIPHER_WRAP:
+      debug_printf(DEBUG_NORMAL, "WRAP is not supported!\n");
+      return -1;
+      break;
+
+    case CIPHER_CCMP:
+      return IW_AUTH_CIPHER_CCMP;
+      break;
+      
+    case CIPHER_WEP104:
+      return IW_AUTH_CIPHER_WEP104;
+      break;
+
+    default:
+      debug_printf(DEBUG_NORMAL, "Unknown cipher value of %d!\n", cipher);
+      return -1;
+      break;
+    }
+#else
+  return -1;
+#endif
+}
+
+/*********************************************************************
+ *
+ * Set all of the card settings that are needed in order to complete an
+ * association, so that we can begin the authentication.
+ *
+ *********************************************************************/
 void cardif_linux_wext_associate(struct interface_data *intdata, char *newssid)
 {
   char *bssid;
-  int len;
+#if WIRELESS_EXT > 17
+  int len, akm = 0;
+  uint32_t cipher;
   u_char wpaie[255];
+#endif
+  struct config_network *network_data;
+  struct config_globals *globals;
+
+  if (!intdata)
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid interface data passed to %s!\n",
+		   __FUNCTION__);
+      return;
+    }
+
+  network_data = config_get_network_config();
+  
+  if (!network_data)
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid network data in %s():%d\n",
+		   __FUNCTION__, __LINE__);
+      return;
+    }
 
   if (config_ssid_get_ssid_abilities() & RSN_IE)
     {
-      cardif_linux_wext_get_wpa2_ie(intdata, wpaie, &len);
+#if WIRELESS_EXT > 17
+      cardif_linux_wext_get_wpa2_ie(intdata, (char *) wpaie, &len);
+      if (cardif_linux_wext_set_iwauth(intdata, IW_AUTH_WPA_VERSION,
+				       IW_AUTH_WPA_VERSION_WPA2,
+				       "WPA version") < 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "Couldn't set WPA version!\n");
+	}
+#endif
     } else if (config_ssid_get_ssid_abilities() & WPA_IE)
       {
-	cardif_linux_wext_get_wpa_ie(intdata, wpaie, &len);
+#if WIRELESS_EXT > 17
+	cardif_linux_wext_get_wpa_ie(intdata, (char *) wpaie, &len);
+	if (cardif_linux_wext_set_iwauth(intdata, IW_AUTH_WPA_VERSION,
+					 IW_AUTH_WPA_VERSION_WPA,
+					 "WPA2 version") < 0)
+	  {
+	    debug_printf(DEBUG_NORMAL, "Couldn't set WPA2 version!\n");
+	  }
+#endif
       }
 
+#if WIRELESS_EXT > 17
+  // If the length of the WPA/WPA2 IE is > 0.
   if (len > 0)
     {
+      // Turn off countermeasures, if they were running.
+      if (cardif_linux_wext_set_iwauth(intdata, IW_AUTH_TKIP_COUNTERMEASURES,
+				       FALSE, "tkip countermeasures") < 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "Couldn't disable TKIP countermeasures!\n");
+	}
+
+      if (cardif_linux_wext_set_iwauth(intdata, IW_AUTH_DROP_UNENCRYPTED,
+				       TRUE, "drop unencrypted data") < 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "Couldn't enable dropping of unencrypted"
+		       " data!\n");
+	}
+
+      if (cardif_linux_wext_set_iwauth(intdata, IW_AUTH_80211_AUTH_ALG,
+				       IW_AUTH_ALG_OPEN_SYSTEM, "802.11 auth."
+				       " alg to open") < 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "Couldn't set 802.11 auth. alg to open.\n");
+	}
+
       if (cardif_linux_wext_set_wpa_ie(intdata, wpaie, len) < 0)
 	{
 	  debug_printf(DEBUG_NORMAL, "Couldn't set WPA IE on device %s!\n",
 		       intdata->intName);
 	}
-    }
 
-  intdata->flags |= DONTSCAN;
+      // For drivers that require something other than just setting a
+      // WPA IE, we will set the components for the IE instead.
+      cipher = cardif_linux_wext_iw_cipher(network_data->wpa_pairwise_crypt);
+      if (cardif_linux_wext_set_iwauth(intdata, IW_AUTH_CIPHER_PAIRWISE,
+       				       cipher, "pairwise cipher") < 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "Couldn't set pairwise cipher.\n");
+	}
+
+      cipher = cardif_linux_wext_iw_cipher(network_data->wpa_group_crypt);
+      if (cardif_linux_wext_set_iwauth(intdata, IW_AUTH_CIPHER_GROUP,
+      				       cipher, "group cipher") < 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "Couldn't set group cipher.\n");
+	}
+
+      if (network_data->methods->method_num != WPA_PSK)
+	{
+	  akm = IW_AUTH_KEY_MGMT_802_1X;
+	} else {
+	  akm = IW_AUTH_KEY_MGMT_PSK;
+	}
+
+      if (cardif_linux_wext_set_iwauth(intdata, IW_AUTH_KEY_MGMT, akm,
+				       "Authenticated Key Management Suite") 
+	  < 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "Couldn't set Authenticated Key "
+		       "Management Suite.\n");
+	}
+    }
+#endif
+
   cardif_linux_wext_set_ssid(intdata, newssid);
 
-  bssid = config_ssid_get_mac();
+  bssid = (char *) config_ssid_get_mac();
   if (bssid != NULL)
     {
       debug_printf(DEBUG_INT, "Dest. BSSID : ");
-      debug_hex_printf(DEBUG_INT, bssid, 6);
+      debug_hex_printf(DEBUG_INT, (u_char *) bssid, 6);
     }
 
-  //  cardif_linux_wext_set_bssid(intdata, bssid);
+  globals = config_get_globals();
+
+  if ((!globals) || (!TEST_FLAG(globals->flags, CONFIG_GLOBALS_FIRMWARE_ROAM)))
+    {
+      cardif_linux_wext_set_bssid(intdata, bssid);
+    }
   
   if ((intdata->wpa_ie == NULL) && (intdata->rsn_ie == NULL))
     {
@@ -980,11 +1296,6 @@ void cardif_linux_wext_associate(struct interface_data *intdata, char *newssid)
       cardif_linux_wext_unencrypted_eapol(intdata, TRUE);
     }
   
-  // We need to force zero keys to cause association. At least this is the default behavior
-  // elsewhere.
-  /*
-    debug_printf(DEBUG_INT, "ASSOCIATING: zeroing keys off.\n");
-    cardif_linux_wext_zero_keys(intdata);*/
   return;
 }
 
@@ -1036,34 +1347,58 @@ int cardif_linux_wext_drop_unencrypted(struct interface_data *intdata,
   return XENONE;
 }
 
-int cardif_linux_wext_delete_key(struct interface_data *intdata, int key_idx,
-				 int set_tx)
+void cardif_linux_wext_enc_capabilities(struct interface_data *intdata)
 {
 #if WIRELESS_EXT > 17
   struct iwreq wrq;
   struct lin_sock_data *sockData;
-  struct iw_encode_ext iwee;
+  struct iw_range *range;
+  char buffer[sizeof(struct iw_range) * 2];
+  int i;
 
   sockData = intdata->sockData;
 
+  intdata->enc_capa = 0;
+
   bzero(&wrq, sizeof(wrq));
   strncpy((char *)&wrq.ifr_name, intdata->intName, IFNAMSIZ);
 
-  bzero(&iwee, sizeof(iwee));
+  wrq.u.data.pointer = (caddr_t) buffer;
+  wrq.u.data.length = sizeof(buffer);
+  wrq.u.data.flags = 0;
 
-  iwee.alg = IW_ENCODE_ALG_NONE;
-  
-  if (set_tx) iwee.ext_flags |= IW_ENCODE_EXT_SET_TX_KEY;
-  
-  iwee.key_len = 0;
-
-  wrq.u.data.pointer = (caddr_t)&iwee;
-  wrq.u.data.length = sizeof(iwee);
-
-  if (ioctl(sockData->sockInt, SIOCSIWENCODEEXT, &wrq) < 0)
+  if (ioctl(sockData->sockInt, SIOCGIWRANGE, &wrq) < 0)
     {
-      debug_printf(DEBUG_NORMAL, "Error with key delete! (IOCTL failure.)\n");
+      debug_printf(DEBUG_NORMAL, "Couldn't get encryption capabilites!\n");
+      return;
     }
+
+  // Otherwise, determine what we have.
+  range = (struct iw_range *)buffer;
+
+  for (i=0; i<range->num_encoding_sizes; i++)
+    {
+      if (range->encoding_size[i] == 5) intdata->enc_capa |= DOES_WEP40;
+      if (range->encoding_size[i] == 13) intdata->enc_capa |= DOES_WEP104;
+    }
+
+  if (range->enc_capa & IW_ENC_CAPA_WPA) intdata->enc_capa |= DOES_WPA;
+  if (range->enc_capa & IW_ENC_CAPA_WPA2) intdata->enc_capa |= DOES_WPA2;
+  if (range->enc_capa & IW_ENC_CAPA_CIPHER_TKIP)
+    intdata->enc_capa |= DOES_TKIP;
+  if (range->enc_capa & IW_ENC_CAPA_CIPHER_CCMP)
+    intdata->enc_capa |= DOES_CCMP;
+#else
+  intdata->enc_capa = 0;
+#endif
+}
+
+int cardif_linux_wext_delete_key(struct interface_data *intdata, int key_idx,
+				 int set_tx)
+{
+#if WIRELESS_EXT > 17
+  return cardif_linux_wext_set_key_ext(intdata, IW_ENCODE_ALG_NONE, NULL,
+				       key_idx, set_tx, NULL, 0, NULL, 0);
 #else
   debug_printf(DEBUG_NORMAL, "%s : Not supported by WE(%d)!\n", __FUNCTION__,
 	       WIRELESS_EXT);
@@ -1079,18 +1414,19 @@ struct cardif_funcs cardif_linux_wext_driver = {
   .set_tkip_key = cardif_linux_wext_set_tkip_key,
   .set_ccmp_key = cardif_linux_wext_set_ccmp_key,
   .delete_key = cardif_linux_wext_delete_key,
-  .set_ssid = cardif_linux_wext_set_ssid,
   .associate = cardif_linux_wext_associate,
   .get_ssid = cardif_linux_wext_get_ssid,
   .get_bssid = cardif_linux_wext_get_bssid,
   .wpa_state = cardif_linux_wext_wpa_state,
   .wpa = cardif_linux_wext_wpa,
-  .roam = cardif_linux_wext_roam,
+  .wep_associate = cardif_linux_wext_wep_associate,
   .countermeasures = cardif_linux_wext_countermeasures,
   .drop_unencrypted = cardif_linux_wext_drop_unencrypted,
   .get_wpa_ie = cardif_linux_wext_get_wpa_ie,
   .get_wpa2_ie = cardif_linux_wext_get_wpa2_ie,
-  .enc_disable = cardif_linux_wext_enc_disable
+  .enc_disable = cardif_linux_wext_enc_disable,
+  .enc_capabilities = cardif_linux_wext_enc_capabilities,
+  .setbssid = cardif_linux_wext_set_bssid,
 };
 
 
