@@ -16,17 +16,19 @@ extern struct cardif_funcs cardif_linux_wext_driver;
 
 int cardif_linux_wext_set_WEP_key(struct interface_data *, u_char *, int, int);
 int cardif_linux_wext_set_ssid(struct interface_data *, char *);
+int cardif_linux_wext_set_bssid(struct interface_data *, u_char *);
 int cardif_linux_wext_check_ssid(struct interface_data *);
 int cardif_linux_wext_get_bssid(struct interface_data *, char *);
 int cardif_linux_wext_set_ssid(struct interface_data *, char *);
-int cardif_linux_wext_roam(struct interface_data *, int);
+int cardif_linux_wext_wep_associate(struct interface_data *, int);
 int cardif_linux_wext_get_ssid(struct interface_data *, char *);
-int cardif_linux_wext_scan(struct interface_data *);
+int cardif_linux_wext_scan(struct interface_data *, char);
 void cardif_linux_wext_associate(struct interface_data *, char *);
 void cardif_linux_wext_use_temp_keys(char);
 int cardif_linux_wext_enc_open(struct interface_data *);
 int cardif_linux_wext_enc_disable(struct interface_data *thisint);
 void cardif_linux_wext_set_zero_keys(struct interface_data *);
 void cardif_linux_wext_zero_keys(struct interface_data *);
+int cardif_linux_wext_wpa_state(struct interface_data *, char);
 
 #endif
