@@ -7,9 +7,41 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: eap.c,v 1.116 2005/08/09 01:39:13 chessing Exp $
- * $Date: 2005/08/09 01:39:13 $
+ * $Id: eap.c,v 1.125 2006/03/08 00:16:04 chessing Exp $
+ * $Date: 2006/03/08 00:16:04 $
  * $Log: eap.c,v $
+ * Revision 1.125  2006/03/08 00:16:04  chessing
+ * Fixed EAP hints code to work correctly when the request ID packet is padded out with null bytes.  (Observed in Aruba APs.)  Some changes/fixes for the EAP-AKA module.
+ *
+ * Revision 1.124  2006/03/02 04:52:41  chessing
+ * Added the ability to load a config based on the hints that are included in an EAP request identity message.  Should consider having a way to override listening to these messages.
+ *
+ * Revision 1.123  2006/02/23 22:26:53  chessing
+ * Fix for bug id #1415020.  'Building Xsupplicant 1.2.3 Fails on FC4'.
+ *
+ * Revision 1.122  2005/10/17 03:56:53  chessing
+ * Updates to the libxsupconfig library.  It no longer relies on other source from the main tree, so it can be used safely in other code with problems.
+ *
+ * Revision 1.121  2005/10/14 02:26:17  shaftoe
+ * - cleanup gcc 4 warnings
+ * - (re)add support for a pid in the form of /var/run/xsupplicant.<iface>.pid
+ *
+ * -- Eric Evans <eevans@sym-link.com>
+ *
+ * Revision 1.120  2005/09/26 02:04:07  shaftoe
+ * Yet another round of log output cleanup.
+ *
+ * -- Eric Evans <eevans@sym-link.com>
+ *
+ * Revision 1.119  2005/09/17 19:08:27  chessing
+ * Updates to the MADwifi driver so that it works correctly with dynamic WEP again.  Because of a requirement in the driver, the normal wext calls will authenticate, but only broadcast traffic will be able to pass.  So, all MAdwifi users should use -D madwifi to make everything work correctly.
+ *
+ * Revision 1.118  2005/09/08 16:27:01  chessing
+ * Some small updates to the new state machine code.  First attempt at an auto association mode.  (It mostly works. ;)
+ *
+ * Revision 1.117  2005/09/05 01:00:34  chessing
+ * Major overhaul to most of the state machines in Xsupplicant.  Also added additional error messages to the TLS functions to try to debug the one of the problems reported on the list.  Basic testing shows this new code to be more stable than previous code, but it needs more testing.
+ *
  * Revision 1.116  2005/08/09 01:39:13  chessing
  * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
@@ -27,7 +59,7 @@
 #include "frame_structs.h"
 #include "xsup_debug.h"
 #include "xsup_err.h"
-#include "config.h"
+#include "xsupconfig.h"
 #include "profile.h"
 #include "eap.h"
 #include "eapol.h"
@@ -102,7 +134,7 @@ void eap_init(struct interface_data *thisint)
  ***************************************************/
 void eap_reset(struct interface_data *thisint)
 {
-  // We have let the active EAP method know the we reset.
+  // We have let the active EAP method know that we reset.
   thisint->statemachine->eapRestart = FALSE;
 }
 
@@ -191,9 +223,9 @@ char eap_response_auth(struct interface_data *thisint)
 
   switch (eap_request_auth(network_data->activemethod,
 			   network_data->methods, 
-			   &thisint->recvframe[OFFSET_TO_EAP],
+			   (char *) &thisint->recvframe[OFFSET_TO_EAP],
 			   thisint->recv_size, 
-			   &thisint->sendframe[OFFSET_TO_EAP], &eapsize))
+			   (char *) &thisint->sendframe[OFFSET_TO_EAP], &eapsize))
     {
     case XINNERSUCCESS:
       // We got a LEAP success, so we need to tell the state machine to
@@ -222,20 +254,14 @@ char eap_response_auth(struct interface_data *thisint)
   
   if (eapsize != 0)
     {
-      eapolver = network_data->force_eapol_ver;
+      eapolver = eapol_get_eapol_ver();
 
-      // If we have not hard set the eapol version, then use the last
-      // version that the authenticator gave us.  This isn't in line with
-      // the 802.1X-REV-2004-d11 standard, but many authenticators are checking
-      // version #s, and breaking when it isn't 1. :-/
-      if ((eapolver < 1) || (eapolver > MAX_EAPOL_VER))
-	eapolver = snmp_get_dot1xSuppLastEapolFrameVersion();
-
-      eapol_build_header(EAP_PACKET, eapsize, eapolver, thisint->sendframe);
+      eapol_build_header(EAP_PACKET, eapsize, eapolver, (char *) thisint->sendframe);
       thisint->send_size = eapsize+OFFSET_TO_EAP;
       return XENONE;
     }
 
+  debug_printf(DEBUG_INT, "No response frame was built!\n");
   return XENOFRAMES;
 }
 
@@ -284,18 +310,11 @@ char eap_response_id(struct interface_data *thisint)
 
   eap_request_id(network_data->identity,
 		 eapdata->eap_identifier,
-		 &thisint->sendframe[OFFSET_TO_EAP], &eapsize);
+		 (char *) &thisint->sendframe[OFFSET_TO_EAP], &eapsize);
 
-  eapolver = network_data->force_eapol_ver;
+  eapolver = eapol_get_eapol_ver();
 
-  // If we have not hard set the eapol version, then use the last
-  // version that the authenticator gave us.  This isn't in line with
-  // the 802.1X-REV-2004-d11 standard, but many authenticators are checking
-  // version #s, and breaking when it isn't 1. :-/
-  if ((eapolver < 1) || (eapolver > MAX_EAPOL_VER))
-    eapolver = snmp_get_dot1xSuppLastEapolFrameVersion();
-
-  eapol_build_header(EAP_PACKET, eapsize, eapolver, thisint->sendframe);
+  eapol_build_header(EAP_PACKET, eapsize, eapolver, (char *) thisint->sendframe);
   thisint->send_size = eapsize+OFFSET_TO_EAP;
 
   return XENONE;
@@ -365,7 +384,8 @@ void eap_do_notify(struct interface_data *thisint)
 
   if (!thisint)
     {
-      debug_printf(DEBUG_EVERYTHING, "Got a Notify, but nothing to process!\n");
+      debug_printf(DEBUG_EVERYTHING, "Got a Notify, but interface context "
+		   "is invalid!\n");
       return;
     }
 
@@ -379,7 +399,7 @@ void eap_do_notify(struct interface_data *thisint)
   // We need to determine how long the string that we were returned is.
   // So, take the EAP length value, and subtract 5 to account for the EAP
   // header.
-  strncpy(&myval[0], &inframe[OFFSET_TO_DATA], (ntohs(myeap->eap_length)-5));
+  strncpy(&myval[0], (char *) &inframe[OFFSET_TO_DATA], (ntohs(myeap->eap_length)-5));
 
   debug_printf(DEBUG_NORMAL, "EAP Notification : %s\n", &myval[0]);
   bzero(thisint->recvframe, thisint->recv_size);
@@ -387,6 +407,72 @@ void eap_do_notify(struct interface_data *thisint)
   xsup_ipc_send_eap_notify(thisint, myval);
 }
 
+/*******************************************
+ *
+ * If the length of the EAP request ID packet is longer than the space needed
+ * to store the basic EAP request, then we may have some network hints.  These
+ * hints may include a network name that we can use to determine which network
+ * profile to read from the configuration.
+ *
+ *******************************************/
+void eap_process_hints(struct interface_data *ctx)
+{
+  struct eap_header     *myeap;
+  char                  *inframe;
+  char                  *hints;
+  char                  *networkid, *nidonly;
+  struct config_globals *globals;
+
+  inframe = ctx->recvframe;
+
+  myeap = (struct eap_header *)&inframe[OFFSET_TO_EAP];
+
+  if (ctx->recv_size > (OFFSET_TO_EAP + sizeof(struct eap_header) + 1))
+    {
+      // We may have some data.  The character that trails the end of the
+      // EAP packet should be 0x00.  So, skip it and show the results.
+      hints = (char *)&inframe[OFFSET_TO_EAP + sizeof(struct eap_header)+1];
+
+      globals = config_get_globals();
+      if (!globals)
+	{
+	  debug_printf(DEBUG_NORMAL, "Couldn't get global configuration "
+		       "options.  Assuming we should use EAP hints.\n");
+	}
+
+      if ((!globals) || 
+	  (!TEST_FLAG(globals->flags, CONFIG_GLOBALS_NO_EAP_HINTS)))
+	{
+
+	  if (hints[0] != 0x00)
+	    {
+	      networkid = strstr(hints, "networkid=");
+	      
+	      if (networkid != NULL)
+		{
+		  nidonly = strtok(networkid, ",");
+		  
+		  nidonly += strlen("networkid=");
+		  
+		  if (nidonly[0] != 0x00)
+		    {
+		      debug_printf(DEBUG_NORMAL, "Network ID from EAP hint : %s\n", nidonly);
+		      debug_printf(DEBUG_NORMAL, "If this is a wired network, the above ID "
+				   "can be used in the configuration file to identify this "
+				   "network.\n");
+		      
+		      if (config_build(ctx, nidonly) != TRUE)
+			{
+			  debug_printf(DEBUG_NORMAL, "Couldn't build config for network %s!\n",
+				       nidonly);
+			}
+		    }
+		}
+	    } 
+	}
+    }
+}
+
 /*******************************************
  *
  * Process the EAP piece of the packet, determine what type of EAP packet it is
@@ -402,7 +488,8 @@ int eap_process_header(struct interface_data *thisint)
 
   if (!thisint)
     {
-      debug_printf(DEBUG_NORMAL, "Nothing to do in eap_process_header()!\n");
+      debug_printf(DEBUG_NORMAL, "Invalid interface context in %s!\n",
+		   __FUNCTION__);
       return XEGENERROR;
     }
 
@@ -431,7 +518,8 @@ int eap_process_header(struct interface_data *thisint)
     case EAP_REQUEST:
       if (eap_get_name_from_type(myeap->eap_type) != NULL)
 	{
-	  debug_printf(DEBUG_EVERYTHING, "Got EAP-Request for %s\n",
+	  debug_printf(DEBUG_EVERYTHING, "Got EAP-Request for type %d (%s).\n",
+		       myeap->eap_type,
 		       eap_get_name_from_type(myeap->eap_type));
 	}
 
@@ -445,6 +533,7 @@ int eap_process_header(struct interface_data *thisint)
 	{
 	case EAP_TYPE_IDENTITY:
 	  debug_printf(DEBUG_EVERYTHING, "Got EAP-Request-Identification.\n");
+	  eap_process_hints(thisint);
 	  return EAP_REQUEST_ID;
 	  break;
 	  
@@ -467,7 +556,6 @@ int eap_process_header(struct interface_data *thisint)
       break;
 
     case EAP_RESPONSE:
-      debug_printf(DEBUG_EVERYTHING, "Got EAP-Response, ignoring(?).\n");
       if (myeap->eap_type == EAP_TYPE_LEAP) {
 	debug_printf(DEBUG_EVERYTHING, "Got LEAP Response Packet.  Ready for AP verification!\n");
 	thisint->statemachine->eapolEap = TRUE;
@@ -500,7 +588,8 @@ int eap_process_header(struct interface_data *thisint)
 	thisint->statemachine->eapolEap = TRUE;
 	return EAP_REQUEST_AUTH;
       } else {
-	debug_printf(DEBUG_NORMAL, "Authenticated!\n");
+	debug_printf(DEBUG_NORMAL, "Successfully authenticated %s\n", 
+			thisint->intName);
 	thisint->statemachine->eapSuccess = TRUE;
       }
 
@@ -513,7 +602,8 @@ int eap_process_header(struct interface_data *thisint)
 
     case EAP_FAILURE:
       debug_printf(DEBUG_EVERYTHING, "Got EAP-Failure!\n");
-      debug_printf(DEBUG_NORMAL, "Failure!\n");
+      debug_printf(DEBUG_NORMAL, "Failed to authenticate %s\n", 
+		      thisint->intName);
       snmp_eapol_fail_rx();
       thisint->statemachine->eapFail = TRUE;
       eap_do_fail(network_data->activemethod);
@@ -646,7 +736,8 @@ int eap_create_active_method(struct generic_eap_data **activemethod,
 
   if (activemethod == NULL)
     {
-      debug_printf(DEBUG_NORMAL, "Invalid pointer passed to eap_create_active_method!\n");
+      debug_printf(DEBUG_NORMAL, "Invalid pointer passed to %s!\n",
+		   __FUNCTION__);
       return XEMALLOC;
     }
 
@@ -659,7 +750,8 @@ int eap_create_active_method(struct generic_eap_data **activemethod,
 
       if (mymethod == NULL)
 	{
-	  debug_printf(DEBUG_NORMAL, "Couldn't allocate memory in eap_create_active_method()!\n");
+	  debug_printf(DEBUG_NORMAL, "Couldn't allocate memory in %s:%d!\n",
+		       __FUNCTION__, __LINE__);
 	  return XEMALLOC;
 	}
 
@@ -701,9 +793,17 @@ int eap_request_auth(struct generic_eap_data *activemethod,
   char *tosendframe;
   int retVal = XENONE, pwd_needed;
 
-  if ((!outframe) || (!eapsize))   
+  if (!outframe)
     {
-      debug_printf(DEBUG_NORMAL, "Invalid parameter passed in to eap_request_auth()!\n");
+      debug_printf(DEBUG_NORMAL, "Invalid value for 'outframe' in %s:%d!\n",
+		   __FUNCTION__, __LINE__);
+      return XEMALLOC;
+    }
+
+  if (!eapsize)
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid value for 'eapsize' in %s:%d!\n",
+		   __FUNCTION__, __LINE__);
       return XEMALLOC;
     }
 
@@ -773,6 +873,8 @@ int eap_request_auth(struct generic_eap_data *activemethod,
 
   if (eaphandlers[eapmethod].eap_auth_type == NO_EAP_AUTH)
     {
+      debug_printf(DEBUG_NORMAL, "No valid EAP type could be found in %s:%d!\n",
+		   __FUNCTION__, __LINE__);
       // We got an error.
       return XEINVALIDEAP;
     }
@@ -814,8 +916,8 @@ int eap_request_auth(struct generic_eap_data *activemethod,
   if ((pwd_needed != 1) || (activemethod->tempPwd != NULL))
     {
       switch ((*eaphandlers[eapmethod].eap_auth_handlers)(activemethod, 
-						      tosendframe, eapinsize, 
-						  &outframe[sizeof(struct eap_header)], 
+						      (u_char *) tosendframe, eapinsize, 
+						  (u_char *) &outframe[sizeof(struct eap_header)], 
 						  eapsize))
 	{
 	case XINNERSUCCESS:
@@ -875,8 +977,9 @@ int eap_request_auth(struct generic_eap_data *activemethod,
 	myouteap->eap_code = EAP_RESPONSE;
 	if (myineap == NULL)
 	  {
-	    debug_printf(DEBUG_NORMAL, "Invalid packet!\n");
-	    return -1;
+	    debug_printf(DEBUG_NORMAL, "Invalid packet! (%s:%d)\n",
+			 __FUNCTION__, __LINE__);
+	    return XENOFRAMES;
 	  }
 	myouteap->eap_identifier = myineap->eap_identifier;
       }
@@ -962,9 +1065,9 @@ int eap_get_keying_material(struct interface_data *thisint)
 
   (*eaphandlers[eapmethod].eap_auth_get_keys)(thisint);
 
-  thisint->statemachine->PMK = thisint->keyingMaterial;
+  thisint->statemachine->PMK = (char *) thisint->keyingMaterial;
   debug_printf(DEBUG_INT, "PMK Keys (%d) :\n", thisint->keyingLength);
-  debug_hex_dump(DEBUG_INT, thisint->statemachine->PMK, thisint->keyingLength);
+  debug_hex_dump(DEBUG_INT, (u_char *) thisint->statemachine->PMK, thisint->keyingLength);
 
   debug_printf(DEBUG_INT, "WEP Keys (%d) :\n", thisint->keyingLength);
   debug_hex_dump(DEBUG_INT, thisint->keyingMaterial, thisint->keyingLength);
