@@ -7,9 +7,18 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: aka.c,v 1.6 2005/08/14 02:11:09 chessing Exp $
- * $Date: 2005/08/14 02:11:09 $
+ * $Id: aka.c,v 1.9 2006/03/21 18:22:10 chessing Exp $
+ * $Date: 2006/03/21 18:22:10 $
  * $Log: aka.c,v $
+ * Revision 1.9  2006/03/21 18:22:10  chessing
+ * Fixes to EAP-AKA code.  Changed a few defaults to disable Xsupplicant based roaming, and passive scanning for this release.  (It would currently cause more problems than it would solve.
+ *
+ * Revision 1.8  2006/03/08 00:16:04  chessing
+ * Fixed EAP hints code to work correctly when the request ID packet is padded out with null bytes.  (Observed in Aruba APs.)  Some changes/fixes for the EAP-AKA module.
+ *
+ * Revision 1.7  2006/02/23 22:26:53  chessing
+ * Fix for bug id #1415020.  'Building Xsupplicant 1.2.3 Fails on FC4'.
+ *
  * Revision 1.6  2005/08/14 02:11:09  chessing
  * Fixes for EAP-AKA.  It should now really behave the way it is supposed to. ;)
  *
@@ -35,7 +44,7 @@
 
 #include "winscard.h"
 #include "profile.h"
-#include "config.h"
+#include "xsupconfig.h"
 #include "eap.h"
 #include "../sim/eapsim.h"
 #include "eapaka.h"
@@ -99,8 +108,15 @@ int aka_do_at_mac(struct generic_eap_data *thisint,
   unsigned char auts[16], sres[16], ck[16], *keydata, mac_val[16];
   unsigned char mac_calc[20], ik[16], *mk, kc[16], *tohash, *framecpy;
 
+  if (!thisint)
+    {
+      debug_printf(DEBUG_NORMAL, "No EAP data available!  This authentication"
+		   " cannot continue!\n");
+      return XEMALLOC;
+    }
+
   debug_printf(DEBUG_AUTHTYPES, "Got an AT_MAC\n");
-  
+
   saved_offset = (*packet_offset);
 
 #ifndef RADIATOR_TEST  
@@ -114,6 +130,7 @@ int aka_do_at_mac(struct generic_eap_data *thisint,
 			    (unsigned char *)&kc) == -2)
     {
       // We have a sync failure.  So, return it.
+      memcpy((unsigned char *)&mydata->auts, (unsigned char *)&auts, 16);
       return XEAKASYNCFAIL;
     }
 #else
@@ -266,11 +283,11 @@ int aka_do_sync_fail(struct aka_eaptypedata *mydata, u_char *out, int *outsize)
   typelenres = (struct typelengthres *)&out[outptr];
   outptr+=4;
 
-  typelenres->type = AT_AUTN;
+  typelenres->type = AT_AUTS;
   typelenres->length = 5;
   typelenres->reserved = 0;
 
-  memcpy(&out[outptr], mydata->autn, 16);
+  memcpy(&out[outptr], mydata->auts, 16);
   outptr+=16;
 
   *outsize = outptr;
