@@ -7,9 +7,15 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: eapaka.c,v 1.8 2005/08/14 02:11:09 chessing Exp $
- * $Date: 2005/08/14 02:11:09 $
+ * $Id: eapaka.c,v 1.10 2006/03/08 00:16:04 chessing Exp $
+ * $Date: 2006/03/08 00:16:04 $
  * $Log: eapaka.c,v $
+ * Revision 1.10  2006/03/08 00:16:04  chessing
+ * Fixed EAP hints code to work correctly when the request ID packet is padded out with null bytes.  (Observed in Aruba APs.)  Some changes/fixes for the EAP-AKA module.
+ *
+ * Revision 1.9  2006/02/23 22:26:53  chessing
+ * Fix for bug id #1415020.  'Building Xsupplicant 1.2.3 Fails on FC4'.
+ *
  * Revision 1.8  2005/08/14 02:11:09  chessing
  * Fixes for EAP-AKA.  It should now really behave the way it is supposed to. ;)
  *
@@ -39,7 +45,7 @@
 
 #include "winscard.h"
 #include "profile.h"
-#include "config.h"
+#include "xsupconfig.h"
 #include "eap.h"
 #include "eapaka.h"
 #include "../sim/eapsim.h"
@@ -279,6 +285,7 @@ int eapaka_process(struct generic_eap_data *thisint, u_char *dataoffs,
 
   if (userdata->username == NULL)
     {
+      debug_printf(DEBUG_NORMAL, "No username, setting from identity!\n");
       username = thisint->identity;
     } else {
       username = userdata->username;
@@ -360,6 +367,7 @@ int eapaka_process(struct generic_eap_data *thisint, u_char *dataoffs,
 				     &packet_offset, username);
 	      if (retval == XEAKASYNCFAIL)
 		{
+		  printf("Sync failure 2..  Doing sync failure.\n");
 		  return aka_do_sync_fail(mydata, out, outsize);
 		} else if (retval != XENONE) return retval;
 	      break;
@@ -415,6 +423,7 @@ int eapaka_process(struct generic_eap_data *thisint, u_char *dataoffs,
       memcpy((char *)&framecpy[5], out, retsize);
       debug_printf(DEBUG_AUTHTYPES, "Preframe :\n");
       debug_hex_dump(DEBUG_AUTHTYPES, framecpy, retsize);
+
       // Zero out the mac.
       bzero((char *)&framecpy[outptr+5], 16);
       debug_printf(DEBUG_AUTHTYPES, "Frame to hash : \n");
