@@ -19,8 +19,6 @@
 #ifndef _EAP_AKA_H_
 #define _EAP_AKA_H_
 
-#define EAP_TYPE_AKA   23
-
 #ifdef EAP_SIM_ENABLE
 
 // These are defined in section 8 of Arkko-12
@@ -70,6 +68,7 @@ struct aka_eaptypedata {
   char *nonce_mt;
   char random_num[16];
   char autn[16];
+  char auts[16];
   char res[16];
   int reslen;
   char K_encr[16], K_aut[16], msk[64], emsk[64];
