@@ -19,7 +19,7 @@
 #include "xsup_err.h"
 #include "frame_structs.h"
 
-#include "config.h"
+#include "xsupconfig.h"
 #include "profile.h"
 #include "eap.h"
 #include "eapleap.h"
@@ -50,7 +50,7 @@ static void ntPwdHash(unsigned char *MD4Hash, char *password) {
         unicodePass[2 * i + 1] = 0;
     }
     /* Encrypt plain text password to a 16-byte MD4 hash */
-    md4_calc(MD4Hash, unicodePass, passLen * 2);
+    md4_calc(MD4Hash, (u_char *) unicodePass, passLen * 2);
 }
 
 void leap_mschap(char * password, char * response) {
@@ -64,7 +64,7 @@ void leap_mschap(char * password, char * response) {
 
     ntPwdHash(MD4Hash, password);
     md4_calc(MD4HashHash, MD4Hash, 16);
-    ChallengeResponse(leapchallenges->apc, MD4HashHash, response);
+    ChallengeResponse((char *) leapchallenges->apc, (char *) MD4HashHash, response);
 }
 
 
@@ -199,7 +199,7 @@ int eapleap_process(struct generic_eap_data *thisint, u_char *dataoffs,
 
   // LEAP shouldn't be used as an inner type, so we should be able to get
   // away with this.
-  data = dataoffs-5;
+  data = (char *) dataoffs-5;
   eapheader = (struct eap_header *)data;
 
   switch (eapheader->eap_code) {
@@ -229,7 +229,8 @@ int eapleap_process(struct generic_eap_data *thisint, u_char *dataoffs,
     debug_printf(DEBUG_AUTHTYPES, "(EAP-LEAP) Incoming Peer Challenge Random Value (Length = %d) : ",leaprequest->count);
     debug_hex_printf(DEBUG_AUTHTYPES,(uint8_t *)leaprequest->randval, leaprequest->count);
 
-    NtChallengeResponse((char *)leapchallenges->pc, userdata->password, (char *)&chall_response);
+    NtChallengeResponse((char *)leapchallenges->pc, userdata->password, 
+			(char *)&chall_response, 0);
 
     debug_printf(DEBUG_AUTHTYPES, "MSCHAP Response Calculated : ");
     debug_hex_printf(DEBUG_AUTHTYPES, (uint8_t *)&chall_response, 24);
@@ -277,9 +278,12 @@ int eapleap_process(struct generic_eap_data *thisint, u_char *dataoffs,
     // ***********************************************************************************************
     debug_printf(DEBUG_AUTHTYPES, "(EAP-LEAP) Got EAP-SUCCESS\n");
     memset(chall_response, 0x0, 8);
-    NtChallengeResponse((char *)leaprequest->randval, userdata->password, (char *)&chall_response);
+    NtChallengeResponse((char *)leaprequest->randval, userdata->password, 
+			(char *)&chall_response, 0);
 
-    GenerateNTResponse((char *)leapchallenges->pr, (char *)leapchallenges->pc, username, userdata->password, chall_response);
+    GenerateNTResponse((char *)leapchallenges->pr, (char *)leapchallenges->pc, 
+		       username, userdata->password, (char *) chall_response, 
+		       0);
 
     // store Access Point Challenge
     memcpy((uint8_t *)leapchallenges->apc, (char *)chall_response, 8);
@@ -337,7 +341,7 @@ int eapleap_process(struct generic_eap_data *thisint, u_char *dataoffs,
 
     leapresponse = (struct leap_responses *)dataoffs;
 
-    challenge_response_got =  (char *)malloc(leapresponse->count+1);
+    challenge_response_got =  (u_char *)malloc(leapresponse->count+1);
     if (!challenge_response_got) {
       debug_printf(DEBUG_NORMAL, "(EAP-LEAP) challenge_response_got is NULL after malloc!\n");
     }
@@ -350,7 +354,7 @@ int eapleap_process(struct generic_eap_data *thisint, u_char *dataoffs,
     memset(challenge_response_expected, 0x0, 24);
 
     // Calculate the 24 bytes MS-CHAP Challenge Response
-    leap_mschap(userdata->password, challenge_response_expected);
+    leap_mschap(userdata->password, (char *) challenge_response_expected);
 
     if (memcmp(challenge_response_got, challenge_response_expected, 24) == 0) {
       debug_printf(DEBUG_AUTHTYPES, "(EAP-LEAP) AP ChallengeResponse got is valid.\n");
@@ -386,7 +390,7 @@ int eapleap_process(struct generic_eap_data *thisint, u_char *dataoffs,
     debug_printf(DEBUG_AUTHTYPES, "(EAP-LEAP) leapchallenges->apr : ");
     debug_hex_printf(DEBUG_AUTHTYPES, (uint8_t *)leapchallenges->apr, 24);
 
-    GetMasterLEAPKey((char *)MD4HashHash, leapchallenges->apc, leapchallenges->apr, leapchallenges->pc, leapchallenges->pr, (char *)&MasterKey);
+    GetMasterLEAPKey((char *)MD4HashHash, (char *) leapchallenges->apc, (char *) leapchallenges->apr, (char *) leapchallenges->pc, (char *) leapchallenges->pr, (char *)&MasterKey);
     debug_printf(DEBUG_AUTHTYPES, "MasterLEAPKey : ");
     debug_hex_printf(DEBUG_AUTHTYPES, (unsigned char *)&MasterKey, 16);
 
@@ -444,7 +448,7 @@ int eapleap_get_keys(struct interface_data *thisint)
   mydata = (struct leap_data *)network_data->activemethod->eap_data;
 
   // Right now, we don't return anything from LEAP.
-  thisint->keyingMaterial = mydata->keyingMaterial;
+  thisint->keyingMaterial = (u_char *) mydata->keyingMaterial;
   thisint->keyingLength = 16;  // We only use 16 bytes for this keying material!
 
   return 0;
