@@ -12,11 +12,11 @@
 #ifndef _MSCHAPV2_H_
 #define _MSCHAPV2_H_
 
-void GenerateNTResponse(char *, char *, char *, char *, char *);
+void GenerateNTResponse(char *, char *, char *, char *, char *, int);
 void ChallengeResponse(char *, char *, char *);
-void NtChallengeResponse(char *, char *, char *);
+void NtChallengeResponse(char *, char *, char *, int);
 void CheckAuthenticatorResponse(char *, char *, char *, char *, char *, char *,
-				int *);
+				int *, int);
 void GetAsymetricStartKey(char *, char *, int, int, int);
 void GetMasterKey(char *, char *, char *);
 void GetMasterLEAPKey(char *, char *, char *, char *, char *, char *);
