@@ -13,9 +13,6 @@
 
 #include "profile.h"
 
-#define EAP_TYPE_OTP    5
-#define EAP_TYPE_GTC    6
-
 int eapotp_setup(struct generic_eap_data *);
 int eapotp_process(struct generic_eap_data *, u_char *, int, u_char *, int *);
 int eapotp_get_keys(struct interface_data *);
