@@ -7,9 +7,12 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: simd11.c,v 1.10 2005/08/09 01:39:17 chessing Exp $
- * $Date: 2005/08/09 01:39:17 $
+ * $Id: simd11.c,v 1.11 2006/02/23 22:26:53 chessing Exp $
+ * $Date: 2006/02/23 22:26:53 $
  * $Log: simd11.c,v $
+ * Revision 1.11  2006/02/23 22:26:53  chessing
+ * Fix for bug id #1415020.  'Building Xsupplicant 1.2.3 Fails on FC4'.
+ *
  * Revision 1.10  2005/08/09 01:39:17  chessing
  * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
@@ -32,7 +35,7 @@
 
 #include "winscard.h"
 #include "profile.h"
-#include "config.h"
+#include "xsupconfig.h"
 #include "eap.h"
 #include "eapsim.h"
 #include "xsup_debug.h"
