@@ -50,9 +50,15 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: sm_handler.c,v 1.14 2005/08/09 01:39:17 chessing Exp $
- * $Date: 2005/08/09 01:39:17 $
+ * $Id: sm_handler.c,v 1.16 2006/03/21 18:22:10 chessing Exp $
+ * $Date: 2006/03/21 18:22:10 $
  * $Log: sm_handler.c,v $
+ * Revision 1.16  2006/03/21 18:22:10  chessing
+ * Fixes to EAP-AKA code.  Changed a few defaults to disable Xsupplicant based roaming, and passive scanning for this release.  (It would currently cause more problems than it would solve.
+ *
+ * Revision 1.15  2006/03/08 00:16:04  chessing
+ * Fixed EAP hints code to work correctly when the request ID packet is padded out with null bytes.  (Observed in Aruba APs.)  Some changes/fixes for the EAP-AKA module.
+ *
  * Revision 1.14  2005/08/09 01:39:17  chessing
  * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
@@ -324,7 +330,8 @@ int sm_handler_init_ctx(SCARDCONTEXT *card_ctx)
   ret = SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, card_ctx);
   if (ret != SCARD_S_SUCCESS)
     {
-      printf("Couldn't establish Smart Card context!  (Is pcscd loaded?)\n");
+      debug_printf(DEBUG_NORMAL, "Couldn't establish Smart Card context!  "
+		   "(Is pcscd loaded?)\n");
       print_sc_error(ret);
       return -1;
     }
@@ -513,6 +520,170 @@ strtohex(u8  *src, u8  *dest, int *blen)
   return 0;
 }
 
+int sm_check_response(u_char s1, u_char s2)
+{
+  u_char t;
+
+  switch (s1)
+    {
+    case 0x67:
+      debug_printf(DEBUG_NORMAL, "SIM : incorrect parameter P3\n");
+      break;
+
+    case 0x6B:
+      debug_printf(DEBUG_NORMAL, "SIM : incorrect parameter P1 or P2\n");
+      break;
+
+    case 0x6D:
+      debug_printf(DEBUG_NORMAL, "SIM : unknown instruction code given in the command\n");
+      break;
+
+    case 0x6E:
+      debug_printf(DEBUG_NORMAL, "SIM : wrong instruction class given in the command\n");
+      break;
+
+    case 0x6F:
+      debug_printf(DEBUG_NORMAL, "SIM : technical problem with no diagnostic gien\n");
+      break;
+
+    case 0x6C:
+      debug_printf(DEBUG_EXCESSIVE, "SIM : Invalid length.  Should have been %d.\n", s2);
+      break;
+
+    case 0x92:
+      if (s2 == 0x40)
+	{
+	  debug_printf(DEBUG_NORMAL, "SIM : memory problem\n");
+	} 
+      else 
+	{
+	  debug_printf(DEBUG_NORMAL, "SIM : command successful but after using an internal update retry routine %d time(s)\n", s2);
+	}
+      break;
+
+    case 0x94:
+      switch (s2)
+	{
+	case 0x00:
+	  debug_printf(DEBUG_NORMAL, "SIM : no EF selected\n");
+	  break;
+
+	case 0x02:
+	  debug_printf(DEBUG_NORMAL, "SIM : out of range (invalid address)\n");
+	  break;
+
+	case 0x04:
+	  debug_printf(DEBUG_NORMAL, "SIM : file ID, or pattern, not found\n");
+	  break;
+
+	case 0x08:
+	  debug_printf(DEBUG_NORMAL, "SIM : file is inconsistent with the command\n");
+	  break;
+
+	default:
+	  return -1;
+	  break;
+	}
+      break;
+
+    case 0x98:
+      switch (s2)
+	{
+	case 0x02:
+	  debug_printf(DEBUG_NORMAL, "SIM : no CHV initialised\n");
+	  break;
+
+	case 0x04:
+	  debug_printf(DEBUG_NORMAL, "SIM : access condition not fulfilled\n");
+	  break;
+
+	case 0x08:
+	  debug_printf(DEBUG_NORMAL, "SIM : in contradiction with CHV status\n");
+	  break;
+
+	case 0x10:
+	  debug_printf(DEBUG_NORMAL, "SIM : in contradiction with invalidation status\n");
+	  break;
+
+	case 0x40:
+	  debug_printf(DEBUG_NORMAL, "SIM : unsuccessful CHV verification, no attempt left\n");
+	  break;
+
+	case 0x50:
+	  debug_printf(DEBUG_NORMAL, "SIM : increase cannot be performed, max value reached\n");
+	  break;
+
+	default:
+	  return -1;
+	  break;
+	}
+      break;
+
+    case 0x69:
+      switch (s2)
+	{
+	case 0x82:
+	  debug_printf(DEBUG_NORMAL, "SIM : Security status not satisfied\n");
+	  break;
+
+	case 0x85:
+	  debug_printf(DEBUG_NORMAL, "SIM : Conditions of use not satisfied\n");
+	  break;
+
+	default:
+	  return -1;
+	  break;
+	}
+
+    case 0x6a:
+      switch (s2)
+	{
+	case 0x88:
+	  debug_printf(DEBUG_NORMAL, "SIM : reference data not found\n");
+	  break;
+
+	default:
+	  if ((s2 & 0x80) == 0x80)
+	    {
+	      debug_printf(DEBUG_EXCESSIVE, "Invalid P1-P2 value.\n");
+	    }
+	  else
+	    {
+	      return -1;
+	    }
+	}
+      break;
+
+    case 0x63:
+      switch (s2)
+	{
+	case 0x00:
+	  debug_printf(DEBUG_NORMAL, "SIM : authentication failed\n");
+	  break;
+
+	case 0x01:
+	  debug_printf(DEBUG_NORMAL, "SIM : synchronisation failure\n");
+	  break;
+
+	default:
+	  if ((s2 & 0xc0) == 0xc0)
+	    {
+	      t = s2 - 0xc0;
+	      debug_printf(DEBUG_NORMAL, "%d pin attempts remain.\n", t);
+	    }
+	  else
+	    {
+	      return -1;
+	    }
+	}
+      break;
+
+    default:
+      return -1;
+    }
+
+  return 0;
+}
 
 /* card_io -
  *    send a command to the card
@@ -526,7 +697,7 @@ cardio(SCARDHANDLE *card_hdl, char *cmd, long reader_protocol, char mode2g,
 	LPBYTE outbuff, LPDWORD olen, char debug)
 {
   static char getresponse[5]= {0xa0,0xc0,0x00,0x00,0x00 };
-  int cmdlen, ret, p;
+  int cmdlen, ret, p, i;
   u8 bcmd[MAXBUFF];
   SCARD_IO_REQUEST scir;
 
@@ -571,6 +742,9 @@ cardio(SCARDHANDLE *card_hdl, char *cmd, long reader_protocol, char mode2g,
     }
   } 
 
+  //  debug_hex_printf(DEBUG_NORMAL, outbuff, *olen);
+  //  printf("\n\n");
+
   if (*olen >= 2)
     {
       t_response *t = response;
@@ -592,7 +766,10 @@ cardio(SCARDHANDLE *card_hdl, char *cmd, long reader_protocol, char mode2g,
 	  t++;
 
 	  if (!found) {
-	    debug_printf(DEBUG_NORMAL, "Sim Card Response : %2.2X %2.2X (unknown response)\n", outbuff[p], outbuff[p+1]);
+	    if (sm_check_response(outbuff[p], outbuff[p+1]) != 0)
+	      {
+		debug_printf(DEBUG_NORMAL, "Sim Card Response : %2.2X %2.2X (unknown response)\n", outbuff[p], outbuff[p+1]);
+	      }
 	  } else {
 	    debug_printf(DEBUG_NORMAL,"\n");
 	  }
@@ -661,7 +838,8 @@ char *sm_handler_2g_imsi(SCARDHANDLE *card_hdl, char reader_mode, char *pin)
 
   if (!card_hdl)
     {
-      printf("Invalid card handle passed to sm_handler_2g_imsi()!\n");
+      debug_printf(DEBUG_NORMAL, "Invalid card handle passed to "
+		   "sm_handler_2g_imsi()!\n");
       return NULL;
     }
 
@@ -936,6 +1114,13 @@ char *sm_handler_3g_imsi(SCARDHANDLE *card_hdl, char reader_mode, char *pin)
 		       pinretries-1);
 	  return NULL;
 	}
+
+      if (cardio(card_hdl, buf2, reader_mode, MODE3G, (LPBYTE)&buf, &len, DO_DEBUG) != 0)
+	{
+	  debug_printf(DEBUG_NORMAL, "Invalid PIN! (%d tries remain.)\n",
+		       pinretries-1);
+	  return NULL;
+	}
     } //else {
       //      debug_printf(DEBUG_AUTHTYPES, "PIN not needed!\n");
       //}
@@ -1002,8 +1187,9 @@ int sm_handler_do_3g_auth(SCARDHANDLE *card_hdl, char reader_mode,
 
   if ((sw1 == 0x90) && (sw2 == 0x00) && (buf[0] == 0xdc))
     {
-      debug_printf(DEBUG_NORMAL, "Sync failure!\n");
-      memcpy(c_auts, buf+2, buf[1]);
+      debug_printf(DEBUG_NORMAL, "Sync failure! (Result length = %d)\n",
+		   len);
+      memcpy(c_auts, buf+2, len);
       return -2;
     }
 
