@@ -7,9 +7,33 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: tls_funcs.c,v 1.39 2005/08/12 03:34:06 chessing Exp $
- * $Date: 2005/08/12 03:34:06 $
+ * $Id: tls_funcs.c,v 1.46 2005/11/18 19:14:47 chessing Exp $
+ * $Date: 2005/11/18 19:14:47 $
  * $Log: tls_funcs.c,v $
+ * Revision 1.46  2005/11/18 19:14:47  chessing
+ * Fixes to the OpenSSL detection settings in the autoconf file.  Added some checks for SHA changes in OpenSSL 0.9.8
+ *
+ * Revision 1.45  2005/11/12 23:59:52  chessing
+ * Some typo/debug patches from Pekka Savola.
+ *
+ * Revision 1.44  2005/10/17 03:56:54  chessing
+ * Updates to the libxsupconfig library.  It no longer relies on other source from the main tree, so it can be used safely in other code with problems.
+ *
+ * Revision 1.43  2005/10/14 02:26:18  shaftoe
+ * - cleanup gcc 4 warnings
+ * - (re)add support for a pid in the form of /var/run/xsupplicant.<iface>.pid
+ *
+ * -- Eric Evans <eevans@sym-link.com>
+ *
+ * Revision 1.42  2005/09/14 03:34:54  chessing
+ * Small cosmetic changes.  Default association mode is now auto instead of manual. Fixes for bug IDs #1290449 & #1290323.
+ *
+ * Revision 1.41  2005/09/08 16:27:02  chessing
+ * Some small updates to the new state machine code.  First attempt at an auto association mode.  (It mostly works. ;)
+ *
+ * Revision 1.40  2005/09/05 01:00:37  chessing
+ * Major overhaul to most of the state machines in Xsupplicant.  Also added additional error messages to the TLS functions to try to debug the one of the problems reported on the list.  Basic testing shows this new code to be more stable than previous code, but it needs more testing.
+ *
  * Revision 1.39  2005/08/12 03:34:06  chessing
  * Fix to the TLS implementation, should help avoid some of the weird 'block cipher pad' errors.  Also includes a partial implementation of the ability to use static WEP keys based on the SSID in use.
  *
@@ -30,7 +54,7 @@
 #include <string.h>
 #include <inttypes.h>
 #include <unistd.h>
-#include "config.h"
+#include "xsupconfig.h"
 #include "profile.h"
 #include "eap.h"
 #include "tls_funcs.h"
@@ -38,6 +62,16 @@
 #include "xsup_debug.h"
 #include "xsup_err.h"
 
+// In OpenSSL 0.9.8 we need to explicitly include the SHA header.
+#ifndef SHA_DIGEST_LENGTH
+#include <openssl/sha.h>
+#endif
+
+// If it *STILL* isn't around, then just define it.
+#ifndef SHA_DIGEST_LENGTH
+#define SHA_DIGEST_LENGTH 20
+#endif
+
 int engine_load_dynamic_opensc(struct smartcard *sc);
 void set_smartcard_pin(char *pin);
 UI_METHOD *UI_noninteractive(void);
@@ -85,6 +119,9 @@ void tls_funcs_process_error()
     {
       debug_printf(DEBUG_NORMAL, "OpenSSL Error -- %s\n", 
 		   ERR_error_string(err, NULL));
+      debug_printf(DEBUG_NORMAL, "Library  : %s\n", ERR_lib_error_string(err));
+      debug_printf(DEBUG_NORMAL, "Function : %s\n", ERR_func_error_string(err));
+      debug_printf(DEBUG_NORMAL, "Reason   : %s\n", ERR_reason_error_string(err));
     }
 }
 
@@ -130,8 +167,10 @@ int tls_funcs_init(struct generic_eap_data *thisint)
   mytls_vars = (struct tls_vars *)thisint->eap_data;
   userdata = (struct config_eap_tls *)thisint->eap_conf_data;
 
-  SSL_library_init();
-  SSL_load_error_strings();
+  if (mytls_vars->ctx != NULL)
+    {
+      debug_printf(DEBUG_NORMAL, "mytls_vars->ctx != NULL!!!\n");
+    }
 
   mytls_vars->ctx = SSL_CTX_new(TLSv1_method());
   if (mytls_vars->ctx == NULL)
@@ -211,7 +250,11 @@ int tls_funcs_start(struct tls_vars *mytls_vars)
   if (!mytls_vars->ssl)
     {
       resval = tls_funcs_build_new_session(mytls_vars);
-      if (resval != XENONE) return resval;
+      if (resval != XENONE) 
+	{
+	  debug_printf(DEBUG_NORMAL, "Error building a new session!\n");
+	  return resval;
+	}
     } else {
       // We already established a connection, so we probably we need to
       // resume the session.
@@ -337,6 +380,12 @@ int tls_funcs_parse(struct generic_eap_data *thisint, u_char *indata,
     }
   mytls_vars = (struct tls_vars *)thisint->eap_data;
 
+  if (!mytls_vars)
+    {
+      debug_printf(DEBUG_NORMAL, "mytls_vars == NULL!\n");
+      return XEMALLOC;
+    }
+
   if (mytls_vars->tlsoutsize==0) 
     {
       if (indata != NULL)
@@ -346,7 +395,10 @@ int tls_funcs_parse(struct generic_eap_data *thisint, u_char *indata,
 	      tls_funcs_process_error();
 	    }
 
-	  BIO_write(mytls_vars->ssl_in, indata, insize);
+	  if (BIO_write(mytls_vars->ssl_in, indata, insize) < 2)
+	    {
+	      tls_funcs_process_error();
+	    }
 	} 
       if (BIO_reset(mytls_vars->ssl_out) < 0)
 	{
@@ -373,6 +425,8 @@ int tls_funcs_parse(struct generic_eap_data *thisint, u_char *indata,
   
   if (mytls_vars->tlsoutsize == 0) 
     {
+      debug_printf(DEBUG_AUTHTYPES, "No data returned!\n");
+
       return XTLSNEEDDATA;
     }
 
@@ -445,7 +499,7 @@ int tls_funcs_decode_packet(struct generic_eap_data *thisint, char *inframe,
     }
 
   debug_printf(DEBUG_AUTHTYPES, "Packet in (%d) :\n", insize);
-  debug_hex_dump(DEBUG_AUTHTYPES, inframe, insize);
+  debug_hex_dump(DEBUG_AUTHTYPES, (u_char *) inframe, insize);
 
   // First, make sure we don't have any errors.
   err = ERR_get_error();
@@ -475,7 +529,11 @@ int tls_funcs_decode_packet(struct generic_eap_data *thisint, char *inframe,
   switch ((uint8_t)inframe[0])
     {
     case EAPTLS_START:
-      tls_funcs_start(mytls_vars);
+      if (tls_funcs_start(mytls_vars) != XENONE)
+	{
+	  debug_printf(DEBUG_NORMAL, "There was an error starting the TLS "
+		       "handshake!\n");
+	}
 
       if (mytls_vars->ssl == NULL)
 	{
@@ -501,6 +559,8 @@ int tls_funcs_decode_packet(struct generic_eap_data *thisint, char *inframe,
 
     case EAPTLS_MORE_FRAGS:
     case EAPTLS_ACK:
+      if (!mytls_vars) printf("mytls_vars is hosed!\n");
+      if (!mytls_vars->ssl) printf("mytls_vars->ssl is hosed!\n");
       if ((SSL_get_state(mytls_vars->ssl) == 0x0003) && (dophase2 != NULL))
 	{
 	  // Handle the phase 2 piece.  We pass in the encrypted piece of
@@ -563,7 +623,7 @@ int tls_funcs_decode_packet(struct generic_eap_data *thisint, char *inframe,
 	  
 	  if ((mytls_vars->resuming != 1) || (mytls_vars->quickResponse != TRUE))
 	    {
-	      if (((*dophase2)(thisint, tlsptr, (insize-tlsindex), outframe, 
+	      if (((*dophase2)(thisint, (u_char *) tlsptr, (insize-tlsindex), outframe, 
 			       outsize)) != XENONE)
 		{
 		  debug_printf(DEBUG_NORMAL, "Phase 2 failure!\n");
@@ -578,7 +638,7 @@ int tls_funcs_decode_packet(struct generic_eap_data *thisint, char *inframe,
 		}
 	    }	  
 	} else {
-	  rtnVal = tls_funcs_parse(thisint, tlsptr, (insize-tlsindex), outframe, outsize, chunksize);
+	  rtnVal = tls_funcs_parse(thisint, (u_char *) tlsptr, (insize-tlsindex), outframe, outsize, chunksize);
 	  if (rtnVal < 0)
 	    {
 	      debug_printf(DEBUG_NORMAL, "Couldn't parse TLS data.\n");
@@ -642,7 +702,7 @@ int tls_funcs_decode_packet(struct generic_eap_data *thisint, char *inframe,
 
 	      if ((mytls_vars->resuming != 1) || (mytls_vars->quickResponse != TRUE))
 		{
-		  if ((*dophase2)(thisint, tlsptr, (insize-tlsindex), 
+		  if ((*dophase2)(thisint, (u_char *) tlsptr, (insize-tlsindex), 
 				  outframe, outsize) != XENONE)
 		    {
 		      debug_printf(DEBUG_NORMAL, "Phase 2 Failure.\n");
@@ -731,12 +791,6 @@ int tls_funcs_load_root_certs(struct generic_eap_data *thisint,
       return XEMALLOC;
     }
 
-  if ((!root_cert) && (!root_dir))
-    {
-      debug_printf(DEBUG_NORMAL, "Invalid data passed in to tls_funcs_load_root_certs()!\n");
-      return XEMALLOC;
-    }
-
   mytls_vars = (struct tls_vars *)thisint->eap_data;
 
   if (!mytls_vars)
@@ -745,13 +799,11 @@ int tls_funcs_load_root_certs(struct generic_eap_data *thisint,
       return XEMALLOC;
     }
 
-  if (root_cert == NULL)
+  if ((!root_cert) && (!root_dir))
     {
       debug_printf(DEBUG_NORMAL, "Error loading cert!  Path to cert is NULL!\n");
       return XETLSCERTLOAD;
-    } else {
-      debug_printf(DEBUG_CONFIG, "Loading certificate %s . . . \n", root_cert);
-    }
+    } 
 
   if (mytls_vars->ctx == NULL)
     {
@@ -759,11 +811,14 @@ int tls_funcs_load_root_certs(struct generic_eap_data *thisint,
       return XEMALLOC;
     }
 
+  debug_printf(DEBUG_CONFIG, "Trying to load root certificate %s or "
+	       "certificate directory %s\n", root_cert, root_dir);
+
   SSL_CTX_set_info_callback(mytls_vars->ctx, (void (*) ()) ssl_info_callback);
   
   if (SSL_CTX_load_verify_locations(mytls_vars->ctx, root_cert, root_dir) == 0)
     {
-      debug_printf(DEBUG_NORMAL, "Failed to initalize path to root certificate!\n");
+      debug_printf(DEBUG_NORMAL, "Failed to initialize path to root certificate!\n");
       tls_funcs_process_error();
 
       if(mytls_vars->ctx)
@@ -774,7 +829,7 @@ int tls_funcs_load_root_certs(struct generic_eap_data *thisint,
       return XETLSCERTLOAD;
     }
 
-  debug_printf(DEBUG_CONFIG, "Loaded root certificate %s and dirctory %s\n",
+  debug_printf(DEBUG_CONFIG, "Loaded root certificate %s and directory %s\n",
 		root_cert, root_dir);
 
   if (crl_dir) {
@@ -798,7 +853,7 @@ int tls_funcs_load_root_certs(struct generic_eap_data *thisint,
     {
       debug_printf(DEBUG_NORMAL, "Failed to initalize default paths for root certificates!\n");
       tls_funcs_process_error();
-      //debug_printf(DEBUG_NORMAL, "Error : %s\n", ERR_error_string(ERR_get_error(), NULL));
+
       if(mytls_vars->ctx)
 	{
 	  SSL_CTX_free(mytls_vars->ctx);
@@ -1083,32 +1138,32 @@ tls_funcs_PRF(u_char *secret, int secret_len, u_char *label, int label_len,
   /* P_MD5 */
   hash = EVP_md5();
   /* Initialize A_MD5 */
-  HMAC(hash, S1, L_S1, P_seed, P_seed_len, A_MD5, &hashed_len);
+  HMAC(hash, S1, L_S1, P_seed, P_seed_len, A_MD5, (u_int *) &hashed_len);
 
   for (i = 0; i < MD5_iterations; i++) {
     HMAC_Init(&ctx, S1, L_S1, hash);
     HMAC_Update(&ctx, A_MD5, MD5_DIGEST_LENGTH);
     HMAC_Update(&ctx, P_seed, P_seed_len);
-    HMAC_Final(&ctx, P_MD5_buf + i*(MD5_DIGEST_LENGTH), &hashed_len);
+    HMAC_Final(&ctx, P_MD5_buf + i*(MD5_DIGEST_LENGTH), (u_int *) &hashed_len);
     HMAC_cleanup(&ctx);
     HMAC(hash, S1, L_S1, A_MD5, MD5_DIGEST_LENGTH,
-	 A_MD5, &hashed_len);
+	 A_MD5, (u_int *) &hashed_len);
   }
     
 
   /* do P_SHA1 */
   hash = EVP_sha1();
   /* Initialize A_SHA1 */
-  HMAC(hash, S2, L_S2, P_seed, P_seed_len, A_SHA1, &hashed_len);
+  HMAC(hash, S2, L_S2, P_seed, P_seed_len, A_SHA1, (u_int *) &hashed_len);
 
   for (i = 0; i < SHA1_iterations; i++) {
     HMAC_Init(&ctx, S2, L_S2, hash);
     HMAC_Update(&ctx, A_SHA1, SHA_DIGEST_LENGTH);
     HMAC_Update(&ctx, P_seed, P_seed_len);
-    HMAC_Final(&ctx, P_SHA1_buf + i*(SHA_DIGEST_LENGTH), &hashed_len);
+    HMAC_Final(&ctx, P_SHA1_buf + i*(SHA_DIGEST_LENGTH), (u_int *) &hashed_len);
     HMAC_cleanup(&ctx);
     HMAC(hash, S2, L_S2, A_SHA1, SHA_DIGEST_LENGTH,
-	 A_SHA1, &hashed_len);
+	 A_SHA1, (u_int *) &hashed_len);
   }
   /* XOR Them for the answer */
   for (i = 0; i < outlen; i++) {
