@@ -7,9 +7,21 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: ttlsphase2.c,v 1.25 2005/08/09 01:39:18 chessing Exp $
- * $Date: 2005/08/09 01:39:18 $
+ * $Id: ttlsphase2.c,v 1.28 2006/01/03 04:02:36 chessing Exp $
+ * $Date: 2006/01/03 04:02:36 $
  * $Log: ttlsphase2.c,v $
+ * Revision 1.28  2006/01/03 04:02:36  chessing
+ * Added the ability to store the PEAP password in a hashed format.  (Basically, an MS-CHAPv1 hash.)  Also added an 'ntpwdhash' program to the tools directory that will convert a cleartext password in to a hash that can be copied to the configuration file.
+ *
+ * Revision 1.27  2005/10/17 03:56:55  chessing
+ * Updates to the libxsupconfig library.  It no longer relies on other source from the main tree, so it can be used safely in other code with problems.
+ *
+ * Revision 1.26  2005/10/14 02:26:18  shaftoe
+ * - cleanup gcc 4 warnings
+ * - (re)add support for a pid in the form of /var/run/xsupplicant.<iface>.pid
+ *
+ * -- Eric Evans <eevans@sym-link.com>
+ *
  * Revision 1.25  2005/08/09 01:39:18  chessing
  * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
@@ -25,7 +37,7 @@
 #include <netinet/in.h>
 #include <openssl/ssl.h>
 
-#include "config.h"
+#include "xsupconfig.h"
 #include "profile.h"
 #include "eap.h"
 #include "../tls/tls_crypt.h"
@@ -184,7 +196,7 @@ void ttls_do_mschapv2(struct generic_eap_data *thisint, char *out_data, int *out
   username_size = strlen(username);
 
   // Send the Username AVP
-  build_avp(USER_NAME_AVP, 0, MANDITORY_FLAG, username, username_size, out_data, &avp_out_size);
+  build_avp(USER_NAME_AVP, 0, MANDITORY_FLAG, (uint8_t *) username, username_size, (uint8_t *) out_data, &avp_out_size);
 
   avp_offset = avp_out_size;
 
@@ -200,7 +212,7 @@ void ttls_do_mschapv2(struct generic_eap_data *thisint, char *out_data, int *out
   id = challenge[16];
 
   // Send the MS-CHAP AVP
-  build_avp(MS_CHAP_CHALLENGE, MS_VENDOR_ATTR, (MANDITORY_FLAG | VENDOR_FLAG), (char *)&mschap_challenge, 16, &out_data[avp_offset], &avp_out_size);
+  build_avp(MS_CHAP_CHALLENGE, MS_VENDOR_ATTR, (MANDITORY_FLAG | VENDOR_FLAG), (uint8_t *) &mschap_challenge, 16, (uint8_t *) &out_data[avp_offset], &avp_out_size);
 
   avp_offset+=avp_out_size;
 
@@ -210,14 +222,14 @@ void ttls_do_mschapv2(struct generic_eap_data *thisint, char *out_data, int *out
   // The first 24 bytes should be left as 0s.
   password = phase2data->password;    // Get our password.
 
-  GenerateNTResponse((char *)&mschap_challenge, (char *)&mschap_challenge, username, password, (char *)&mschap_result);
+  GenerateNTResponse((char *)&mschap_challenge, (char *)&mschap_challenge, username, password, (char *)&mschap_result, 0);
 
   mschap_answer[0] = id;
   mschap_answer[1] = 0;
   memcpy(&mschap_answer[2], &mschap_challenge, 16);
   memcpy(&mschap_answer[26], &mschap_result, 24);
 
-  build_avp(MS_CHAP2_RESPONSE, MS_VENDOR_ATTR, (MANDITORY_FLAG | VENDOR_FLAG), (char *)&mschap_answer, 50, &out_data[avp_offset], &avp_out_size);
+  build_avp(MS_CHAP2_RESPONSE, MS_VENDOR_ATTR, (MANDITORY_FLAG | VENDOR_FLAG), (uint8_t *) &mschap_answer, 50, (uint8_t *) &out_data[avp_offset], &avp_out_size);
   avp_offset+=avp_out_size;
   *out_size = avp_offset;
 }
@@ -283,7 +295,7 @@ void ttls_do_mschap(struct generic_eap_data *thisint, char *out_data, int *out_s
   username_size = strlen(username);
 
   // Send the Username AVP
-  build_avp(USER_NAME_AVP, 0, MANDITORY_FLAG, username, username_size, out_data, &avp_out_size);
+  build_avp(USER_NAME_AVP, 0, MANDITORY_FLAG, (uint8_t *) username, username_size, (uint8_t *) out_data, &avp_out_size);
 
   avp_offset = avp_out_size;
 
@@ -299,7 +311,7 @@ void ttls_do_mschap(struct generic_eap_data *thisint, char *out_data, int *out_s
   id = challenge[8];
 
   // Send the MS-CHAP AVP
-  build_avp(MS_CHAP_CHALLENGE, MS_VENDOR_ATTR, (MANDITORY_FLAG | VENDOR_FLAG), (char *)&mschap_challenge, 8, &out_data[avp_offset], &avp_out_size);
+  build_avp(MS_CHAP_CHALLENGE, MS_VENDOR_ATTR, (MANDITORY_FLAG | VENDOR_FLAG), (uint8_t *) &mschap_challenge, 8, (uint8_t *) &out_data[avp_offset], &avp_out_size);
 
   avp_offset+=avp_out_size;
 
@@ -307,13 +319,13 @@ void ttls_do_mschap(struct generic_eap_data *thisint, char *out_data, int *out_s
 
   password = phase2data->password;    // Get our password.
 
-  NtChallengeResponse((char *)&mschap_challenge, password, (char *)&mschap_result);
+  NtChallengeResponse((char *)&mschap_challenge, password, (char *)&mschap_result, 0);
 
   mschap_answer[0] = id;
   mschap_answer[1] = 1; // Use NT Style Passwords.
   memcpy((char *)&mschap_answer[26], (char *)&mschap_result, 24);
 
-  build_avp(MS_CHAP_RESPONSE, MS_VENDOR_ATTR, (MANDITORY_FLAG | VENDOR_FLAG), (char *)&mschap_answer, 50, &out_data[avp_offset], &avp_out_size);
+  build_avp(MS_CHAP_RESPONSE, MS_VENDOR_ATTR, (MANDITORY_FLAG | VENDOR_FLAG), (uint8_t *) &mschap_answer, 50, (uint8_t *) &out_data[avp_offset], &avp_out_size);
   avp_offset+=avp_out_size;
 
   *out_size = avp_offset;
@@ -386,12 +398,12 @@ void ttls_do_chap(struct generic_eap_data *thisint, char *out_data, int *out_siz
       username = phase2data->username;
     }
   username_size = strlen(username);
-  build_avp(USER_NAME_AVP, 0, MANDITORY_FLAG, username, username_size, out_data, &avp_out_size);
+  build_avp(USER_NAME_AVP, 0, MANDITORY_FLAG, (uint8_t *) username, username_size, (uint8_t *) out_data, &avp_out_size);
 
   avp_offset = avp_out_size;
 
   // Get the implicit challenge.
-  challenge = implicit_challenge(thisint);
+  challenge = (u_char *) implicit_challenge(thisint);
   if (challenge == NULL)
     {
       debug_printf(DEBUG_NORMAL, "Invalid implicit challenge in ttls_do_chap()!\n");
@@ -409,9 +421,9 @@ void ttls_do_chap(struct generic_eap_data *thisint, char *out_data, int *out_siz
       return;
     }
 
-  user_passwd = phase2data->password;
+  user_passwd = (u_char *) phase2data->password;
 
-  tohash = (char *)malloc(1+16+strlen(user_passwd));
+  tohash = (u_char *)malloc(1+16+strlen((char *) user_passwd));
   if (tohash == NULL)
     {
       debug_printf(DEBUG_NORMAL, "Error with malloc of \"tohash\" in ttls_do_chap().\n");
@@ -419,13 +431,13 @@ void ttls_do_chap(struct generic_eap_data *thisint, char *out_data, int *out_siz
     }
 
   tohash[0] = session_id;
-  memcpy(&tohash[1], user_passwd, strlen(user_passwd));
-  memcpy(&tohash[1+strlen(user_passwd)], &chap_challenge, 16);
-  hashlen = 1+strlen(user_passwd)+16;
+  memcpy(&tohash[1], user_passwd, strlen((char *) user_passwd));
+  memcpy(&tohash[1+strlen((char *) user_passwd)], &chap_challenge, 16);
+  hashlen = 1+strlen((char *) user_passwd)+16;
 
   EVP_DigestInit(ctx, EVP_md5());
   EVP_DigestUpdate(ctx, tohash, hashlen);
-  EVP_DigestFinal(ctx, (char *)&chap_hash[1], (int *)&md5_length);
+  EVP_DigestFinal(ctx, (u_char *)&chap_hash[1], (u_int *)&md5_length);
   
   if (md5_length != 16)  // We didn't get back a valid hash!
     {
@@ -433,11 +445,11 @@ void ttls_do_chap(struct generic_eap_data *thisint, char *out_data, int *out_siz
     }
   chap_hash[0]=session_id;
 
-  build_avp(CHAP_PASSWORD_AVP, 0, MANDITORY_FLAG, chap_hash, 17, &out_data[avp_offset], &avp_out_size);
+  build_avp(CHAP_PASSWORD_AVP, 0, MANDITORY_FLAG, chap_hash, 17, (uint8_t *) &out_data[avp_offset], &avp_out_size);
 
   avp_offset += avp_out_size;
 
-  build_avp(CHAP_CHALLENGE_AVP, 0, MANDITORY_FLAG, (char *)&chap_challenge, 16, &out_data[avp_offset], &avp_out_size);
+  build_avp(CHAP_CHALLENGE_AVP, 0, MANDITORY_FLAG, (uint8_t *) &chap_challenge, 16, (uint8_t *) &out_data[avp_offset], &avp_out_size);
 
   if (tohash != NULL)
     {
@@ -518,8 +530,8 @@ void ttls_do_pap(struct generic_eap_data *thisint, char *out_data, int *out_size
 
   avp_offset = 0;
 
-  build_avp(USER_NAME_AVP, 0, MANDITORY_FLAG, username, 
-	    strlen(username), out_data, &avp_out_size);
+  build_avp(USER_NAME_AVP, 0, MANDITORY_FLAG, (uint8_t *) username, 
+	    strlen(username), (uint8_t *) out_data, &avp_out_size);
 
   avp_offset += avp_out_size;
 
@@ -537,7 +549,7 @@ void ttls_do_pap(struct generic_eap_data *thisint, char *out_data, int *out_size
   bzero(tempbuf, passwd_size);
   memcpy(tempbuf, phase2data->password, strlen(phase2data->password));
 
-  build_avp(USER_PASSWORD_AVP, 0, MANDITORY_FLAG, tempbuf, passwd_size, &out_data[avp_offset], &avp_out_size);
+  build_avp(USER_PASSWORD_AVP, 0, MANDITORY_FLAG, (uint8_t *) tempbuf, passwd_size, (uint8_t *) &out_data[avp_offset], &avp_out_size);
 
   *out_size = avp_offset + avp_out_size;
   
@@ -548,7 +560,7 @@ void ttls_do_pap(struct generic_eap_data *thisint, char *out_data, int *out_size
     }
 
   debug_printf(DEBUG_AUTHTYPES, "Returning from do_pap :\n");
-  debug_hex_dump(DEBUG_AUTHTYPES, out_data, *out_size);
+  debug_hex_dump(DEBUG_AUTHTYPES, (u_char *) out_data, *out_size);
 }
 
 // We don't do anything with the "in" stuff for now..
@@ -567,7 +579,7 @@ int ttls_do_phase2(struct generic_eap_data *thisint, char *in, int in_size, char
     }
 
   debug_printf(DEBUG_AUTHTYPES, "Encrypted Inner (%d) : \n", in_size);
-  debug_hex_dump(DEBUG_AUTHTYPES, in, in_size);
+  debug_hex_dump(DEBUG_AUTHTYPES, (u_char *) in, in_size);
 
   userdata = (struct config_eap_ttls *)thisint->eap_conf_data;
 
@@ -597,10 +609,10 @@ int ttls_do_phase2(struct generic_eap_data *thisint, char *in, int in_size, char
   if ((in_size > 0) && (in[0] != 0x14))
     {
       // We have something to decrypt!
-      tls_crypt_decrypt(thisint, in, in_size, decr_data, &decrsize);
+      tls_crypt_decrypt(thisint, (u_char *) in, in_size, (u_char *) decr_data, &decrsize);
 
       debug_printf(DEBUG_AUTHTYPES, "Decrypted Inner (%d) : \n", in_size);
-      debug_hex_dump(DEBUG_AUTHTYPES, decr_data, decrsize);
+      debug_hex_dump(DEBUG_AUTHTYPES, (u_char *) decr_data, decrsize);
 
       if (decr_data[0] == 0x00)
 	{
@@ -638,11 +650,11 @@ int ttls_do_phase2(struct generic_eap_data *thisint, char *in, int in_size, char
       return XENONE;
     }
 
-  tls_crypt_encrypt_nolen(thisint, toencout, toencsize, out, out_size);
+  tls_crypt_encrypt_nolen(thisint, (u_char *) toencout, toencsize, (u_char *) out, out_size);
   free(toencout);
 
   debug_printf(DEBUG_AUTHTYPES, "Returning from (TTLS) do_phase2 : \n");
-  debug_hex_dump(DEBUG_AUTHTYPES, out, *out_size);
+  debug_hex_dump(DEBUG_AUTHTYPES, (u_char *) out, *out_size);
   return XENONE;
 }
 
