@@ -7,9 +7,42 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: eapol_key_type2.c,v 1.12 2005/08/09 01:39:14 chessing Exp $
- * $Date: 2005/08/09 01:39:14 $
+ * $Id: eapol_key_type2.c,v 1.22 2006/01/24 04:42:26 chessing Exp $
+ * $Date: 2006/01/24 04:42:26 $
  * $Log: eapol_key_type2.c,v $
+ * Revision 1.22  2006/01/24 04:42:26  chessing
+ * A few more fixes to WPA code, along with a fix to the scan reaping code.
+ *
+ * Revision 1.21  2006/01/23 20:33:47  chessing
+ * Added support for the replay counters in WPA/WPA2.  BSSID is now selected based on the signal quality information returned during the scan.  (Assuming signal quality information is returned during a scan. ;)
+ *
+ * Revision 1.20  2006/01/23 07:13:49  chessing
+ * A couple of fixups to the WPA and WPA2 code.
+ *
+ * Revision 1.19  2006/01/23 05:28:37  chessing
+ * Fixed a few settings that were causing errors with IOCTLs on some cards.  Updated WPA2 code to properly process group key packets. We now record quality, signal level, and noise level from scan results, so that we can start to make better decisions on which AP to associate to.
+ *
+ * Revision 1.18  2006/01/19 05:39:52  chessing
+ * One more small fix.
+ *
+ * Revision 1.17  2006/01/19 05:37:04  chessing
+ * WPA2 is working correctly.  Added the ability to query the card to gather encryption/authentication capabilities.  1.2.3 is now ready to go.
+ *
+ * Revision 1.16  2006/01/15 06:56:29  chessing
+ * Some fixes to WPA/WPA2.  WPA is working correctly now.  WPA2 needs a little work to correctly identify key frames that are for the GTK.
+ *
+ * Revision 1.15  2005/10/17 03:56:53  chessing
+ * Updates to the libxsupconfig library.  It no longer relies on other source from the main tree, so it can be used safely in other code with problems.
+ *
+ * Revision 1.14  2005/10/14 02:26:17  shaftoe
+ * - cleanup gcc 4 warnings
+ * - (re)add support for a pid in the form of /var/run/xsupplicant.<iface>.pid
+ *
+ * -- Eric Evans <eevans@sym-link.com>
+ *
+ * Revision 1.13  2005/09/05 01:00:34  chessing
+ * Major overhaul to most of the state machines in Xsupplicant.  Also added additional error messages to the TLS functions to try to debug the one of the problems reported on the list.  Basic testing shows this new code to be more stable than previous code, but it needs more testing.
+ *
  * Revision 1.12  2005/08/09 01:39:14  chessing
  * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
@@ -22,7 +55,7 @@
 #include "xsup_debug.h"
 #include "xsup_err.h"
 #include "profile.h"
-#include "config.h"
+#include "xsupconfig.h"
 #include "key_statemachine.h"
 #include "eapol_key_type2.h"
 #include "eapol.h"
@@ -33,6 +66,7 @@
 #include "psk.h"
 #include "mic.h"
 #include "snmp.h"
+#include "config_ssid.h"
 #include "cardif/cardif.h"
 #include "eap_types/mschapv2/mschapv2.h"
 
@@ -51,7 +85,7 @@ void eapol_key_type2_dump(struct interface_data *intdata, char *framedata)
   uint16_t keylen, version = 0;
   struct wpa2_key_packet *keydata;
   uint8_t *keypayload;
-  u_char rc4_ek[32], key[16];
+  u_char rc4_ek[32], key[1500];
 
   keydata = (struct wpa2_key_packet *)&framedata[OFFSET_TO_EAPOL+4];
 
@@ -170,10 +204,14 @@ void eapol_key_type2_dump(struct interface_data *intdata, char *framedata)
 	  break;
 	  
 	case 2:
-	  // XXX BROKEN!  FIX!
 	  bzero(key, 16);
-	  aes_unwrap(&intdata->statemachine->PTK[16], (keylen-8)/8, keypayload,
-		     key);
+	  if (aes_unwrap((u_char *) &intdata->statemachine->PTK[16], 
+			 (keylen-8)/8, keypayload, key) != 0)
+	    {
+	      debug_printf(DEBUG_NORMAL, "Failed AES unwrap. (Data will be"
+			   " invalid!)\n");
+	    }
+	  keypayload = key;
 	  break;
 	  
 	default:
@@ -204,30 +242,36 @@ char *eapol_key_type2_gen_ptk(struct interface_data *intdata, char *Anonce)
     {
       memcpy((char *)&prfdata[0], (char *)&intdata->source_mac, 6);
       memcpy((char *)&prfdata[6], (char *)&intdata->dest_mac, 6);
-    } else if (memcmp((char *)&intdata->source_mac, (char *)&intdata->dest_mac,
-		      6) > 0)
-      {
-	memcpy((char *)&prfdata[0], (char *)&intdata->dest_mac, 6);
-	memcpy((char *)&prfdata[6], (char *)&intdata->source_mac, 6);
-      } else {
-	debug_printf(DEBUG_NORMAL, "Source and Destination MAC addresses "
-		     "match!  The PTK won't be valid!\n");
-	return NULL;
-      }
+    } 
+  else if (memcmp((char *)&intdata->source_mac, (char *)&intdata->dest_mac,
+		  6) > 0)
+    {
+      memcpy((char *)&prfdata[0], (char *)&intdata->dest_mac, 6);
+      memcpy((char *)&prfdata[6], (char *)&intdata->source_mac, 6);
+    } 
+  else 
+    {
+      debug_printf(DEBUG_NORMAL, "Source and Destination MAC addresses "
+		   "match!  The PTK won't be valid!\n");
+      return NULL;
+    }
 
   if (memcmp(intdata->statemachine->SNonce, Anonce, 32) < 0)
     {
       memcpy((char *)&prfdata[12], intdata->statemachine->SNonce, 32);
       memcpy((char *)&prfdata[44], Anonce, 32);
-    } else if (memcmp(intdata->statemachine->SNonce, Anonce, 32) > 0)
-      {
-	memcpy((char *)&prfdata[12], Anonce, 32);
-	memcpy((char *)&prfdata[44], intdata->statemachine->SNonce, 32);
-      } else {
-	debug_printf(DEBUG_NORMAL, "ANonce and SNonce match!  The PTK won't"
-		     " be valid!\n");
-	return NULL;
-      }
+    } 
+  else if (memcmp(intdata->statemachine->SNonce, Anonce, 32) > 0)
+    {
+      memcpy((char *)&prfdata[12], Anonce, 32);
+      memcpy((char *)&prfdata[44], intdata->statemachine->SNonce, 32);
+    } 
+  else 
+    {
+      debug_printf(DEBUG_NORMAL, "ANonce and SNonce match!  The PTK won't"
+		   " be valid!\n");
+      return NULL;
+    }
   
   retval = (char *)malloc(80);
   if (retval == NULL)
@@ -238,12 +282,12 @@ char *eapol_key_type2_gen_ptk(struct interface_data *intdata, char *Anonce)
     }
 
   debug_printf(DEBUG_INT, "PMK : ");
-  debug_hex_printf(DEBUG_INT, intdata->statemachine->PMK, 32);
-  wpa_PRF(intdata->statemachine->PMK, 32, "Pairwise key expansion", 22,
-	  (char *)&prfdata, 76, retval, 64);
+  debug_hex_printf(DEBUG_INT, (u_char *) intdata->statemachine->PMK, 32);
+  wpa_PRF((u_char *) intdata->statemachine->PMK, 32, (u_char *) "Pairwise key expansion", 22,
+	  (u_char *)&prfdata, 76, (u_char *) retval, 64);
 
   debug_printf(DEBUG_INT, "PTK : ");
-  debug_hex_printf(DEBUG_INT, retval, 64);
+  debug_hex_printf(DEBUG_INT, (u_char *) retval, 64);
 
   return retval;
 }
@@ -300,25 +344,261 @@ void eapol_key_type2_request_new_key(struct interface_data *intdata,
   len = sizeof(struct wpa2_key_packet);
   intdata->send_size = len+OFFSET_TO_EAPOL+4;
 
-  eapolver = network_data->force_eapol_ver;
-
-  // If we have not hard set the eapol version, then use the last
-  // version that the authenticator gave us.  This isn't in line with
-  // the 802.1X-REV-2004-d11 standard, but many authenticators are checking
-  // version #s, and breaking when it isn't 1. :-/
-  if ((eapolver < 1) || (eapolver > MAX_EAPOL_VER))
-    eapolver = snmp_get_dot1xSuppLastEapolFrameVersion();
+  eapolver = eapol_get_eapol_ver();
 
   eapol_build_header(EAPOL_KEY, (intdata->send_size-OFFSET_TO_EAPOL-4), 
-		     eapolver, intdata->sendframe); 
+		     eapolver, (char *) intdata->sendframe); 
   
   memcpy(key, intdata->statemachine->PTK, 16);
-  mic_wpa_populate(intdata->sendframe, intdata->send_size+4, key, 16);
+  mic_wpa_populate((char *) intdata->sendframe, intdata->send_size+4, key, 16);
 
   cardif_sendframe(intdata);
   intdata->statemachine->eapolEap = FALSE;
 }
 
+/*********************************************************************
+ *
+ * Process a GTK KDE, and set the keys as needed.
+ *
+ *********************************************************************/
+char eapol_key_type2_do_gtk_kde(struct interface_data *intdata, uint8_t *key, 
+				uint16_t kdelen, uint8_t keylen, 
+				uint8_t *keyrsc, char version)
+{
+  char keyindex = 0, txkey = 0;
+
+  debug_printf(DEBUG_INT, "KDE : \n");
+  debug_hex_dump(DEBUG_INT, key, kdelen);
+  
+  keyindex = WPA2_KEY_INDEX_MASK & key[0];
+  
+  if (WPA2_TX_KEY_MASK & key[0]) txkey = 1;
+  
+  kdelen -= 2;
+  key += 2;
+
+  debug_printf(DEBUG_INT, "Setting GTK of length %d with index of %d.\n", 
+	       keylen, keyindex);
+
+  debug_printf(DEBUG_INT, "Key is : ");
+  debug_hex_printf(DEBUG_INT, key, keylen);
+  
+  switch (version)
+    {
+    case 1:
+      wpa_common_swap_rx_tx_mic(key);
+
+      cardif_set_tkip_key(intdata, NULL, keyindex+1, txkey, keyrsc, RSC_LEN, 
+			  key, keylen);
+      break;
+
+    case 2:
+      cardif_set_ccmp_key(intdata, NULL, keyindex+1, txkey, keyrsc, RSC_LEN, 
+			  key, keylen);
+      break;
+
+    default:
+      debug_printf(DEBUG_NORMAL, "Unknown encryption/MAC type %d.\n", version);
+      break;
+    }
+  return XENONE;
+}
+
+/*********************************************************************
+ *
+ * Process an STAKey KDE and set keys as needed.
+ *
+ *********************************************************************/
+char eapol_key_type2_do_stakey_kde(struct interface_data *intdata, 
+				   uint8_t *key, uint16_t keylen, 
+				   uint8_t *keyrsc, char version)
+{
+  debug_printf(DEBUG_NORMAL, "STAkeys are not supported at this time! "
+	       "(Ignoring)\n");
+  return XENONE;
+}
+
+/*********************************************************************
+ *
+ * Process a MAC address KDE.
+ *
+ *********************************************************************/
+char eapol_key_type2_do_mac_kde(struct interface_data *intdata, uint8_t *key,
+				uint16_t keylen, char version)
+{
+  debug_printf(DEBUG_NORMAL, "MAC KDEs are not supported at this time! "
+	       "(Ignoring)\n");
+  return XENONE;
+}
+
+/*********************************************************************
+ *
+ * Process a PMKID KDE.
+ *
+ *********************************************************************/
+char eapol_key_type2_do_pmkid_kde(struct interface_data *intdata,
+				  uint8_t *key, uint16_t keylen, char version)
+{
+  return XENONE;
+}
+
+/********************************************************
+ *
+ * Given the IE from the decrypted data, verify that it matches what
+ * the AP originally told us.  If not, we should fail, and send a deauth.
+ *
+ ********************************************************/
+int eapol_key_type2_cmp_ie(struct interface_data *intdata, u_char *iedata,
+			   int len)
+{
+  u_char *apie, ielen;
+
+  if (iedata[0] != WPA2_EID)
+    {
+      debug_printf(DEBUG_NORMAL, "Data presented doesn't appear to be a valid"
+		   " WPA2/802.11i IE!\n");
+      return -1;
+    }
+
+  debug_printf(DEBUG_INT, "Got an IE of        : \n");
+  debug_hex_dump(DEBUG_INT, (u_char *) iedata, len);
+
+  config_ssid_get_rsn_ie(&apie, &ielen);
+  debug_printf(DEBUG_INT, "AP sent us an IE of : \n");
+  debug_hex_dump(DEBUG_INT, (u_char *) apie, ielen);
+
+  if (len != ielen)
+    {
+      debug_printf(DEBUG_NORMAL, "The length of the IE from the AP and IE in"
+		   " key messages were different!  IEs are invalid!\n");
+      free(apie);
+      return -1;
+    }
+
+  if (memcmp(apie, iedata, len) != 0)
+    {
+      debug_printf(DEBUG_NORMAL, "The IE from the AP and the IE in the key "
+		   "messages don't match!\n");
+      free(apie);
+      return -1;
+    }
+
+  return XENONE;
+}
+
+/*********************************************************************
+ *
+ * Given a string of bytes, go through it, and parse the information looking
+ * for different Key Data Encapsulations (KDEs).  Depending on the KDE that
+ * we find, we will pass the data on for further processing.
+ *
+ *********************************************************************/
+char eapol_key_type2_process_keydata(struct interface_data *intdata, 
+				     u_char *keydata, uint16_t len, 
+				     uint8_t keylen, uint8_t *keyrsc, 
+				     char version)
+{
+  uint8_t kdeval[3] = {0x00, 0x0f, 0xac};
+  int i = 0, done = 0;
+  uint8_t *p, kdelen;
+
+  while ((i < len) && (!done))
+    {
+      if (keydata[i] == 0x30)
+	{
+	  // We are testing the IE to see if it matches what the AP gave us.
+	  if (eapol_key_type2_cmp_ie(intdata, &keydata[i], (keydata[i+1]+2))
+	      != XENONE)
+	    {
+	      debug_printf(DEBUG_NORMAL, "IE presented by the AP doesn't "
+			   "match the IE in the key message!  Assuming "
+			   "an active attack is in progress!\n");
+	      cardif_disassociate(intdata, DISASSOC_INVALID_IE);
+	      return -1;
+	    }
+	}
+
+      if (keydata[i] == WPA2_EXTENDED_KEY_DATA)
+	{
+	  kdelen = keydata[i+1];
+	  if (kdelen == 0)
+	    {
+	      debug_printf(DEBUG_INT, "Remaining KDE is padding.\n");
+	      return XENONE;
+	    }
+
+	  if (memcmp(&keydata[i+2], kdeval, 3) == 0)
+	    {
+	      // This isn't a key type that we know how to deal with,
+	      // ignore it.
+	      p = (uint8_t *)&keydata[i+6];
+
+	      switch (keydata[i+5])
+		{
+		case 0:
+		  debug_printf(DEBUG_INT, "KDE : Reserved\n");
+		  break;
+		  
+		case WPA2_EXTENDED_GTK_KEY:
+		  debug_printf(DEBUG_INT, "KDE : GTK KDE\n");
+		  if (eapol_key_type2_do_gtk_kde(intdata, p, kdelen-4, keylen, 
+						 keyrsc, version) != XENONE)
+		    {
+		      debug_printf(DEBUG_NORMAL, "Couldn't set the GTK! "
+				   "Aborting the connection!\n");
+		      return -1;
+		    }
+		  break;
+		  
+		case WPA2_EXTENDED_STA_KEY:
+		  debug_printf(DEBUG_INT, "KDE : STAKey KDE\n");
+		  if (eapol_key_type2_do_stakey_kde(intdata, p, kdelen-4, 
+						    keyrsc, version) != XENONE)
+		    {
+		      debug_printf(DEBUG_NORMAL, "Couldn't set STAKey!\n");
+		      return -1;
+		    }
+		  break;
+		  
+		case WPA2_EXTENDED_MAC_ADDRESS:
+		  debug_printf(DEBUG_INT, "KDE : MAC address KDE\n");
+		  if (eapol_key_type2_do_mac_kde(intdata, p, kdelen-4, 
+						 version) != XENONE)
+		    {
+		      debug_printf(DEBUG_NORMAL, "Couldn't processing MAC "
+				   "address KDE!\n");
+		      return -1;
+		    }
+		  break;
+		  
+		case WPA2_EXTENDED_PMKID:
+		  debug_printf(DEBUG_INT, "KDE : PMKID KDE\n");
+		  if (eapol_key_type2_do_pmkid_kde(intdata, p, kdelen-4, 
+						   version) != XENONE)
+		    {
+		      debug_printf(DEBUG_NORMAL, "Couldn't process PMKID!\n");
+		      return -1;
+		    }
+		  break;
+		  
+		default:
+		  debug_printf(DEBUG_INT, "KDE : Unknown\n");
+		  break;
+		}
+	    }
+	  else 
+	    {
+	      debug_printf(DEBUG_NORMAL, "KDE contained an OUI that we don't"
+			   " understand!\n");
+	    }
+	}
+      i += (keydata[i+1] + 2);
+    }
+  i++;
+
+  return XENONE;
+}
+
 /****************************************************************
  *
  * When we have completed the PTK piece, and the pairwise key has been
@@ -331,11 +611,11 @@ void eapol_key_type2_request_new_key(struct interface_data *intdata,
 void eapol_key_type2_do_gtk(struct interface_data *intdata)
 {
   struct wpa2_key_packet *inkeydata, *outkeydata;
-  uint16_t value16, keyflags, version, keyindex = 0, len;
+  uint16_t value16, keyflags, version, len;
   int eapolver;
   unsigned char *keydata;
   char key[16], rc4_ek[32];
-  u_char allfs[6] = {0xff,0xff,0xff,0xff,0xff,0xff};
+  char zeros[8] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
   struct config_network *network_data;
 
   network_data = config_get_network_config();
@@ -350,6 +630,17 @@ void eapol_key_type2_do_gtk(struct interface_data *intdata)
   inkeydata = (struct wpa2_key_packet *)&intdata->recvframe[OFFSET_TO_EAPOL+4];
   outkeydata = (struct wpa2_key_packet *)&intdata->sendframe[OFFSET_TO_EAPOL+4];
 
+  // First, make sure that the inkeydata replay counter is higher than
+  // the last counter we saw.
+  if ((memcmp(inkeydata->key_replay_counter, 
+	      intdata->statemachine->replay_counter, 8) <= 0) && 
+      (memcmp(inkeydata->key_replay_counter, zeros, 8) != 0))
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid replay counter!  Discarding!\n");
+      intdata->recv_size = 0;
+      return;
+    }
+
   // Clear everything out.
   bzero(&intdata->sendframe[OFFSET_TO_EAPOL+4], sizeof(struct wpa2_key_packet));
 
@@ -375,7 +666,7 @@ void eapol_key_type2_do_gtk(struct interface_data *intdata)
 
   value16 = ntohs(inkeydata->key_material_len);
   
-  keydata = (unsigned char *)malloc(value16);
+  keydata = (unsigned char *)malloc(value16+8);
   if (keydata == NULL)
     {
       debug_printf(DEBUG_NORMAL, "Couldn't allocate memory for key data!\n");
@@ -394,34 +685,39 @@ void eapol_key_type2_do_gtk(struct interface_data *intdata)
       bzero(rc4_ek, 32);
       memcpy(rc4_ek, inkeydata->key_iv, 16);
       memcpy(&rc4_ek[16], &intdata->statemachine->PTK[16], 16);
-      rc4_skip(rc4_ek, 32, 256, keydata, value16);
+      rc4_skip((u_char *) rc4_ek, 32, 256, keydata, value16);
 
       debug_printf(DEBUG_INT, "Decrypted data : (%d)\n", value16);
       debug_hex_dump(DEBUG_INT, keydata, value16);
 
-      // Grab our index, and scoot ahead to the actual key data.
-      keyindex = keydata[6];
-      keydata += 8;
-      value16 -= 8;
-
-      debug_printf(DEBUG_INT, "Key : \n");
-      debug_hex_dump(DEBUG_INT, keydata, value16);
-
-      wpa_common_swap_rx_tx_mic(keydata);
-
-      debug_printf(DEBUG_INT, "Setting GTK with index of %d\n", keyindex);
-      cardif_set_tkip_key(intdata, allfs, keyindex, FALSE, inkeydata->key_rsc,
-			  6, keydata, value16);
+      if (eapol_key_type2_process_keydata(intdata, keydata, value16, 
+					  ntohs(inkeydata->key_length),
+					  inkeydata->key_rsc, version) 
+	  != XENONE)
+	{
+	  return;
+	}
       break;
 
     case 2:
       // First, decrypt the GTK
       bzero(key, 16);
-      aes_unwrap(&intdata->statemachine->PTK[16], (value16-8)/8, keydata,
-		 key);
+      if (aes_unwrap((u_char *) &intdata->statemachine->PTK[16], 
+		     (value16-8)/8, keydata, (u_char *) key) != 0)
+      {
+	debug_printf(DEBUG_NORMAL, "Failed AES unwrap.\n");
+      }
 
-      cardif_set_ccmp_key(intdata, NULL, keyindex, FALSE, inkeydata->key_rsc,
-     			  6, key, (value16-8));
+      debug_printf(DEBUG_INT, "Result : ");
+      debug_hex_printf(DEBUG_INT, key, value16);
+
+      if (eapol_key_type2_process_keydata(intdata, key, value16, 
+					  ntohs(inkeydata->key_length),
+					  inkeydata->key_rsc, version)
+	  != XENONE)
+	{
+	  return;
+	}
       break;
     }
 
@@ -436,23 +732,16 @@ void eapol_key_type2_do_gtk(struct interface_data *intdata)
   outkeydata->key_length = inkeydata->key_length;
   memcpy(&outkeydata->key_replay_counter, &inkeydata->key_replay_counter, 8);
 
-  eapolver = network_data->force_eapol_ver;
-
-  // If we have not hard set the eapol version, then use the last
-  // version that the authenticator gave us.  This isn't in line with
-  // the 802.1X-REV-2004-d11 standard, but many authenticators are checking
-  // version #s, and breaking when it isn't 1. :-/
-  if ((eapolver < 1) || (eapolver > MAX_EAPOL_VER))
-    eapolver = snmp_get_dot1xSuppLastEapolFrameVersion();
+  eapolver = eapol_get_eapol_ver();
 
   eapol_build_header(EAPOL_KEY, (intdata->send_size-OFFSET_TO_EAPOL-4), 
-		     eapolver, intdata->sendframe); 
+		     eapolver, (char *) intdata->sendframe); 
   
   memcpy(key, intdata->statemachine->PTK, 16);
-  mic_wpa_populate(intdata->sendframe, intdata->send_size+4, key, 16);
+  mic_wpa_populate((char *) intdata->sendframe, intdata->send_size+4, key, 16);
 
   // Dump what we built.
-  eapol_key_type2_dump(intdata, intdata->sendframe);
+  eapol_key_type2_dump(intdata, (char *) intdata->sendframe);
 
   if (network_data->methods->method_num == WPA_PSK)
     {
@@ -478,6 +767,7 @@ void eapol_key_type2_do_type1(struct interface_data *intdata)
   int i, version, eapolver, ielen;
   char key[16];
   char wpa_ie[26];
+  char zeros[8] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
   struct config_network *network_data;
 
   network_data = config_get_network_config();
@@ -492,6 +782,15 @@ void eapol_key_type2_do_type1(struct interface_data *intdata)
   inkeydata = (struct wpa2_key_packet *)&intdata->recvframe[OFFSET_TO_EAPOL+4];
   outkeydata = (struct wpa2_key_packet *)&intdata->sendframe[OFFSET_TO_EAPOL+4];
 
+  if ((memcmp(inkeydata->key_replay_counter, 
+	      intdata->statemachine->replay_counter, 8) <= 0) && 
+      (memcmp(inkeydata->key_replay_counter, zeros, 8) != 0))
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid replay counter!  Discarding!\n");
+      intdata->recv_size = 0;
+      return;
+    }
+
   // Clear everything out.
   bzero(&intdata->sendframe[OFFSET_TO_EAPOL+4], sizeof(struct wpa2_key_packet));
 
@@ -550,247 +849,16 @@ void eapol_key_type2_do_type1(struct interface_data *intdata)
 
   outkeydata->key_material_len = value16;
 
-  eapolver = network_data->force_eapol_ver;
-
-  // If we have not hard set the eapol version, then use the last
-  // version that the authenticator gave us.  This isn't in line with
-  // the 802.1X-REV-2004-d11 standard, but many authenticators are checking
-  // version #s, and breaking when it isn't 1. :-/
-  if ((eapolver < 1) || (eapolver > MAX_EAPOL_VER))
-    eapolver = snmp_get_dot1xSuppLastEapolFrameVersion();
+  eapolver = eapol_get_eapol_ver();
 
   eapol_build_header(EAPOL_KEY, (intdata->send_size-OFFSET_TO_EAPOL-4), 
-		     eapolver, intdata->sendframe);
+		     eapolver, (char *) intdata->sendframe);
 
   memcpy(key, intdata->statemachine->PTK, 16);
-  mic_wpa_populate(intdata->sendframe, intdata->send_size+4, key, 16);
+  mic_wpa_populate((char *) intdata->sendframe, intdata->send_size+4, key, 16);
 
   // Dump what we built.
-  eapol_key_type2_dump(intdata, intdata->sendframe);
-}
-
-/********************************************************
- *
- * Given the IE from the decrypted data, verify that it matches what
- * the AP originally told us.  If not, we should fail, and send a deauth.
- *
- ********************************************************/
-int eapol_key_type2_cmp_ie(struct interface_data *intdata, u_char *indata,
-			   int *offs)
-{
-  char iedata[MAX_IE_LEN];
-  int len;
-
-  bzero(iedata, MAX_IE_LEN);
-
-  if (indata[*offs] != WPA2_EID)
-    {
-      debug_printf(DEBUG_NORMAL, "Data presented doesn't appear to be a valid"
-		   " WPA2/802.11i IE!\n");
-      return -1;
-    }
-
-  len = indata[(*offs)+1]+2;
-
-  // Otherwise, grab offs+1 and memcpy that +2 in to iedata.
-  memcpy(iedata, indata, len);
-
-  debug_printf(DEBUG_INT, "Got an IE of : \n");
-  debug_hex_dump(DEBUG_INT, iedata, len);
-
-  // XXX Todo : Verify the IE against the IE we have from the AP.
-
-  // Then, advance offs to the next section.
-  *offs += len;
-
-  return XENONE;
-}
-
-/********************************************************
- *
- * Parse a secondary IE.
- *
- ********************************************************/
-int eapol_key_type2_parse_extended(struct interface_data *intdata, 
-				    u_char *indata, int ielen, int *offs)
-{
-  // XXX These oids should go in a master OID .h file!
-  u_char valid_oid[3] = {0x00, 0x0f, 0xac};
-  u_char wpaoui[3] = {0x00, 0x50, 0xf2};
-
-  /*
-  debug_printf(DEBUG_NORMAL, "Data : ");
-  debug_hex_printf(DEBUG_NORMAL, &indata[((*offs)-3)], 6);
-  debug_printf(DEBUG_NORMAL, "\n");
-  */
-
-  // Make sure we know how to handle this data.
-  if (indata[*offs] != WPA2_EXTENDED_DATA)
-    {
-      debug_printf(DEBUG_NORMAL, "Unknown data type in decrypted data! "
-		   "(%02X)\n", indata[(*offs)+1]);
-      return WPA2_EXTENDED_UNKNOWN_DATA;
-    }
-
-  // If the next byte is 0x00, then it is padding.
-  if (indata[(*offs)+1] == 0x00)
-    {
-      debug_printf(DEBUG_INT, "Remaining data is padding.\n");
-      return WPA2_EXTENDED_PADDING;
-    }
-
-  // Make sure it is an OUI we know how to deal with.
-
-  if (memcmp((u_char *)&wpaoui, (u_char *)&indata[(*offs)+2], 3) == 0)
-    {
-      debug_printf(DEBUG_EVERYTHING, "Found WPA OUI!\n");
-      return WPA2_EMBEDDED_WPA_IE;
-    }
-
-  if (memcmp((u_char *)&valid_oid, (u_char *)&indata[(*offs)+2], 3) != 0)
-    {
-      debug_printf(DEBUG_NORMAL, "Unknown OUI : ");
-      debug_hex_printf(DEBUG_NORMAL, &indata[(*offs)+2], 3);
-      debug_printf_nl(DEBUG_NORMAL, "\n");
-      return WPA2_EXTENDED_UNKNOWN_OUI;
-    }
-
-  // Since we know we are dealing with something we understand, now
-  // decide what it is.
-  switch (indata[(*offs)+5])
-    {
-    case WPA2_EXTENDED_GTK_KEY:
-      debug_printf(DEBUG_INT, "Key is GTK type.\n");
-      return WPA2_EXTENDED_GTK_KEY;
-      break;
-
-    case WPA2_EXTENDED_STA_KEY:
-      debug_printf(DEBUG_INT, "Key is STAkey.\n");
-      return WPA2_EXTENDED_STA_KEY;
-      break;
-
-    case WPA2_EXTENDED_MAC_ADDRESS:
-      debug_printf(DEBUG_INT, "Key is MAC Address.\n");
-      return WPA2_EXTENDED_MAC_ADDRESS;
-      break;
-
-    default:
-      debug_printf(DEBUG_NORMAL, "Unknown key type %d!\n",
-		   indata[(*offs)+5]);
-      return WPA2_EXTENDED_UNKNOWN_DATA;
-      break;
-    }
-      
-  return XENONE;
-}
-
-/********************************************************
- *
- * Actually get our key, and the index associated with it.
- *
- ********************************************************/
-int eapol_key_type2_parse_key(struct interface_data *intdata, u_char *indata, 
-			      int *dataoffs, uint16_t *index, int *txkey, 
-			      u_char *key, int keylen)
-{
-  int len;
-
-  // If we got this far, we need to skip the OUI information.
-  len = indata[*dataoffs+1];
-
-  *dataoffs += 5;
-
-  // Apply the key index mask to get our key index.
-  *index = indata[*dataoffs] & WPA2_KEY_INDEX_MASK;
-  
-  // Check the TX bit
-  *txkey = indata[*dataoffs] & WPA2_TX_KEY_MASK;
-
-  // Skip to the beginning of the key data
-  *dataoffs += 3;
-  len -=6;
-
-  memcpy(key, &indata[*dataoffs], len);
-
-  *dataoffs += len;
-
-  return XENONE;
-}
-
-/********************************************************
- *
- * Parse encrypted key data, remove padding, pull out secondary IEs,
- * verify the IE we used, grab the key index, and pull out the key data.
- *
- ********************************************************/
-int eapol_key_type2_parse_enc_data(struct interface_data *intdata, 
-				   u_char *indata, int datalen, 
-				   uint16_t *index, int *txkey,
-				   u_char *key, int keylen)
-{
-  int dataoffs = 0, done = 0;
-
-  *index = 0xffff;    // Return an invalid key index to indicate we don't have
-                      // a key.
-
-  // The IE should be at the beginning of the list.  So, check it first.
-  if (eapol_key_type2_cmp_ie(intdata, indata, &dataoffs) != XENONE)
-    {
-      debug_printf(DEBUG_NORMAL, "Encrypted IE check failed!\n");
-      return -1;
-    }
-
-  while ((!done) && (datalen > dataoffs))
-    {
-      switch (eapol_key_type2_parse_extended(intdata, indata, indata[1], 
-					     &dataoffs))
-	{
-	case WPA2_EMBEDDED_WPA_IE:
-	  debug_printf(DEBUG_INT, "Got WPA IE, skipping!\n");
-	  // We need to skip 2 extra bytes, since the IE length doesn't 
-	  // include the identifier byte, or the length.
-	  dataoffs += indata[dataoffs+1]+2;
-	  break;
-
-	case WPA2_EXTENDED_UNKNOWN_OUI:
-	  // Since we don't know what to do with this, just skip it.
-	  debug_printf(DEBUG_INT, "Unknown OUI, skipping!\n");
-	  dataoffs += indata[dataoffs+1];
-	  break;
-	  
-	case WPA2_EXTENDED_KEY_RESERVED:
-	  // We got a reserved value.  Complain, and move on.
-	  debug_printf(DEBUG_NORMAL, "Got a reserved value!  Ignoring!\n");
-	  dataoffs += indata[dataoffs+1];
-	  break;
-
-	case WPA2_EXTENDED_GTK_KEY:
-	  // Gather data on the GTK we are using.
-	  eapol_key_type2_parse_key(intdata, indata, &dataoffs, index, txkey, 
-				    key, keylen);
-	  break;
-	  
-	case WPA2_EXTENDED_STA_KEY:
-	case WPA2_EXTENDED_MAC_ADDRESS:
-	  debug_printf(DEBUG_NORMAL, "Unsupported key type! Ignoring!\n");
-	  dataoffs += indata[dataoffs+1];
-	  break;
-
-	case WPA2_EXTENDED_PADDING:
-	  // If we hit padding, we are done.
-	  done = 1;
-	  break;
-
-	case WPA2_EXTENDED_UNKNOWN_DATA:
-	default:
-	  debug_printf(DEBUG_NORMAL, "Confused about what to do! (%s:%d)"
-		       "\n", __FUNCTION__, __LINE__);
-	  return WPA2_EXTENDED_UNKNOWN_DATA;
-	  break;
-	}
-    }
-
-  return XENONE;
+  eapol_key_type2_dump(intdata, (char *) intdata->sendframe);
 }
 
 /********************************************************
@@ -803,10 +871,12 @@ void eapol_key_type2_do_type3(struct interface_data *intdata)
 {
   struct wpa2_key_packet *inkeydata, *outkeydata;
   uint8_t *keydata, *aesval;
-  uint16_t keyflags, len, value16, keyindex = 0, keylen = 0;
-  int eapolver, version = 0, encdata = 0, txkey = 0;
+  uint16_t keyflags, len, value16, keylen = 0;
+  int eapolver, version = 0, encdata = 0;
   char key[32], rc4_ek[32];
-  u_char allfs[6] = {0xff,0xff,0xff,0xff,0xff,0xff}, gtk[32];
+  u_char framecpy[1530];
+  int framesize;
+  char zeros[8] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
   struct config_network *network_data;
 
   network_data = config_get_network_config();
@@ -823,9 +893,24 @@ void eapol_key_type2_do_type3(struct interface_data *intdata)
   inkeydata = (struct wpa2_key_packet *)&intdata->recvframe[OFFSET_TO_EAPOL+4];
   outkeydata = (struct wpa2_key_packet *)&intdata->sendframe[OFFSET_TO_EAPOL+4];
 
+  // First, make sure that the inkeydata replay counter is higher than
+  // the last counter we saw.
+  if ((memcmp(inkeydata->key_replay_counter, 
+	      intdata->statemachine->replay_counter, 8) <= 0) && 
+      (memcmp(inkeydata->key_replay_counter, zeros, 8) != 0))
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid replay counter!  Discarding!\n");
+      intdata->recv_size = 0;
+      return;
+    }
+
+  // Update our replay counter.
+  memcpy(intdata->statemachine->replay_counter, inkeydata->key_replay_counter,
+	 8);
+
   // Clear everything out.
   bzero(&intdata->sendframe[OFFSET_TO_EAPOL], 
-	sizeof(struct wpa2_key_packet)+28);
+	sizeof(struct wpa2_key_packet)+280);
 
   outkeydata->key_descriptor = WPA2_KEY_TYPE;
 
@@ -844,6 +929,7 @@ void eapol_key_type2_do_type3(struct interface_data *intdata)
   if (value16 & WPA2_SECURE_FLAG) keyflags |= WPA2_SECURE_FLAG;
   if (value16 & WPA2_ENCRYPTED_DATA) encdata = 1;
 
+
   keyflags = htons(keyflags);
 
   memcpy(&outkeydata->key_information, &keyflags, 2);
@@ -859,41 +945,42 @@ void eapol_key_type2_do_type3(struct interface_data *intdata)
   memcpy(outkeydata->key_nonce, intdata->statemachine->SNonce,
 	 32);
 
-  eapolver = network_data->force_eapol_ver;
-  
-  // If we have not hard set the eapol version, then use the last
-  // version that the authenticator gave us.  This isn't in line with
-  // the 802.1X-REV-2004-d11 standard, but many authenticators are checking
-  // version #s, and breaking when it isn't 1. :-/
-  if ((eapolver < 1) || (eapolver > MAX_EAPOL_VER))
-    eapolver = snmp_get_dot1xSuppLastEapolFrameVersion();
+  eapolver = eapol_get_eapol_ver();
 
   eapol_build_header(EAPOL_KEY, (intdata->send_size-OFFSET_TO_EAPOL-4), 
-		     eapolver, intdata->sendframe); 
+		     eapolver, (char *) intdata->sendframe); 
 
   memcpy(key, intdata->statemachine->PTK, 16);
-  mic_wpa_populate(intdata->sendframe, intdata->send_size+4, key, 16);  
+  mic_wpa_populate((char *) intdata->sendframe, intdata->send_size+4, key, 16);  
 
   // Dump what we built.
-  eapol_key_type2_dump(intdata, intdata->sendframe);
+  eapol_key_type2_dump(intdata, (char *) intdata->sendframe);
 
   // Get TK1
   value16 = ntohs(inkeydata->key_length);
   memcpy(key, (char *)&intdata->statemachine->PTK[32], value16);
   
   debug_printf(DEBUG_INT, "TK1 : ");
-  debug_hex_printf(DEBUG_INT, key, value16);
+  debug_hex_printf(DEBUG_INT, (u_char *) key, value16);
   
+  // We need to send the result frame now, but that will result in the
+  // "sendframe" being zeroed out.  So, we need to make a copy.
+  framesize = intdata->recv_size;
+  memcpy(&framecpy, &intdata->recvframe, framesize);
+  inkeydata = (struct wpa2_key_packet *)&framecpy[OFFSET_TO_EAPOL+4];
+
   cardif_sendframe(intdata);
   intdata->statemachine->eapolEap = FALSE;
   intdata->send_size = 0;
 
-  if (encdata)
+  keylen = ntohs(inkeydata->key_material_len);
+
+  eapol_key_type2_dump(intdata, (char *) &framecpy);
+
+  if ((encdata) && (keylen > 0))
     {
-      keylen = ntohs(inkeydata->key_material_len);
-      
       keydata = (uint8_t *)malloc(keylen);
-      
+
       memcpy(keydata, inkeydata->keydata, keylen);
       
       switch (version)
@@ -902,13 +989,18 @@ void eapol_key_type2_do_type3(struct interface_data *intdata)
 	  bzero(rc4_ek, 32);
 	  memcpy(rc4_ek, inkeydata->key_iv, 16);
 	  memcpy(&rc4_ek[16], &intdata->statemachine->PTK[16], 16);
-	  rc4_skip(rc4_ek, 32, 256, keydata, keylen);
+	  rc4_skip((u_char *) rc4_ek, 32, 256, keydata, keylen);
 	  break;
 	  
 	case 2:
+	  debug_printf(DEBUG_NORMAL, "keylen = %d\n", keylen);
+
 	  aesval = (uint8_t *)malloc(keylen);
-	  if (aes_unwrap(&intdata->statemachine->PTK[16], (keylen-8)/8, keydata,
-		     aesval))
+
+	  bzero(aesval, keylen);
+
+	  if (aes_unwrap((u_char *) &intdata->statemachine->PTK[16], 
+			 ((keylen)/8)-1, keydata, aesval) != 0)
 	    {
 	      debug_printf(DEBUG_NORMAL, "Failed AES unwrap!\n");
 	      return;
@@ -924,64 +1016,47 @@ void eapol_key_type2_do_type3(struct interface_data *intdata)
 	  return;
 	  break;
 	}
-      
-      bzero(gtk, 32);
 
-      eapol_key_type2_parse_enc_data(intdata, keydata, keylen, &keyindex, 
-				     &txkey, (u_char *)&gtk, 
-				     ntohs(inkeydata->key_length));
+      debug_printf(DEBUG_INT, "Keydata (%d) : \n", keylen);
+      debug_hex_dump(DEBUG_INT, keydata, keylen);
 
-      if (keyindex != 0xffff)
+      if (eapol_key_type2_process_keydata(intdata, keydata, keylen,
+					  ntohs(inkeydata->key_length), 
+					  inkeydata->key_rsc, version)
+	  != XENONE)
 	{
-	  debug_printf(DEBUG_INT, "GTK included in part 3 of the 4-way "
-		       "handshake!\n");
-
-	  debug_printf(DEBUG_INT, "Key (index = %d) : \n", keyindex);
-	  debug_hex_dump(DEBUG_INT, gtk, ntohs(inkeydata->key_length));
-	  
-	  switch (version) 
-	    {
-	    case 1:
-	      wpa_common_swap_rx_tx_mic(gtk);
-	      cardif_set_tkip_key(intdata, allfs, keyindex, txkey, 
-				  inkeydata->key_rsc, RSC_LEN, gtk, value16);
-	      break;
-	      
-	    case 2:
-	      cardif_set_ccmp_key(intdata, allfs, keyindex, txkey, 
-				  inkeydata->key_rsc, RSC_LEN, gtk, value16);
-	      break;
-	    }
-	  
-	  if (network_data->methods->method_num == WPA_PSK)
-	    {
-	      // If we are using PSK, and we made it here, then we are in 
-	      // AUTHENTICATED state.
-	      intdata->statemachine->eapSuccess = TRUE;
-	      cardif_drop_unencrypted(intdata, FALSE);
-	    } else {
-	      // Drop unencrypted frames.
-	      cardif_drop_unencrypted(intdata, TRUE);
-	    }
+	  debug_printf(DEBUG_NORMAL, "Error processing key data!\n");
+	  return;
 	}
     }
+
+  if (network_data->methods->method_num == WPA_PSK)
+    {
+      // If we are using PSK, and we made it here, then we are in 
+      // AUTHENTICATED state.
+      intdata->statemachine->eapSuccess = TRUE;
+      cardif_drop_unencrypted(intdata, FALSE);
+    } else {
+      // Drop unencrypted frames.
+      cardif_drop_unencrypted(intdata, TRUE);
+    }
   
   debug_printf(DEBUG_INT, "Setting PTK!\n");
-
+  
   switch (version) 
     {
     case 1:
       // We need to swap the TX/RX MIC values since we are the supplicant.  
       // (Thanks to Jouni Malinen for pointing this out!)
-      wpa_common_swap_rx_tx_mic(key);
-
+      wpa_common_swap_rx_tx_mic((u_char *) key);
+      
       cardif_set_tkip_key(intdata, intdata->dest_mac, 0, TRUE, 
-			  inkeydata->key_rsc, RSC_LEN, key, value16);
+			  (char *) inkeydata->key_rsc, RSC_LEN, key, value16);
       break;
 
     case 2:
       cardif_set_ccmp_key(intdata, intdata->dest_mac, 0, TRUE, 
-			  inkeydata->key_rsc, RSC_LEN, key, value16);
+			  (char *) inkeydata->key_rsc, RSC_LEN, key, value16);
       break;
     }
 }
@@ -1008,7 +1083,7 @@ void eapol_key_type2_determine_key(struct interface_data *intdata)
 
   if (keyflags & WPA2_KEY_MIC_FLAG)
     {
-      if (mic_wpa_validate(intdata->recvframe, intdata->recv_size, 
+      if (mic_wpa_validate((char *) intdata->recvframe, intdata->recv_size, 
 			   intdata->statemachine->PTK, 16) == FALSE)
 	{
 	  intdata->statemachine->MICVerified = TRUE;
@@ -1025,17 +1100,20 @@ void eapol_key_type2_determine_key(struct interface_data *intdata)
     {
       debug_printf(DEBUG_INT, "Key Packet #3 (response) :\n");
       eapol_key_type2_do_type3(intdata);
-    } else if ((keyflags & WPA2_PAIRWISE_KEY) && (keyflags & WPA2_KEY_ACK_FLAG))
+    } 
+  else if ((keyflags & WPA2_PAIRWISE_KEY) && (keyflags & WPA2_KEY_ACK_FLAG))
     {
       debug_printf(DEBUG_INT, "Key Packet #1 (response) :\n");
       eapol_key_type2_do_type1(intdata);
-    } else if ((keyflags & WPA2_KEY_MIC_FLAG) && (keyflags & WPA2_PAIRWISE_KEY))
+    } 
+  else if ((keyflags & WPA2_KEY_MIC_FLAG) && (keyflags & WPA2_PAIRWISE_KEY))
     {
       debug_printf(DEBUG_NORMAL, "Got Key Packet #2!  (This shouldn't happen!"
 		   ")\n");
       return;
-    } else if ((keyflags & WPA2_SECURE_FLAG) && 
-	       (keyflags & WPA2_KEY_MIC_FLAG) && (WPA2_PAIRWISE_KEY))
+    } 
+  else if ((keyflags & WPA2_SECURE_FLAG) && 
+	   (keyflags & WPA2_KEY_MIC_FLAG) && (keyflags & WPA2_PAIRWISE_KEY))
     {
       debug_printf(DEBUG_NORMAL, "Got Key Packet #4!  (This shouldn't happen"
 		   "!)\n");
@@ -1048,17 +1126,10 @@ void eapol_key_type2_determine_key(struct interface_data *intdata)
 
   if (intdata->recv_size > 0)
     {
-      eapolver = network_data->force_eapol_ver;
-
-      // If we have not hard set the eapol version, then use the last
-      // version that the authenticator gave us.  This isn't in line with
-      // the 802.1X-REV-2004-d11 standard, but many authenticators are checking
-      // version #s, and breaking when it isn't 1. :-/
-      if ((eapolver < 1) || (eapolver > MAX_EAPOL_VER))
-	eapolver = snmp_get_dot1xSuppLastEapolFrameVersion();
+      eapolver = eapol_get_eapol_ver();
 
       eapol_build_header(EAPOL_KEY, (intdata->recv_size-OFFSET_TO_EAPOL-4), 
-			 eapolver, intdata->recvframe);
+			 eapolver, (char *) intdata->recvframe);
       cardif_sendframe(intdata);
       intdata->statemachine->eapolEap = FALSE;
     }      
@@ -1127,8 +1198,8 @@ void eapol_key_type2_process(struct interface_data *intdata)
 		}
 
 	      // We have an ASCII password, so calculate it.
-	      if (psk_wpa_pbkdf2(psk->key, intdata->cur_essid, 
-				 strlen(intdata->cur_essid), (char *)&tpmk) 
+	      if (psk_wpa_pbkdf2(psk->key, (u_char *) intdata->cur_essid, 
+				 strlen(intdata->cur_essid), (u_char *)&tpmk) 
 		  == TRUE)
 		{
 		  intdata->statemachine->PMK = (char *)malloc(32);
@@ -1168,7 +1239,7 @@ void eapol_key_type2_process(struct interface_data *intdata)
 	}
     } 
 
-  eapol_key_type2_dump(intdata, inframe);
+  eapol_key_type2_dump(intdata, (char *) inframe);
 
   eapol_key_type2_determine_key(intdata);
 
