@@ -7,9 +7,42 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: eapol_key_type254.c,v 1.41 2005/08/09 01:39:14 chessing Exp $
- * $Date: 2005/08/09 01:39:14 $
+ * $Id: eapol_key_type254.c,v 1.51 2006/01/24 04:42:26 chessing Exp $
+ * $Date: 2006/01/24 04:42:26 $
  * $Log: eapol_key_type254.c,v $
+ * Revision 1.51  2006/01/24 04:42:26  chessing
+ * A few more fixes to WPA code, along with a fix to the scan reaping code.
+ *
+ * Revision 1.50  2006/01/23 20:33:47  chessing
+ * Added support for the replay counters in WPA/WPA2.  BSSID is now selected based on the signal quality information returned during the scan.  (Assuming signal quality information is returned during a scan. ;)
+ *
+ * Revision 1.49  2006/01/23 07:13:49  chessing
+ * A couple of fixups to the WPA and WPA2 code.
+ *
+ * Revision 1.48  2006/01/15 06:56:29  chessing
+ * Some fixes to WPA/WPA2.  WPA is working correctly now.  WPA2 needs a little work to correctly identify key frames that are for the GTK.
+ *
+ * Revision 1.47  2006/01/12 17:54:02  chessing
+ * WPA almost works again on cards that use the GENIE setting.  Added a small fix to allow Xsupplicant to build on Open SuSE 10.
+ *
+ * Revision 1.46  2006/01/03 04:02:35  chessing
+ * Added the ability to store the PEAP password in a hashed format.  (Basically, an MS-CHAPv1 hash.)  Also added an 'ntpwdhash' program to the tools directory that will convert a cleartext password in to a hash that can be copied to the configuration file.
+ *
+ * Revision 1.45  2005/10/17 03:56:53  chessing
+ * Updates to the libxsupconfig library.  It no longer relies on other source from the main tree, so it can be used safely in other code with problems.
+ *
+ * Revision 1.44  2005/10/14 02:26:17  shaftoe
+ * - cleanup gcc 4 warnings
+ * - (re)add support for a pid in the form of /var/run/xsupplicant.<iface>.pid
+ *
+ * -- Eric Evans <eevans@sym-link.com>
+ *
+ * Revision 1.43  2005/09/14 03:34:54  chessing
+ * Small cosmetic changes.  Default association mode is now auto instead of manual. Fixes for bug IDs #1290449 & #1290323.
+ *
+ * Revision 1.42  2005/09/05 01:00:34  chessing
+ * Major overhaul to most of the state machines in Xsupplicant.  Also added additional error messages to the TLS functions to try to debug the one of the problems reported on the list.  Basic testing shows this new code to be more stable than previous code, but it needs more testing.
+ *
  * Revision 1.41  2005/08/09 01:39:14  chessing
  * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
@@ -22,7 +55,7 @@
 #include "xsup_debug.h"
 #include "xsup_err.h"
 #include "profile.h"
-#include "config.h"
+#include "xsupconfig.h"
 #include "key_statemachine.h"
 #include "eapol_key_type254.h"
 #include "eapol.h"
@@ -32,6 +65,7 @@
 #include "psk.h"
 #include "mic.h"
 #include "snmp.h"
+#include "config_ssid.h"
 #include "cardif/cardif.h"
 #include "eap_types/mschapv2/mschapv2.h"
 
@@ -192,12 +226,12 @@ char *eapol_key_type254_gen_ptk(struct interface_data *intdata, char *Anonce)
     }
 
   debug_printf(DEBUG_INT, "PMK : ");
-  debug_hex_printf(DEBUG_INT, intdata->statemachine->PMK, 32);
-  wpa_PRF(intdata->statemachine->PMK, 32, "Pairwise key expansion", 22,
-	  (char *)&prfdata, 76, retval, 64);
+  debug_hex_printf(DEBUG_INT, (u_char *) intdata->statemachine->PMK, 32);
+  wpa_PRF((u_char *) intdata->statemachine->PMK, 32, (u_char *) "Pairwise key expansion", 22,
+	  (u_char *)&prfdata, 76, (u_char *) retval, 64);
 
   debug_printf(DEBUG_INT, "PTK : ");
-  debug_hex_printf(DEBUG_INT, retval, 64);
+  debug_hex_printf(DEBUG_INT, (u_char *) retval, 64);
 
   return retval;
 }
@@ -254,20 +288,13 @@ void eapol_key_type254_request_new_key(struct interface_data *intdata,
   len = sizeof(struct wpa_key_packet);
   intdata->send_size = len+OFFSET_TO_EAPOL+4;
 
-  eapolver = network_data->force_eapol_ver;
-
-  // If we have not hard set the eapol version, then use the last
-  // version that the authenticator gave us.  This isn't in line with
-  // the 802.1X-REV-2004-d11 standard, but many authenticators are checking
-  // version #s, and breaking when it isn't 1. :-/
-  if ((eapolver < 1) || (eapolver > MAX_EAPOL_VER))
-    eapolver = snmp_get_dot1xSuppLastEapolFrameVersion();
+  eapolver = eapol_get_eapol_ver();
 
   eapol_build_header(EAPOL_KEY, (intdata->send_size-OFFSET_TO_EAPOL-4), 
-		     eapolver, intdata->sendframe); 
+		     eapolver, (char *) intdata->sendframe); 
   
   memcpy(key, intdata->statemachine->PTK, 16);
-  mic_wpa_populate(intdata->sendframe, intdata->send_size+4, key, 16);
+  mic_wpa_populate((char *) intdata->sendframe, intdata->send_size+4, key, 16);
 
   cardif_sendframe(intdata);
   intdata->statemachine->eapolEap = FALSE;
@@ -287,9 +314,10 @@ void eapol_key_type254_do_gtk(struct interface_data *intdata)
   struct wpa_key_packet *inkeydata, *outkeydata;
   uint16_t value16, keyflags, version, keyindex, len;
   int eapolver;
-  unsigned char *keydata;
+  unsigned char *keydata = NULL;
   char key[16], rc4_ek[32];
   u_char allfs[6] = {0xff,0xff,0xff,0xff,0xff,0xff};
+  char zeros[8] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
   struct config_network *network_data;
 
   network_data = config_get_network_config();
@@ -304,6 +332,17 @@ void eapol_key_type254_do_gtk(struct interface_data *intdata)
   inkeydata = (struct wpa_key_packet *)&intdata->recvframe[OFFSET_TO_EAPOL+4];
   outkeydata = (struct wpa_key_packet *)&intdata->sendframe[OFFSET_TO_EAPOL+4];
 
+  // First, make sure that the inkeydata replay counter is higher than
+  // the last counter we saw.
+  if ((memcmp(inkeydata->key_replay_counter, 
+	      intdata->statemachine->replay_counter, 8) <= 0) &&
+      (memcmp(inkeydata->key_replay_counter, zeros, 8) != 0))
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid replay counter!  Discarding!\n");
+      intdata->recv_size = 0;
+      return;
+    }
+
   // Clear everything out.
   bzero(&intdata->sendframe[OFFSET_TO_EAPOL+4], sizeof(struct wpa_key_packet));
 
@@ -330,7 +369,7 @@ void eapol_key_type254_do_gtk(struct interface_data *intdata)
 
   value16 = ntohs(inkeydata->key_material_len);
   
-  keydata = (unsigned char *)malloc(value16);
+  keydata = (unsigned char *)malloc(value16+8);
   if (keydata == NULL)
     {
       debug_printf(DEBUG_NORMAL, "Couldn't allocate memory for key data!\n");
@@ -341,8 +380,8 @@ void eapol_key_type254_do_gtk(struct interface_data *intdata)
 	 &intdata->recvframe[OFFSET_TO_EAPOL+4+sizeof(struct wpa_key_packet)],
 	 value16);
 
-  debug_printf(DEBUG_INT, "Setting GTK! (Version : %d  Length : %d)\n", 
-	       version, value16);
+  debug_printf(DEBUG_INT, "Setting GTK! (Version : %d  Length : %d  Slot : %d)"
+	       "\n", version, value16, keyindex);
 
   switch (version)
     {
@@ -351,22 +390,27 @@ void eapol_key_type254_do_gtk(struct interface_data *intdata)
       bzero(rc4_ek, 32);
       memcpy(rc4_ek, inkeydata->key_iv, 16);
       memcpy(&rc4_ek[16], &intdata->statemachine->PTK[16], 16);
-      rc4_skip(rc4_ek, 32, 256, keydata, value16);
+      rc4_skip((u_char *) rc4_ek, 32, 256, keydata, value16);
       
+      debug_printf_nl(DEBUG_INT, "GTK : ");
+      debug_hex_printf(DEBUG_INT, keydata, value16);
+
       wpa_common_swap_rx_tx_mic(keydata);
-      cardif_set_tkip_key(intdata, allfs, keyindex, FALSE, inkeydata->key_rsc,
-			  6, keydata, value16);
+      cardif_set_tkip_key(intdata, (char *) allfs, keyindex, FALSE, 
+			  (char *) inkeydata->key_rsc, RSC_LEN, 
+			  (char *) keydata, value16);
       break;
 
     case 2:
       // First, decrypt the GTK
       bzero(key, 16);
-      aes_unwrap(&intdata->statemachine->PTK[16], (value16-8)/8, keydata,
-		 key);
+      aes_unwrap((u_char *) &intdata->statemachine->PTK[16], (value16-8)/8, 
+		 keydata, (u_char *) key);
 
       wpa_common_swap_rx_tx_mic(keydata);
-      cardif_set_ccmp_key(intdata, NULL, keyindex, FALSE, inkeydata->key_rsc,
-     			  6, key, (value16-8));
+      cardif_set_ccmp_key(intdata, NULL, keyindex, FALSE, 
+			  (char *) inkeydata->key_rsc, RSC_LEN, key, 
+			  (value16-8));
       break;
     }
 
@@ -381,23 +425,16 @@ void eapol_key_type254_do_gtk(struct interface_data *intdata)
   outkeydata->key_length = inkeydata->key_length;
   memcpy(&outkeydata->key_replay_counter, &inkeydata->key_replay_counter, 8);
 
-  eapolver = network_data->force_eapol_ver;
-
-  // If we have not hard set the eapol version, then use the last
-  // version that the authenticator gave us.  This isn't in line with
-  // the 802.1X-REV-2004-d11 standard, but many authenticators are checking
-  // version #s, and breaking when it isn't 1. :-/
-  if ((eapolver < 1) || (eapolver > MAX_EAPOL_VER))
-    eapolver = snmp_get_dot1xSuppLastEapolFrameVersion();
+  eapolver = eapol_get_eapol_ver();
 
   eapol_build_header(EAPOL_KEY, (intdata->send_size-OFFSET_TO_EAPOL-4), 
-		     eapolver, intdata->sendframe); 
+		     eapolver, (char *) intdata->sendframe); 
   
-  memcpy(key, intdata->statemachine->PTK, 16);
-  mic_wpa_populate(intdata->sendframe, intdata->send_size+4, key, 16);
+  memcpy(&key, intdata->statemachine->PTK, 16);
+  mic_wpa_populate((char *) intdata->sendframe, intdata->send_size+4, key, 16);
 
   // Dump what we built.
-  eapol_key_type254_dump(intdata->sendframe);
+  eapol_key_type254_dump((char *) intdata->sendframe);
 
   if (network_data->methods->method_num == WPA_PSK)
     {
@@ -408,6 +445,12 @@ void eapol_key_type254_do_gtk(struct interface_data *intdata)
 
   // Drop unencrypted frames.
   cardif_drop_unencrypted(intdata, 1);
+
+  if (keydata)
+    {
+      free(keydata);
+      keydata = NULL;
+    }
 }
 
 /***************************************************************
@@ -420,8 +463,8 @@ void eapol_key_type254_do_type1(struct interface_data *intdata)
   struct wpa_key_packet *inkeydata, *outkeydata;
   uint16_t keyflags, len, value16;
   int i, version, eapolver, ielen;
-  char key[16];
-  char wpa_ie[26];
+  char key[16], wpa_ie[26];
+  char zeros[8] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
   struct config_network *network_data;
 
   network_data = config_get_network_config();
@@ -436,6 +479,17 @@ void eapol_key_type254_do_type1(struct interface_data *intdata)
   inkeydata = (struct wpa_key_packet *)&intdata->recvframe[OFFSET_TO_EAPOL+4];
   outkeydata = (struct wpa_key_packet *)&intdata->sendframe[OFFSET_TO_EAPOL+4];
 
+  // First, make sure that the inkeydata replay counter is higher than
+  // the last counter we saw.
+  if ((memcmp(inkeydata->key_replay_counter, 
+	      intdata->statemachine->replay_counter, 8) <= 0) &&
+      (memcmp(inkeydata->key_replay_counter, zeros, 8) != 0))
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid replay counter!  Discarding!\n");
+      intdata->recv_size = 0;
+      return;
+    }
+
   // Clear everything out.
   bzero(&intdata->sendframe[OFFSET_TO_EAPOL+4], sizeof(struct wpa_key_packet));
 
@@ -494,23 +548,53 @@ void eapol_key_type254_do_type1(struct interface_data *intdata)
 
   outkeydata->key_material_len = value16;
 
-  eapolver = network_data->force_eapol_ver;
-
-  // If we have not hard set the eapol version, then use the last
-  // version that the authenticator gave us.  This isn't in line with
-  // the 802.1X-REV-2004-d11 standard, but many authenticators are checking
-  // version #s, and breaking when it isn't 1. :-/
-  if ((eapolver < 1) || (eapolver > MAX_EAPOL_VER))
-    eapolver = snmp_get_dot1xSuppLastEapolFrameVersion();
+  eapolver = eapol_get_eapol_ver();
 
   eapol_build_header(EAPOL_KEY, (intdata->send_size-OFFSET_TO_EAPOL-4), 
-		     eapolver, intdata->sendframe);
+		     eapolver, (char *) intdata->sendframe);
 
   memcpy(key, intdata->statemachine->PTK, 16);
-  mic_wpa_populate(intdata->sendframe, intdata->send_size+4, key, 16);
+  mic_wpa_populate((char *) intdata->sendframe, intdata->send_size+4, key, 16);
 
   // Dump what we built.
-  eapol_key_type254_dump(intdata->sendframe);
+  eapol_key_type254_dump((char *) intdata->sendframe);
+}
+
+/********************************************************
+ *
+ * Compare the IE that we got from a key packet to the IE that we got from
+ * the AP, to see if they match.
+ *
+ ********************************************************/
+char eapol_key_type254_cmp_ie(struct interface_data *intdata, u_char *wpaie, 
+			      char wpaielen)
+{
+  u_char *apie, apielen;
+
+  debug_printf(DEBUG_INT, "WPA IE from Key Packet (%d) : ", wpaielen);
+  debug_hex_printf(DEBUG_INT, wpaie, wpaielen);
+
+  config_ssid_get_wpa_ie(&apie, &apielen);
+  debug_printf(DEBUG_INT, "WPA IE from AP Scan (%d)    : ", apielen);
+  debug_hex_printf(DEBUG_INT, apie, apielen);
+
+  if (wpaielen != apielen)
+    {
+      debug_printf(DEBUG_NORMAL, "IE from the AP and IE from the key messages"
+		   " are different lengths!\n");
+      free(apie);
+      return -1;
+    }
+
+  if (memcmp(wpaie, apie, apielen) != 0)
+    {
+      debug_printf(DEBUG_NORMAL, "IE from the AP and IE from the key messages"
+		   " do not match!\n");
+      free(apie);
+      return -1;
+    }
+
+  return XENONE;
 }
 
 /********************************************************
@@ -525,6 +609,7 @@ void eapol_key_type254_do_type3(struct interface_data *intdata)
   uint16_t keyflags, len, value16, keyindex;
   int eapolver, version;
   char key[32];
+  char zeros[8] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
   struct config_network *network_data;
 
   network_data = config_get_network_config();
@@ -541,6 +626,21 @@ void eapol_key_type254_do_type3(struct interface_data *intdata)
   inkeydata = (struct wpa_key_packet *)&intdata->recvframe[OFFSET_TO_EAPOL+4];
   outkeydata = (struct wpa_key_packet *)&intdata->sendframe[OFFSET_TO_EAPOL+4];
 
+  // First, make sure that the inkeydata replay counter is higher than
+  // the last counter we saw.
+  if ((memcmp(inkeydata->key_replay_counter, 
+	      intdata->statemachine->replay_counter, 8) <= 0) &&
+      (memcmp(inkeydata->key_replay_counter, zeros, 8) != 0))
+    {
+      debug_printf(DEBUG_NORMAL, "Invalid replay counter!  Discarding!\n");
+      intdata->recv_size = 0;
+      return;
+    }
+
+  // Update our replay counter.
+  memcpy(intdata->statemachine->replay_counter, inkeydata->key_replay_counter,
+	 8);
+
   // Clear everything out.
   bzero(&intdata->sendframe[OFFSET_TO_EAPOL], 
 	sizeof(struct wpa_key_packet)+28);
@@ -576,50 +676,54 @@ void eapol_key_type254_do_type3(struct interface_data *intdata)
   memcpy(outkeydata->key_nonce, intdata->statemachine->SNonce,
 	 32);
 
-  eapolver = network_data->force_eapol_ver;
+  eapolver = eapol_get_eapol_ver();
   
-  // If we have not hard set the eapol version, then use the last
-  // version that the authenticator gave us.  This isn't in line with
-  // the 802.1X-REV-2004-d11 standard, but many authenticators are checking
-  // version #s, and breaking when it isn't 1. :-/
-  if ((eapolver < 1) || (eapolver > MAX_EAPOL_VER))
-    eapolver = snmp_get_dot1xSuppLastEapolFrameVersion();
-
   eapol_build_header(EAPOL_KEY, (intdata->send_size-OFFSET_TO_EAPOL-4), 
-		     eapolver, intdata->sendframe); 
+		     eapolver, (char *) intdata->sendframe); 
 
   memcpy(key, intdata->statemachine->PTK, 16);
-  mic_wpa_populate(intdata->sendframe, intdata->send_size+4, key, 16);  
+  mic_wpa_populate((char *) intdata->sendframe, intdata->send_size+4, key, 16);  
 
   // Dump what we built.
-  eapol_key_type254_dump(intdata->sendframe);
+  eapol_key_type254_dump((char *) intdata->sendframe);
+
+  if (eapol_key_type254_cmp_ie(intdata, inkeydata->keydata, 
+			       ntohs(inkeydata->key_material_len)) != XENONE)
+    {
+      debug_printf(DEBUG_NORMAL, "Error comparing IEs.  Possible attack in "
+		   "progress!  Disconnecting.\n");
+      cardif_disassociate(intdata, DISASSOC_INVALID_IE);
+      return;
+    }
 
   // Get TK1
   value16 = ntohs(inkeydata->key_length);
   memcpy(key, (char *)&intdata->statemachine->PTK[32], value16);
   
   debug_printf(DEBUG_INT, "TK1 : ");
-  debug_hex_printf(DEBUG_INT, key, value16);
+  debug_hex_printf(DEBUG_INT, (u_char *) key, value16);
   
   cardif_sendframe(intdata);
   intdata->statemachine->eapolEap = FALSE;
   intdata->send_size = 0;
 
-  debug_printf(DEBUG_INT, "Setting PTK1!\n");
+  debug_printf(DEBUG_INT, "Setting PTK1! (Index : %d Length : %d)\n", 
+	       keyindex, value16);
 
   switch (version) 
     {
     case 1:
       // We need to swap the TX/RX MIC values since we are the supplicant.  
       // (Thanks to Jouni Malinen for pointing this out!)
-      wpa_common_swap_rx_tx_mic(key);
+      wpa_common_swap_rx_tx_mic((u_char *) key);
+
       cardif_set_tkip_key(intdata, intdata->dest_mac, keyindex, TRUE, 
-			  inkeydata->key_rsc, 6, key, value16);
+			  (char *) inkeydata->key_rsc, RSC_LEN, key, value16);
       break;
 
     case 2:
       cardif_set_ccmp_key(intdata, intdata->dest_mac, keyindex, TRUE, 
-			  inkeydata->key_rsc, 6, key, value16);
+			  (char *) inkeydata->key_rsc, RSC_LEN, key, value16);
       break;
     }
 }
@@ -646,7 +750,7 @@ void eapol_key_type254_determine_key(struct interface_data *intdata)
 
   if (keyflags & WPA_KEY_MIC_FLAG)
     {
-      if (mic_wpa_validate(intdata->recvframe, intdata->recv_size, 
+      if (mic_wpa_validate((char *) intdata->recvframe, intdata->recv_size, 
 			   intdata->statemachine->PTK, 16) == FALSE)
 	{
 	  intdata->statemachine->MICVerified = TRUE;
@@ -678,17 +782,10 @@ void eapol_key_type254_determine_key(struct interface_data *intdata)
 
   if (intdata->recv_size > 0)
     {
-      eapolver = network_data->force_eapol_ver;
-
-      // If we have not hard set the eapol version, then use the last
-      // version that the authenticator gave us.  This isn't in line with
-      // the 802.1X-REV-2004-d11 standard, but many authenticators are checking
-      // version #s, and breaking when it isn't 1. :-/
-      if ((eapolver < 1) || (eapolver > MAX_EAPOL_VER))
-	eapolver = snmp_get_dot1xSuppLastEapolFrameVersion();
+      eapolver = eapol_get_eapol_ver();
 
       eapol_build_header(EAPOL_KEY, (intdata->recv_size-OFFSET_TO_EAPOL-4), 
-			 eapolver, intdata->recvframe);
+			 eapolver, (char *) intdata->recvframe);
       cardif_sendframe(intdata);
       intdata->statemachine->eapolEap = FALSE;
     }      
@@ -757,8 +854,8 @@ void eapol_key_type254_process(struct interface_data *intdata)
 		}
 
 	      // We have an ASCII password, so calculate it.
-	      if (psk_wpa_pbkdf2(psk->key, intdata->cur_essid, 
-				 strlen(intdata->cur_essid), (char *)&tpmk) 
+	      if (psk_wpa_pbkdf2(psk->key, (u_char *) intdata->cur_essid, 
+				 strlen(intdata->cur_essid), (u_char *)&tpmk) 
 		  == TRUE)
 		{
 		  intdata->statemachine->PMK = (char *)malloc(32);
@@ -798,7 +895,7 @@ void eapol_key_type254_process(struct interface_data *intdata)
 	}
     } 
 
-  eapol_key_type254_dump(inframe);
+  eapol_key_type254_dump((char *) inframe);
 
   eapol_key_type254_determine_key(intdata);
 
