@@ -19,7 +19,7 @@
 #define WPA2_KEY_TYPE      2
 #define WPA_KEY_TYPE       254
 
-void run_key_statemachine(struct interface_data *);
+void key_statemachine_run(struct interface_data *);
 
 
 #endif
