@@ -5,9 +5,12 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: snmp.c,v 1.7 2005/08/09 01:39:14 chessing Exp $
- * $Date: 2005/08/09 01:39:14 $
+ * $Id: snmp.c,v 1.8 2005/09/08 16:27:01 chessing Exp $
+ * $Date: 2005/09/08 16:27:01 $
  * $Log: snmp.c,v $
+ * Revision 1.8  2005/09/08 16:27:01  chessing
+ * Some small updates to the new state machine code.  First attempt at an auto association mode.  (It mostly works. ;)
+ *
  * Revision 1.7  2005/08/09 01:39:14  chessing
  * Cleaned out old commit notes from the released version.  Added a few small features including the ability to disable the friendly warnings that are spit out.  (Such as the warning that is displayed when keys aren't rotated after 10 minutes.)  We should also be able to start when the interface is down.  Last, but not least, we can handle empty network configs.  (This may be useful for situations where there isn't a good reason to have a default network defined.)
  *
@@ -47,9 +50,10 @@ void snmp_dump_stats(char *intname)
   debug_printf(DEBUG_SNMP, "Last EAPOL Version : %10d   Last EAPOL Src.    :",
 	       snmpdata.dot1xSuppLastEapolFrameVersion);
   debug_hex_printf(DEBUG_SNMP, snmpdata.dot1xSuppLastEapolFrameSource, 6);
-  debug_printf(DEBUG_SNMP, "EAPOL Success      : %10d   EAPOL Failure      : %10d\n\n",
+  debug_printf(DEBUG_SNMP, "EAPOL Success      : %10d   EAPOL Failure      : %10d\n",
 	       snmpdata.eapol_success_rx, snmpdata.eapol_fail_rx);
-  
+  debug_printf(DEBUG_SNMP, "Backend Timeouts   : %10d\n\n", 
+	       snmpdata.backend_timeout);
 }
 
 void snmp_inc_dot1xSuppInvalidEapolFramesRx()
@@ -102,6 +106,11 @@ void snmp_eapol_fail_rx()
   snmpdata.eapol_fail_rx++;
 }
 
+void snmp_backend_timeout()
+{
+  snmpdata.backend_timeout++;
+}
+
 void snmp_dot1xSuppEapolRespIdFramesTx()
 {
   snmpdata.dot1xSuppEapolRespIdFramesTx++;
