@@ -12,14 +12,14 @@
 #define _TIMER_H_
 
 // Different types of timers that we will use.
-#define NUM_TIMER_IDS   5
+#define NUM_TIMER_IDS   6
 
 // BE SURE THAT IF YOU ADD A TIMER HERE, THAT YOU ADD IT'S DEF. IN timer.c!
 #define COUNTERMEASURE_TIMER 1
 #define REKEY_PROB_TIMER     2
 #define ASSOCIATION_TIMER    3
 #define STALE_KEY_WARN_TIMER 4
-#define AUTHENTICATION_TIMER 5 
+#define PASSIVE_SCAN_TIMER   5
 
 void timer_init();
 void timer_add_timer(uint16_t, uint16_t, void *, void *);
@@ -30,5 +30,7 @@ uint8_t timer_check_existing(uint16_t);
 struct timer_data *timer_get_by_id(uint16_t);
 void timer_cancel(uint16_t);
 void timer_cleanup();
+void timer_clock_start();
+void timer_sleep(uint16_t);
 
 #endif
