@@ -9,8 +9,6 @@
 #ifndef __WPA_H__
 #define __WPA_H__
 
-#define WPA_PSK        -1
-
 #define WPA_EID        0xdd
 
 
@@ -25,7 +23,7 @@ struct wpa_ie_struct {
   // The rest of the IE is variable, and can be structured here.
 } __attribute__((__packed__));
 
-void wpa_parse_ie(char *);
+int wpa_parse_ie(char *);
 void wpa_gen_ie(struct interface_data *, char *);
 void wpa_gen_ie_caps(struct interface_data *, char *);
 char *wpa_minmax(char *, char *, int, char);
