@@ -9,8 +9,6 @@
 #ifndef __WPA2_H__
 #define __WPA2_H__
 
-//#define WPA_PSK        -1
-
 #define WPA2_EID        0x30
 
 struct wpa2_ie_struct {
@@ -26,6 +24,6 @@ struct wpa2_ie_struct {
 void wpa2_gen_ie(struct interface_data *, char *, int *);
 void wpa2_gen_ie_caps(struct interface_data *, char *);
 char *wpa_minmax(char *, char *, int, char);
-void wpa2_parse_ie(char *iedata);
+int wpa2_parse_ie(char *iedata);
 
 #endif
