@@ -6,8 +6,8 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: xsup_debug.h,v 1.11 2005/07/21 04:24:25 chessing Exp $
- * $Date: 2005/07/21 04:24:25 $
+ * $Id: xsup_debug.h,v 1.12 2005/11/22 19:49:42 chessing Exp $
+ * $Date: 2005/11/22 19:49:42 $
  *******************************************************************/
 #ifndef XSUP_DEBUG_H_
 #define XSUP_DEBUG_H_
@@ -38,7 +38,7 @@
 #define NUM_DEBUG_EXCESSIVE  7
 #define DEBUG_EXCESSIVE      0x40
 
-void logfile_setup(char *);
+int logfile_setup(char *);
 void logfile_cleanup();
 void lowercase(char *);
 void debug_setdaemon(int);
