@@ -8,9 +8,63 @@
  *
  * Authors: Chris.Hessing@utah.edu
  *
- * $Id: xsup_driver.c,v 1.93 2005/08/21 18:12:13 chessing Exp $
- * $Date: 2005/08/21 18:12:13 $
+ * $Id: xsup_driver.c,v 1.110 2006/01/24 04:42:26 chessing Exp $
+ * $Date: 2006/01/24 04:42:26 $
  * $Log: xsup_driver.c,v $
+ * Revision 1.110  2006/01/24 04:42:26  chessing
+ * A few more fixes to WPA code, along with a fix to the scan reaping code.
+ *
+ * Revision 1.109  2006/01/19 22:23:36  chessing
+ * Added the ability to associate to an AP without having wpa_group_cipher, or wpa_pairwise_cipher defined in the config file.  If they are defined, then the value in the config file will be used no matter what.  Also, changed a piece in xsup_driver.c so that we print the capabilities that the cards reports on startup.  (This should help debug some problems that are almost certain to pop up with this new code. ;)
+ *
+ * Revision 1.108  2006/01/19 05:37:04  chessing
+ * WPA2 is working correctly.  Added the ability to query the card to gather encryption/authentication capabilities.  1.2.3 is now ready to go.
+ *
+ * Revision 1.107  2005/12/24 04:51:51  chessing
+ * Removed an unneeded file.  Updated scanning code to parse IWEVGENIE events correctly, and make use of them.
+ *
+ * Revision 1.106  2005/12/24 02:39:48  chessing
+ * Fixes to autoconf script to correctly identify the number of arguments that iw_extract_event_stream() takes, along with checking iwlib.h compiles correctly.
+ *
+ * Revision 1.105  2005/12/02 03:15:21  chessing
+ * A few small fixes to the madwifi driver code.
+ *
+ * Revision 1.104  2005/11/22 19:49:42  chessing
+ * When the desired interface isn't available when Xsupplicant starts, Xsupplicant will wait for it to show up before starting to authenticate.  In this case, the interface isn't 'available' when an ifconfig doesn't show that the interface exists.  It has *NOTHING* to do with the up/down state of the interface.  This is mostly useful for situations where a cardbus wireless card is plugged in before linux is booted.
+ *
+ * Revision 1.103  2005/11/14 17:27:39  chessing
+ * Removed the driver specific code for hostap, ipw, and ndiswrapper, since they all support WE18 now.
+ *
+ * Revision 1.102  2005/10/17 03:56:54  chessing
+ * Updates to the libxsupconfig library.  It no longer relies on other source from the main tree, so it can be used safely in other code with problems.
+ *
+ * Revision 1.101  2005/10/14 02:26:17  shaftoe
+ * - cleanup gcc 4 warnings
+ * - (re)add support for a pid in the form of /var/run/xsupplicant.<iface>.pid
+ *
+ * -- Eric Evans <eevans@sym-link.com>
+ *
+ * Revision 1.100  2005/09/19 21:45:46  chessing
+ * Fix for the madwifi driver when it connects to certain APs to do WEP.  (Currently the only known one with this problem is the Trapeze APs when they are running MSS 4.x+.)
+ *
+ * Revision 1.99  2005/09/14 02:50:44  chessing
+ * Major updates.  Auto association now works.  Started to rewrite the rtnetlink pieces using iwlib from the wireless tools to avoid compatibility issues.  As a result, getting the WPA and RSN IEs via the IWEVGENIE event no longer works for some reason, but the old style using IWEVCUSTOM events should still work like a champ.
+ *
+ * Revision 1.98  2005/09/08 16:27:01  chessing
+ * Some small updates to the new state machine code.  First attempt at an auto association mode.  (It mostly works. ;)
+ *
+ * Revision 1.97  2005/09/05 01:00:34  chessing
+ * Major overhaul to most of the state machines in Xsupplicant.  Also added additional error messages to the TLS functions to try to debug the one of the problems reported on the list.  Basic testing shows this new code to be more stable than previous code, but it needs more testing.
+ *
+ * Revision 1.96  2005/08/25 03:40:28  chessing
+ * Small fix to resolve an issue with some new code that would have broken wired connections.
+ *
+ * Revision 1.95  2005/08/25 03:34:05  chessing
+ * Removed a bunch of functions from config.c that could be handled better in other ways.
+ *
+ * Revision 1.94  2005/08/25 02:20:20  chessing
+ * Some cleanup in xsup_debug.c, added the ability to wait for an interface to come up if it is down when Xsupplicant is started.  Roughed in the ability to choose between having Xsupplicant auto associate you, or allow you to set the ssid manually.  The stale key timer can now be set in the config file.  The association timeout can be set in the config file, and will also be used once the functionality is in place to attempt to guess the key settings needed for association, as well as the functionality to auto associate.
+ *
  * Revision 1.93  2005/08/21 18:12:13  chessing
  * Added the ability to reload our configuration using a HUP.  Cleaned up a bunch of memory leaks in the config parse code.
  *
@@ -25,7 +79,6 @@
 /***
  *** This code implements 802.1X Authentication on a supplicant
  *** and supports multiple Authentication types.  
- *** See IEEE Draft P802.1X/D11, March 27, 2001 for more details
  ***/
 
 #include <stdlib.h>
@@ -35,9 +88,14 @@
 #include <strings.h>
 #include <string.h>
 #include <sys/wait.h>
+#include <sys/types.h>
+#include <pwd.h>
+#include <grp.h>
+#include <openssl/ssl.h>
+#include <errno.h>
 
 #include "profile.h"
-#include "config.h"
+#include "xsupconfig.h"
 #include "eap.h"
 #include "config_ssid.h"
 #include "statemachine.h"
@@ -46,19 +104,57 @@
 #include "xsup_err.h"
 #include "eapol.h"
 #include "snmp.h"
+#include "wireless_sm.h"
 #include "cardif/cardif.h"
 #include "cardif/core.h"
 #include "timer.h"
+#include "event_core.h"
 
 // This is needed to support the use of IW_ENCODE_TEMP keys for Cisco cards
 // based on the airo driver.
+#ifdef __LINUX__
 #include "cardif/linux/cardif_linux_wext.h"
+#endif
+
+#define PIDBASE "/var/run/xsupplicant"
 
 int wpa_keying = 0;
 struct interface_data *intiface;
 int dsd = 0;
 char block_wpa = 0;
 char *config_path = NULL;
+char pid_filename[64];
+
+/*********************************************
+ *
+ * Create a file and store our PID in it
+ *
+ *********************************************/
+int create_pidfile()
+{
+  FILE *pidfile;
+
+  pidfile = fopen(pid_filename, "w");
+  if (pidfile)
+    {
+      fprintf(pidfile, "%d", getpid());
+      fclose(pidfile);
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+/*********************************************
+ *
+ * Delete the pid file
+ *
+ *********************************************/
+int delete_pidfile()
+{
+  unlink(pid_filename);
+  return TRUE;
+}
 
 /*********************************************
  *
@@ -88,11 +184,6 @@ int xsup_driver_get_driver(char *drname)
 
   if (strcmp(drname, "none") == 0) return DRIVER_NONE;
   if (strcmp(drname, "wext") == 0) return DRIVER_WEXT;
-  if (strcmp(drname, "hostap") == 0) 
-    {
-      printf("Using hostap driver!\n");
-      return DRIVER_HOSTAP;
-    }
 
   if (strcmp(drname, "atmel") == 0) 
     {
@@ -100,57 +191,32 @@ int xsup_driver_get_driver(char *drname)
       return DRIVER_ATMEL;
     }
 
-  if (strcmp(drname, "ndiswrapper") == 0)
-    {
-      printf("Using ndiswrapper driver!\n");
-      return DRIVER_NDISWRAPPER;
-    }
-
-  if (strcmp(drname, "ipw") == 0)
-    {
-      printf("Using ipw driver!\n");
-      return DRIVER_IPW;
-    }
-
+#ifdef ENABLE_MADWIFI
   if (strcmp(drname, "madwifi") == 0)
     {
       printf("Using madwifi driver!\n");
       return DRIVER_MADWIFI;
     }
+#endif
 
+  printf("Unknown driver '%s' requested.\n", drname);
   return DRIVER_WEXT;
 }
 
 /*********************************************
  *
- * Initialize all of the pieces that will be needed for our supplicant.
- * We need to initialize in the correct order, in order to make sure
- * that other pieces of the initialization happen with the correct 
- * information available.
+ * Initialize the configuration data.
  *
- * THIS FUNCTION SHOULD NEVER BE CALLED OUTSIDE OF THIS FILE!
+ * NOTE : DO NOT debug_printf ANYTHING in this function, because the result
+ * of debug_printf is undefined before a valid configuration is loaded.
  *
  *********************************************/
-int global_init(int xdaemon, char *device, char *config, char *drivernam,
-		FDEPTH flags)
+void xsup_driver_init_config(char *config)
 {
-  struct interface_data *intcur;
-  char driveridx = 0;
-  char newssid[33];
   char *default_cfg = "/etc/xsupplicant.conf";
 
-  // Set our debug level.
-  intiface = NULL;
-  intcur = NULL;
-
-  timer_init();
-
-  driveridx = xsup_driver_get_driver(drivernam);
-  debug_setdaemon(xdaemon);
-
   if (config == NULL) 
     {
-      printf("Using default config path!\n");
       config = default_cfg;
     }
 
@@ -160,36 +226,29 @@ int global_init(int xdaemon, char *device, char *config, char *drivernam,
   switch(config_setup(config))
     {
     case XECONFIGFILEFAIL:
-      debug_printf(DEBUG_NORMAL, "\tPlease ensure that \"%s\" contains a valid"
-		   " xsupplicant configuration.\n", config);
+      printf("\tPlease ensure that \"%s\" contains a valid"
+	     " xsupplicant configuration.\n", config);
       exit(255);
       break;
     case XECONFIGPARSEFAIL:
-      debug_printf(DEBUG_NORMAL, "There was a problem with the config file.  "
-		   "We cannot continue.\n");
+      printf("There was a problem with the config file.  "
+	     "We cannot continue.\n");
+      exit(255);
+    case XECONFIGALREADYLOADED:
+      printf("config_setup() was called, but a "
+	     "configuration is already loaded.\n");
       exit(255);
       break;
     }
+}
 
-  logfile_setup(config_get_logfile());
-
-  // Initalize the event core.
-  event_core_setup();
-
-  if (device == NULL)
-    {
-      device = config_get_default_int();
-      if (device == NULL)
-	{
-	  debug_printf(DEBUG_NORMAL, "No valid interface defined!  Cannot"
-		       " continue!\n");
-	  return XENOTINT;
-	}
-
-      debug_printf(DEBUG_NORMAL, "Interface not specified on the command line"
-		   " we will use the default of '%s' instead!\n", device);
-    }
-
+/*********************************************
+ *
+ * Call what's needed to set up the interface to be used.
+ *
+ *********************************************/
+int xsup_driver_init_interface(char *device, char driveridx, FDEPTH flags)
+{
   intiface = (struct interface_data *)malloc(sizeof(struct interface_data));
   if (intiface == NULL) return XEMALLOC;
   memset(intiface, 0, sizeof(struct interface_data));
@@ -204,72 +263,131 @@ int global_init(int xdaemon, char *device, char *config, char *drivernam,
 
   // Set our flags.
   intiface->flags = flags;
-  
+
   // Establish a handler to the interface. Only need to manually turn on WPA if
   // force is set, otherwise should only turn it on if AP indicates support.
   if (cardif_init(intiface, driveridx, block_wpa) < 0) 
     return XENOTINT;
 
+  cardif_get_abilities(intiface);
+
+  if (intiface->enc_capa == 0)
+    {
+      debug_printf(DEBUG_INT, "Interface has no encryption capabilities, or "
+		   "unknown abilitites.\n");
+    } else {
+      debug_printf(DEBUG_NORMAL, "Card reported capabilitites :");
+      if (intiface->enc_capa & DOES_WEP40) debug_printf_nl(DEBUG_NORMAL, 
+							   " WEP40");
+      if (intiface->enc_capa & DOES_WEP104) debug_printf_nl(DEBUG_NORMAL,
+							    " WEP104");
+      if (intiface->enc_capa & DOES_WPA) debug_printf_nl(DEBUG_NORMAL, " WPA");
+      if (intiface->enc_capa & DOES_WPA2) debug_printf_nl(DEBUG_NORMAL, 
+							  " WPA2");
+      if (intiface->enc_capa & DOES_TKIP) debug_printf_nl(DEBUG_NORMAL, 
+							  " TKIP");
+      if (intiface->enc_capa & DOES_CCMP) debug_printf_nl(DEBUG_NORMAL, 
+							  " CCMP");
+      debug_printf_nl(DEBUG_INT, "\n");
+    }
+
   debug_printf(DEBUG_INT, "Interface initialized!\n");
 
-  if (cardif_get_if_state(intiface) == FALSE)
+  return XENONE;
+}
+
+int xsup_driver_init_logfile(int xdaemon)
+{
+ struct config_globals *globals;
+
+ debug_setdaemon(xdaemon);
+
+ // This line *MUST* always come after the call to xsup_driver_init_config.
+ // If config_get_globals() is called before it, then you will always get
+ // a NULL value, and probably return an error.
+ globals = config_get_globals();
+
+ if (!globals)
+   {
+     // Do *NOT* debug_printf this function, or you will have problems!
+     // debug_printf may not work until logfile_setup() is called!
+     printf("No valid configuration globals in %s!\n", __FUNCTION__);
+     return XEMALLOC;
+   }
+ 
+ return logfile_setup(globals->logfile);
+}
+
+/*********************************************
+ *
+ * Initialize all of the pieces that will be needed for our supplicant.
+ * We need to initialize in the correct order, in order to make sure
+ * that other pieces of the initialization happen with the correct 
+ * information available.
+ *
+ * THIS FUNCTION SHOULD NEVER BE CALLED OUTSIDE OF THIS FILE!
+ *
+ *********************************************/
+int global_init(char *device, char *drivernam, FDEPTH flags)
+{
+  struct interface_data *intcur;
+  char driveridx = 0, retval = XENONE;
+  struct config_globals *globals;
+
+  // Set our debug level.
+  intiface = NULL;
+  intcur = NULL;
+
+  driveridx = xsup_driver_get_driver(drivernam);
+
+  // This line *MUST* always come after the call to xsup_driver_init_config.
+  // If config_get_globals() is called before it, then you will always get
+  // a NULL value, and probably return an error.
+  globals = config_get_globals();
+
+  if (!globals)
     {
-      debug_printf(DEBUG_NORMAL, "Interface %s is down!  Bring it up, and try"
-		   " again!\n", intiface->intName);
-      exit(1);
+      // Do *NOT* debug_printf this function, or you will have problems!
+      // debug_printf may not work until logfile_setup() is called!
+      printf("No valid configuration globals in %s!\n", __FUNCTION__);
+      return XEMALLOC;
     }
 
-  // Check to see if this interface is wireless.
-  if (cardif_int_is_wireless(intiface->intName) == TRUE)
+  if (device == NULL)
     {
-      SET_FLAG(intiface->flags, IS_WIRELESS);
-
-      bzero(newssid, 33);
-      if (cardif_GetSSID(intiface, (char *)&newssid) != XENONE)
+      device = globals->default_int;
+      if (device == NULL)
 	{
-	  debug_printf(DEBUG_NORMAL, "Couldn't determine the current SSID!\n");
-	} else {
-	  debug_printf(DEBUG_NORMAL, "Your card is currently set for "
-		       "wireless network \"%s\".  Looking for a config.\n",
-		       newssid);
-
-	  intiface->cur_essid = strdup(newssid);
-
-	  if (config_build(intiface->cur_essid) != TRUE)
-	    {
-	      debug_printf(DEBUG_NORMAL, "Couldn't build a config for ESSID "
-			   "%s!\n", intiface->cur_essid);
-	      // XXX  Need to determine what to do when we fail.
-	    }
-	  config_set_globals(intiface);
-
-
-	  cardif_do_wireless_scan(intiface);
-	  
-	  cardif_reassociate(intiface, DISASSOC_UNSPECIFIED);
-
-	  if (intiface->flags & ZEROONROAM)
-	    {
-	      // DKS -- currently only want to do this if not associated; aren't picking up 
-	      // state properly even if we then associatecardif/
-	      cardif_roam(intiface);
-	    }
+	  debug_printf(DEBUG_NORMAL, "No valid interface defined!  Cannot"
+		       " continue!\n");
+	  return XENOTINT;
 	}
-    } else {
-      UNSET_FLAG(intiface->flags, IS_WIRELESS);
+
+      debug_printf(DEBUG_NORMAL, "Interface not specified on the command line"
+		   " we will use the default of '%s' instead!\n", device);
+    }
+
+  retval = xsup_driver_init_interface(device, driveridx, flags);
+  if (retval != XENONE) 
+    {
+      debug_printf(DEBUG_NORMAL, "Couldn't init interface!\n");
+      return retval;
     }
   
-  if (config_build(intiface->cur_essid) != TRUE)
+  if (config_build(intiface, intiface->cur_essid) != TRUE)
     {
+      // If we fail to build a config for this SSID, display a message so
+      // that the user has a chance of knowing that there is a problem, but
+      // continue to run so that we can authenticate if the user changes
+      // SSIDs to something we know about.
       debug_printf(DEBUG_NORMAL, "Couldn't build config for network %s!\n",
 		   intiface->cur_essid);
-      // XXX Need to determine what to do when we fail.
     }
   config_set_globals(intiface);
 
-  // were doing cardif_roam here if zeros_on_roam is true
-
-  cardif_check_associated(intiface);
+  // Initialize OpenSSL library
+  SSL_library_init();
+  SSL_load_error_strings();
   
   // Then, initialize EAPoL (which will initialize EAP methods).
 
@@ -286,6 +404,7 @@ void global_sigseg()
     fprintf(stderr, "[FATAL] SIGSEGV  (Segmentation Fault)!!!\n");
     xsup_ipc_cleanup(intiface);
     fflush(stderr); fflush(stdout);
+    delete_pidfile();
     exit(-1);
 }
 
@@ -324,9 +443,11 @@ void global_deinit()
 
   config_ssid_clear();
 
+  wireless_sm_deinit();
+
   timer_cleanup();
 
-  event_core_cleanup();
+  event_core_deinit();
 
   config_destroy();
 
@@ -335,6 +456,8 @@ void global_deinit()
   free(config_path);
   config_path = NULL;
 
+  delete_pidfile();
+
   exit(0);
 }
 
@@ -348,9 +471,55 @@ void global_config_reload()
   debug_printf(DEBUG_NORMAL, "Reloading configuration...\n");
   config_destroy();
   config_setup(config_path);
+  config_set_globals(intiface);
   signal(SIGHUP, global_config_reload);
 }
 
+/****************************************
+ *
+ * Wait for the interface we are looking for is up or inserted.
+ *
+ ****************************************/
+int xsup_driver_int_wait()
+{
+  char newssid[33];
+
+  /*
+  while (cardif_get_if_state(intiface) == FALSE)
+    {
+      sleep(1);        // Try again every second.
+    }
+  */
+  
+  // Check to see if this interface is wireless.
+  if (cardif_int_is_wireless(intiface->intName) == TRUE)
+    {
+      SET_FLAG(intiface->flags, IS_WIRELESS);
+
+      bzero(newssid, 33);
+      if (cardif_GetSSID(intiface, (char *)&newssid) != XENONE)
+	{
+	  debug_printf(DEBUG_NORMAL, "Couldn't determine the current SSID!\n");
+	} else {
+	  debug_printf(DEBUG_NORMAL, "Your card is currently set for "
+		       "wireless network \"%s\".  Looking for a config.\n",
+		       newssid);
+
+	  intiface->cur_essid = strdup(newssid);
+
+	  if (config_build(intiface, intiface->cur_essid) != TRUE)
+	    {
+	      debug_printf(DEBUG_NORMAL, "Couldn't build a config for ESSID "
+			   "%s!\n", intiface->cur_essid);
+	      // XXX  Need to determine what to do when we fail.
+	    }
+	  config_set_globals(intiface);
+	}
+    } else {
+      UNSET_FLAG(intiface->flags, IS_WIRELESS);
+    }
+  return XENONE;
+}
 
 /****************************************
  *
@@ -359,8 +528,13 @@ void global_config_reload()
  ****************************************/
 void usage(char *prog)
 {
+  debug_printf(DEBUG_NORMAL, "\n\nXsupplicant %s\n", VERSION);
+  debug_printf(DEBUG_NORMAL, "(c) Copyright 2002 - 2005 The Open1x Group\n");
+  debug_printf(DEBUG_NORMAL, "Dual licensed under the GPL and BSD licenses."
+	       "\n\n");
+  debug_printf(DEBUG_NORMAL, "This product makes use of the OpenSSL libraries"
+	       ". (http://www.openssl.org)\n\n");
   debug_printf(DEBUG_NORMAL, "Usage: %s "
-	       "[-W] "
 	       "[-w] "
 	       "[-c config file] "
 	       "[-i device] "
@@ -375,7 +549,6 @@ void usage(char *prog)
 	       "\n", prog);
 
   debug_printf(DEBUG_NORMAL, "\n\n");
-  debug_printf(DEBUG_NORMAL, "-W : Use wpa_supplicant for WPA keying\n");
   debug_printf(DEBUG_NORMAL, "-w : Don't enable WPA support.  (On drivers that"
 	       "support WPA/WPA2.)\n");
   debug_printf(DEBUG_NORMAL, "-c <path> : Use the config file <path> instead "
@@ -401,12 +574,7 @@ void usage(char *prog)
   debug_printf(DEBUG_NORMAL, " <driver name> can be any of : \n");
   debug_printf(DEBUG_NORMAL, "\twext : Interface uses Linux wireless "
 	       "extensions. (Default)\n");
-  debug_printf(DEBUG_NORMAL, "\thostap : Interface uses hostap driver.\n");
   debug_printf(DEBUG_NORMAL, "\tatmel : Interface uses atmel driver.\n");
-  debug_printf(DEBUG_NORMAL, "\tndiswrapper : Interface uses ndiswrapper "
-	       "driver.\n");
-  debug_printf(DEBUG_NORMAL, "\tipw     : Interface uses IPW2100/2200 "
-	       "driver.\n");
 
 #if ENABLE_MADWIFI
   debug_printf(DEBUG_NORMAL, "\tmadwifi : Interface uses madwifi driver.\n");
@@ -439,7 +607,7 @@ int main(int argc, char *argv[])
   char *theOpts = "c:i:d:D:Wwfztqas";
   char *config = NULL, *device = NULL;
   char *drivername = NULL;
-  int xdaemon = 1, new_debug;
+  int xdaemon = 1, new_debug, zeros=0;
   FDEPTH flags = 0;
 
   new_debug = 0;
@@ -494,7 +662,7 @@ int main(int argc, char *argv[])
 
 	case 'z':
 	  // Reset the encryption key to 0s on roam.
-	  SET_FLAG(flags, ZEROONROAM);
+	  zeros = 1;
 	  break;
 
 	case 'w':
@@ -542,7 +710,6 @@ int main(int argc, char *argv[])
       exit(0);
     }
 
-  // XXX Probably need to move this.
   if (xdaemon == 1)
     {
       printf("Starting XSupplicant v. %s\n",VERSION);
@@ -560,8 +727,27 @@ int main(int argc, char *argv[])
 
   snmp_init();
 
+  timer_init();
+
+  xsup_driver_init_config(config);
+
+  if (xsup_driver_init_logfile(xdaemon) != 0)
+    {
+      exit(255);
+    }
+
+  // Wait for our interface to become available before we continue.
+  cardif_wait_for_int(device);
+
+  sprintf(pid_filename, "%s.%s.pid", PIDBASE, device);
+
+  if(!(create_pidfile()))
+    {
+      debug_printf(DEBUG_NORMAL, "Failed to create the pid file!\n");
+    }
+
   // We have our options set, to initalize, then go in to our event loop.
-  if (global_init(xdaemon, device, config, drivername, flags) != 0)
+  if (global_init(device, drivername, flags) != 0)
     {
       printf("Couldn't initalize!!!!\n");
       exit(255);
@@ -580,7 +766,7 @@ int main(int argc, char *argv[])
       global_deinit();
       exit(255);
     }
-
+  
   // When we quit, cleanup.
   signal(SIGTERM, global_deinit);
   signal(SIGINT, global_deinit);
@@ -588,6 +774,21 @@ int main(int argc, char *argv[])
   signal(SIGSEGV, global_sigseg);
   signal(SIGHUP, global_config_reload);
 
+  event_core_init();
+
+  // XXX Remove this in favor of using the wireless state machine.
+  if (xsup_driver_int_wait() != XENONE)
+    {
+      printf("Error waiting for interface to become ready!\n");
+      global_deinit();
+      exit(255);
+    }
+
+  if (TEST_FLAG(intiface->flags, IS_WIRELESS))
+  {
+    wireless_sm_init(zeros, intiface);
+  }
+
   // Set up a handler, and start our timer.
   signal(SIGALRM, alrmclock);
   alarm(1);
@@ -596,7 +797,17 @@ int main(int argc, char *argv[])
     {
       xsup_ipc_process(intiface);
 
-      event_core(intiface);
+      if (TEST_FLAG(intiface->flags, IS_WIRELESS))
+	{
+	  // If we are wireless, we have an association state machine that
+	  // needs to be taking place.  It should handle calling the event
+	  // core, once it is in the state needed to make that call.
+	  wireless_sm_do_state(intiface);
+	} else {
+	  // If we are wired, we should just move right on to processing
+	  // events.
+	  event_core(intiface);
+	}
     }
 
   return XENONE;
