@@ -59,9 +59,15 @@
  *
  * Authors: npetroni@cs.umd.edu
  *
- * $Id: config-parser.c,v 1.6 2004/10/22 05:11:03 chessing Exp $
- * $Date: 2004/10/22 05:11:03 $
+ * $Id: config-parser.c,v 1.8 2005/11/16 02:18:39 chessing Exp $
+ * $Date: 2005/11/16 02:18:39 $
  * $Log: config-parser.c,v $
+ * Revision 1.8  2005/11/16 02:18:39  chessing
+ * Updates to the config writing library.  The config-parser tool has been modified to be able to parse a config, and rewrite it.
+ *
+ * Revision 1.7  2005/10/17 03:56:55  chessing
+ * Updates to the libxsupconfig library.  It no longer relies on other source from the main tree, so it can be used safely in other code with problems.
+ *
  * Revision 1.6  2004/10/22 05:11:03  chessing
  * Major changes to Xsupplicant. Xsupplicant now only works on a single interface, with a much more simplistic set of goals.  As a result, many configuration options have been removed.  For now, the options that have been removed will generate a warning letting users know that they need to update their config.  Following the next release, we can remove them completely.
  *
@@ -117,21 +123,26 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <signal.h>
+#include <errno.h>
+#include <string.h>
 
 #include "xsup_debug.h"
 #include "xsup_err.h"
 #include "profile.h"
-#include "config.h"
+#include "xsupconfig.h"
+#include "xsupconfwrite.h"
 
-#define CONFIG_PARSE_VERBOSE   0x00000001
-#define CONFIG_PARSE_HAVE_FILE 0x00000002
+#define CONFIG_PARSE_VERBOSE    0x00000001
+#define CONFIG_PARSE_HAVE_FILE  0x00000002
+#define CONFIG_PARSE_WRITE_FILE 0x00000004
 
 void usage(char *prog)
 {
-  debug_printf(DEBUG_NORMAL, "Usage: %s [-v] [-h] [-f file]"
+  debug_printf(DEBUG_NORMAL, "Usage: %s [-v] [-h] [-f file] [-o file]"
 	       "\n\t-f  file to parse (required)"
 	       "\n\t-h  print this message"
 	       "\n\t-v  verbose"
+	       "\n\t-o  rewrite the config to file"
 	       "\n", prog);
 }
 
@@ -146,9 +157,10 @@ extern struct config_data *config_info;
 int main(int argc, char *argv[])
 {
   int op;
-  char *theOpts = "f:vh";
-  char *config_fname = NULL;
+  char *theOpts = "f:vho:";
+  char *config_fname = NULL, *config_ofname = NULL;
   int flags = 0x00000000;
+  int retval = 0;
 
   // We should have at least one argument passed in!
   if (argc<2)
@@ -173,6 +185,10 @@ int main(int argc, char *argv[])
 	  usage(argv[0]);
 	  exit(0);
 	  break;
+	case 'o':
+	  config_ofname = optarg;
+	  flags |= CONFIG_PARSE_WRITE_FILE;
+	  break;
 	default:
 	  usage(argv[0]);
 	  exit(0);
@@ -188,10 +204,27 @@ int main(int argc, char *argv[])
   if (config_setup(config_fname) == XENONE) {
     if (flags & CONFIG_PARSE_VERBOSE)
       {
-	if (config_info == NULL) printf("config_info is NULL!\n");
 	dump_config_data(config_info);
       }
-    debug_printf(DEBUG_NORMAL, "Parsed successfully. Exiting.\n");
+    debug_printf(DEBUG_NORMAL, "Parsed successfully.\n");
+
+    if (flags & CONFIG_PARSE_WRITE_FILE)
+      {
+	retval = xsupconfwrite_write_config(config_info, config_ofname);
+	if (retval != XSUPCONFWRITE_ERRNONE)
+	  {
+	    debug_printf(DEBUG_NORMAL, "Error writing config file. (Error was"
+			 " : %s)\n", xsupconfwrite_strerr(retval));
+	    if (retval == XSUPCONFWRITE_CHECKERRNO)
+	      {
+		debug_printf(DEBUG_NORMAL, "Errno result was %d (%s)\n",
+			     errno, strerror(errno));
+	      }
+	  } else {
+	    debug_printf(DEBUG_NORMAL, "File written.\n");
+	  }
+      }
+    debug_printf(DEBUG_NORMAL, "Exiting.\n");
     config_destroy();
   }
   else {
